<?php
/**
 * i-doit
 *
 * DAO for OCS
 *
 * @package i-doit
 * @subpackage Components
 * @author Dennis Blümer <dbluemer@synetics.de>
 * @copyright synetics GmbH
 * @license http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */

class isys_component_dao_ocs extends isys_component_dao {

	/**
	 * Gets all OCS databases
	 * @return isys_component_dao_result
	 */
	public function getOCSDBs() {
		return $this->retrieve("SELECT * FROM isys_ocs_db");
	}

	/**
	 * Gets OCS database by ID
	 * @param null $p_id
	 * @return array|null
	 */
	public function getOCSDB($p_id = null) {
		$l_query = "SELECT * FROM isys_ocs_db";

		if ($p_id != null)
			$l_query .= " WHERE isys_ocs_db__id = ".$this->convert_sql_id($p_id);

		$l_res = $this->retrieve($l_query);

		if($l_res->num_rows() == 0)
			return null;

		return $l_res->get_row();
	}

	/**
	 * Get OCS database by schema
	 * @param $p_schema
	 * @return bool|isys_component_dao_result
	 */
	public function get_ocs_db_by_schema($p_schema){
		if(empty($p_schema)) return false;

		$l_query = 'SELECT * FROM isys_ocs_db WHERE isys_ocs_db__schema LIKE '.$this->convert_sql_text($p_schema);
		return $this->retrieve($l_query);
	}

	/**
	 * Get OCS database id by schema
	 * @param $p_schema
	 * @return bool
	 */
	public function get_ocs_db_id_by_schema($p_schema){
		if(empty($p_schema)) return false;

		$l_query = 'SELECT * FROM isys_ocs_db WHERE isys_ocs_db__schema LIKE '.$this->convert_sql_text($p_schema);
		$l_res = $this->retrieve($l_query);

		if($l_res){
			$l_data = $l_res->__to_array();
			return $l_data['isys_ocs_db__id'];
		} else{
			return false;
		}
	}

	/**
	 * Update or create a new ocs database
	 * @param $p_id
	 * @return bool|int
	 */
	public function saveOCSDB($p_id) {
		if ($p_id != null)
			$l_update = "UPDATE isys_ocs_db SET ";
		else
			$l_update = "INSERT INTO isys_ocs_db SET ";

		$l_update .= "isys_ocs_db__host = '".$_POST["C__MODULE__IMPORT__OCS_HOST"]."', ".
					 "isys_ocs_db__port = '".$_POST["C__MODULE__IMPORT__OCS_PORT"]."', ".
					 "isys_ocs_db__schema = '".$_POST["C__MODULE__IMPORT__OCS_SCHEMA"]."', ".
					 "isys_ocs_db__user = '".$_POST["C__MODULE__IMPORT__OCS_USER"]."', ".
					 "isys_ocs_db__pass = '".isys_helper_crypt::encrypt($_POST["C__MODULE__IMPORT__OCS_PASS"])."'";

		if ($p_id != null)
			$l_update .= " WHERE isys_ocs_db__id = '".$p_id."'";

		if ($this->update($l_update)) {
			$this->apply_update();

			if ($p_id != null) {
				return $p_id;
			}
			else
				return $this->get_last_insert_id();
		}
		else
			return false;
	}

	/**
	 * Remove existing ocs database from the db
	 * @param $p_id
	 * @return bool
	 */
	public function delete_ocsdb($p_id) {
		$l_update = "DELETE FROM isys_ocs_db WHERE isys_ocs_db__id = ".$this->convert_sql_id($p_id);

		$this->begin_update();

		try {
			$this->update($l_update);
			$this->apply_update();
		}
		catch (Exception $e) {
			$this->cancel_update();
			return false;
		}
		return true;
	}

	/**
	 * Get all Device IDs by hostname
	 * @param $p_arHostnames
	 * @return isys_component_dao_result
	 */
	public function getHardwareIDs($p_arHostnames) {
		$l_query = "SELECT ID FROM hardware WHERE FALSE ";
		foreach ($p_arHostnames as $l_hostname) {
			$l_query .= " OR NAME = ".$this->convert_sql_text($l_hostname);
		}
		return $this->retrieve($l_query);
	}

	/**
	 * Get all devices
	 * @return isys_component_dao_result
	 */
	public function getHardware() {
		return $this->retrieve("SELECT * FROM hardware LEFT JOIN accountinfo ON hardware.ID = accountinfo.HARDWARE_ID ORDER BY ID ASC");
	}

	/**
	 * Get Tag
	 * @param $p_hardwareID
	 * @return mixed
	 */
	public function getTag($p_hardwareID) {
		$l_res = $this->retrieve("SELECT TAG FROM accountinfo WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		$l_row = $l_res->get_row();
		return $l_row["TAG"];
	}

	/**
	 * Get device info by hardware ID
	 * @param $p_hardwareID
	 * @return array
	 */
	public function getHardwareItem($p_hardwareID) {
		$l_res = $this->retrieve("SELECT hardware.*, accountinfo.*, bios.*, GROUP_CONCAT(DISTINCT MACADDR) AS macaddr FROM hardware
			LEFT JOIN accountinfo ON accountinfo.HARDWARE_ID = hardware.ID
			LEFT JOIN bios ON bios.HARDWARE_ID = hardware.ID
			LEFT JOIN networks ON networks.HARDWARE_ID = hardware.ID
			WHERE hardware.ID = ".$this->convert_sql_id($p_hardwareID)."
			GROUP BY hardware.ID");
		return $l_res->get_row();
	}

	/**
	 * Get Memory by hardware ID
	 * @param      $p_hardwareID
	 * @param bool $p_snmp
	 * @return isys_component_dao_result
	 */
	public function getMemory($p_hardwareID, $p_snmp = false) {
		if($p_snmp)
		{
			return $this->retrieve("SELECT * FROM snmp_memories WHERE SNMP_ID = ".$this->convert_sql_id($p_hardwareID));
		}
		else
		{
			return $this->retrieve("SELECT * FROM memories WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get graphic adapter by hardware ID
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getGraphicsAdapter($p_hardwareID) {
		return $this->retrieve("SELECT * FROM videos WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
	}

	/**
	 * Get sound card info by hardware ID
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getSoundAdapter($p_hardwareID) {
		return $this->retrieve("SELECT * FROM sounds WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
	}

	/**
	 * Get OS info by hardware ID
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getBios($p_hardwareID, $p_snmp = false) {
		if($p_snmp)
		{
			$l_sql = 'SELECT SERIALNUMBER, NULL AS MANUFACTURER, NULL AS REFERENCE, NULL AS FIRMVERSION, NULL AS DESCRIPTION
				FROM snmp_printers WHERE SNMP_ID = ' . $this->convert_sql_id($p_hardwareID) . '
			 	UNION
			 	SELECT SERIALNUMBER, MANUFACTURER, REFERENCE, FIRMVERSION, DESCRIPTION FROM snmp_switchs WHERE SNMP_ID = ' . $this->convert_sql_id($p_hardwareID);

			return $this->retrieve($l_sql);
		}
		else{
			return $this->retrieve("SELECT * FROM bios WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get application assignments by hardware ID
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getSoftware($p_hardwareID) {
		$l_query = "SELECT * FROM hardware ".
				   "INNER JOIN softwares ON softwares.HARDWARE_ID = hardware.ID AND softwares.NAME != '' ".
				   "WHERE hardware.ID = ".$this->convert_sql_id($p_hardwareID)." ".
				   "AND hardware.OSNAME != softwares.NAME";

		return $this->retrieve($l_query);
	}

	/**
	 * Get all applications only from hardware
	 * @return isys_component_dao_result
	 */
	public function getAllSoftware()
	{
		$l_query = "SELECT DISTINCT(softwares.NAME) FROM softwares ".
			"LEFT JOIN hardware ON hardware.OSNAME = softwares.NAME ".
			"WHERE hardware.OSNAME IS NULL";

		return $this->retrieve($l_query);
	}

	/**
	 * Get all network info
	 * @param      $p_hardwareID
	 * @param bool $p_snmp
	 * @return isys_component_dao_result
	 */
	public function getNetworkAdapter($p_hardwareID, $p_snmp = false) {
		if($p_snmp)
		{
			return $this->retrieve("SELECT * FROM snmp_networks WHERE SNMP_ID = ".$this->convert_sql_id($p_hardwareID));
		}
		else
		{
			return $this->retrieve("SELECT * FROM networks WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get ports
	 * @param      $p_hardwareID
	 * @param bool $p_snmp
	 * @return isys_component_dao_result
	 */
	public function getPorts($p_hardwareID, $p_snmp = false) {
		if($p_snmp)
		{
			return $this->retrieve("SELECT * FROM snmp_ports WHERE SNMP_ID = ".$this->convert_sql_id($p_hardwareID));
		}
		else
		{
			return $this->retrieve("SELECT * FROM ports WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get storage info
	 * @param      $p_hardwareID
	 * @param bool $p_snmp
	 * @return isys_component_dao_result
	 */
	public function getStorage($p_hardwareID, $p_snmp = false) {
		if($p_snmp)
		{
			return $this->retrieve("SELECT * FROM snmp_storages WHERE SNMP_ID = ".$this->convert_sql_id($p_hardwareID));
		}
		else
		{
			return $this->retrieve("SELECT * FROM storages WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get drive info
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getDrives($p_hardwareID) {
		return $this->retrieve("SELECT * FROM drives WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
	}

	/**
	 * Get virtual machine info
	 * @param $p_hardwareID
	 * @return isys_component_dao_result
	 */
	public function getVirtualMachines($p_hardwareID) {
		return $this->retrieve("SELECT * FROM virtualmachines WHERE HARDWARE_ID = ".$this->convert_sql_id($p_hardwareID));
	}


	//SNMP Area

	/**
	 * Get CPU info
	 * @param      $p_hardwareID
	 * @param bool $p_snmp
	 * @return isys_component_dao_result
	 */
	public function getCPU($p_hardwareID, $p_snmp = false)
	{
		if($p_snmp)
		{
			return $this->retrieve("SELECT * FROM snmp_cpus WHERE SNMP_ID = " . $this->convert_sql_id($p_hardwareID));
		}
		else
		{
			return $this->retrieve("SELECT * FROM cpus WHERE HARDWARE_ID = " . $this->convert_sql_id($p_hardwareID));
		}
	}

	/**
	 * Get all SNMP devices
	 * @return isys_component_dao_result
	 */
	public function getHardwareSnmp()
	{
		return $this->retrieve("SELECT * FROM snmp LEFT JOIN snmp_accountinfo ON snmp.ID = snmp_accountinfo.SNMP_ID ORDER BY ID ASC");
	}

	/**
	 * Get SNMP device id by hostname
	 * @param $p_arHostnames
	 * @return isys_component_dao_result
	 */
	public function getHardwareSnmpIDs($p_arHostnames) {
		$l_query = "SELECT ID FROM snmp WHERE FALSE ";
		foreach ($p_arHostnames as $l_hostname) {
			$l_query .= " OR NAME = ".$this->convert_sql_text($l_hostname);
		}
		return $this->retrieve($l_query);

	}

	/**
	 * Get all info of the snmp device by ID
	 * @param $p_hwid
	 * @return array
	 */
	public function getHardwareItemBySNMP($p_hwid)
	{
        if ($this->does_snmp_exist())
        {

            $l_res = $this->retrieve(
                "SELECT snmp.*, snmp_accountinfo.*, CONCAT(GROUP_CONCAT(DISTINCT snmp_networks.MACADDR), ',', snmp.MACADDR) AS macaddr,
                (
                CASE
                    WHEN snmp_computers.SNMP_ID > 0 THEN 'server'
                    WHEN snmp_blades.SNMP_ID > 0 THEN 'blade'
                    WHEN snmp_printers.SNMP_ID > 0 THEN 'printer'
                    WHEN LOWER(snmp_switchinfos.TYPE) = 'switch' THEN 'switch'
                    WHEN snmp.TYPE = 'Network' THEN 'router'
                    WHEN LOWER(snmp.DESCRIPTION) REGEXP 'server' THEN 'server'
                    ELSE 'unknown'
                END
                ) AS OBJTYPE

                FROM snmp
                LEFT JOIN snmp_accountinfo ON snmp_accountinfo.SNMP_ID = snmp.ID
                LEFT JOIN snmp_networks ON snmp_networks.SNMP_ID = snmp.ID

                LEFT JOIN snmp_blades ON snmp_blades.SNMP_ID = snmp.ID
                LEFT JOIN snmp_computers ON snmp_computers.SNMP_ID = snmp.ID
                LEFT JOIN snmp_printers ON snmp_printers.SNMP_ID = snmp.ID
                LEFT JOIN snmp_switchinfos ON snmp_switchinfos.SNMP_ID = snmp.ID

                WHERE snmp.ID = " . $this->convert_sql_id($p_hwid) . "
                GROUP BY snmp.ID"
            );

            return $l_res->get_row();
        }
        else return array();
	}

	/**
	 * Method to check if the current OCS Version is able to discover SNMP devices
	 * @return bool
	 */
	public function does_snmp_exist()
	{
		return (bool) $this->retrieve('SHOW TABLES LIKE '.$this->convert_sql_text('snmp'))->num_rows();
	}
}