<?php
/**
 * i-doit - Updates
 * Migration of the "archive" right. Every user who already has "delete" rights will get it.
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Leonard Fischer <lfischer@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_absdir;

if (file_exists($g_absdir.DIRECTORY_SEPARATOR.'idoit_data.sql') && is_writeable($g_absdir.DIRECTORY_SEPARATOR.'idoit_data.sql'))     @unlink($g_absdir.DIRECTORY_SEPARATOR.'update_data.sql');
if (file_exists($g_absdir.DIRECTORY_SEPARATOR.'idoit_system.sql') && is_writeable($g_absdir.DIRECTORY_SEPARATOR.'idoit_system.sql')) @unlink($g_absdir.DIRECTORY_SEPARATOR.'idoit_system.sql');

$g_migration_identifier = "migration_archive_rights";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = '<span class="bold grey indent">Migration of "archive" right has already been executed.</span>';
}
else
{
	$g_migration_log[] = '<span class="bold grey indent">Starting the "archive" right migration.</span>';

	try
	{
		// Migration log instance.
		$l_mig_log = isys_log_migration::get_instance()
			->reset_log();

		$l_mig_log->notice('Starting migration of "archive" rights.');

		$l_mig_log->notice('Starting the authorization module migration. Everyone who has the "DELETE" right will now receive the "ARCHIVE" right.');

		/** @var  isys_auth_dao  $l_dao */
		$l_dao = isys_factory_cmdb_dao::get_instance('isys_auth_dao', $g_comp_database);

		// Retrieve all rights which inherit the "isys_auth::DELETE" right.
		$l_res = $l_dao->get_paths(null, C__MODULE__CMDB, ' AND isys_auth__type & ' . $l_dao->convert_sql_int(isys_auth::DELETE), null);

		while ($l_row = $l_res->get_row())
		{
			$l_sql = 'UPDATE isys_auth SET isys_auth__type = ' . $l_dao->convert_sql_int($l_row['isys_auth__type'] + isys_auth::ARCHIVE) . ' WHERE isys_auth__id = ' . $l_dao->convert_sql_id($l_row['isys_auth__id']) . ';';

			try
			{
				$l_mig_log->debug('Migration SQL - "' . $l_sql . '"');

				$l_dao->update($l_sql);
			}
			catch (Exception $e)
			{
				$l_mig_log->error('Error while migrating a right: ' . $e->getMessage());
			} // try
		} // while

		$l_dao->apply_update();

		$g_migration_log[] = '<span class="bold grey indent">Migration finished.</span>';

		$g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);

		// Set flag in db
		$this->migration_done($g_migration_identifier);
	}
	catch (Exception $e)
	{
		$g_migration_log[] = '<span class="bold red">' . $e->getMessage() . '</span>';
	} // try
} // if
