<?php
/**
 * i-doit - Updates
 *
 * This migration fixes all logbook entries which has been corrupted by the restore function from the logbook module
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_absdir;
$g_migration_identifier = "migration_logbook_archive_restore";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Migration of logbook entries which has been corrupted while restoring.</span>";
} else
{
	try
	{
		$l_dao = new isys_cmdb_dao($g_comp_database);

		// Migration log instance.
		$l_mig_log = isys_log_migration::get_instance()
			->reset_log()
			->add_migration_log('notice', 'Starting migration of logbook entries.');

		$l_sql = 'SELECT * FROM isys_catg_logb_list WHERE isys_catg_logb_list__isys_obj__id IS NULL;';
		$l_res = $l_dao->retrieve($l_sql);
		while($l_row = $l_res->get_row())
		{
			// Check if object still exists
			$l_obj_res = $l_dao->get_object_by_id($l_row['isys_catg_logb_list__isys_logbook__id']);
			if($l_obj_res->num_rows())
			{
				$l_update = 'UPDATE isys_catg_logb_list SET
				isys_catg_logb_list__isys_logbook__id = ' . $l_dao->convert_sql_id($l_row['isys_catg_logb_list__status']) . ',
				isys_catg_logb_list__status = ' . $l_dao->convert_sql_int($l_row['isys_catg_logb_list__property']) . ',
				isys_catg_logb_list__property = ' . $l_dao->convert_sql_int($l_row['isys_catg_logb_list__title']) . ',
				isys_catg_logb_list__title = ' . $l_dao->convert_sql_text($l_row['isys_catg_logb_list__description']) . ',
				isys_catg_logb_list__description = ' . $l_dao->convert_sql_text($l_row['isys_catg_logb_list__isys_obj__id']) . ',
				isys_catg_logb_list__isys_obj__id = ' . $l_dao->convert_sql_id($l_row['isys_catg_logb_list__isys_logbook__id']) . '
				WHERE isys_catg_logb_list__id = ' . $l_dao->convert_sql_id($l_row['isys_catg_logb_list__id']);
				$l_dao->update($l_update);
			}
		}
		$l_dao->apply_update();

		// Set flag in db
		$this->migration_done($g_migration_identifier);

		$g_migration_log[] = "<span class=\"bold indent\">Migration successfull.</span>";

		$g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);
	} catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
}