<?php
/**
 * i-doit - Updates
 *
 * Migration tenant settings
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_comp_database_system;
$g_migration_identifier = "migration_tenant_settings";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Migration of the tenant settings has already been executed.</span>";
} else
{
	try
	{
		$l_dao = new isys_cmdb_dao($g_comp_database_system);
		$l_set_migration_flag = true;

		// Migration log instance.
		$l_mig_log = isys_log_migration::get_instance()
			->reset_log()
			->add_migration_log('notice', 'Starting migration of tenant settings.');

		$l_sql_check = 'SELECT * FROM isys_settings WHERE isys_settings__isys_mandator__id > 0;';

		if(count($l_dao->retrieve($l_sql_check)) == 0)
		{
			$l_sql = 'SELECT isys_settings__value FROM isys_settings WHERE isys_settings__key = \'qrcode.config\'';
			$l_qr_code_config = $l_dao->retrieve($l_sql)->get_row_value('isys_settings__value');

			$l_sql = 'SELECT isys_settings__value FROM isys_settings WHERE isys_settings__key = \'cmdb.unique.hostname\'';
			$l_unique_hostname = $l_dao->retrieve($l_sql)->get_row_value('isys_settings__value');

			$l_sql = 'SELECT isys_settings__value FROM isys_settings WHERE isys_settings__key = \'cmdb.unique.ip-address\'';
			$l_unique_ip = $l_dao->retrieve($l_sql)->get_row_value('isys_settings__value');

			$l_sql = 'SELECT isys_settings__value FROM isys_settings WHERE isys_settings__key = \'cmdb.unique.layer-2-net\'';
			$l_unique_layer_2 = $l_dao->retrieve($l_sql)->get_row_value('isys_settings__value');

			$l_sql = 'SELECT isys_settings__value FROM isys_settings WHERE isys_settings__key = \'cmdb.unique.object-title\'';
			$l_unique_obj_title = $l_dao->retrieve($l_sql)->get_row_value('isys_settings__value');

			$l_delete = 'DELETE FROM isys_settings WHERE (isys_settings__key = \'cmdb.unique.object-title\' OR ' .
				'isys_settings__key = \'cmdb.unique.layer-2-net\' OR ' .
				'isys_settings__key = \'cmdb.unique.ip-address\' OR ' .
				'isys_settings__key = \'cmdb.unique.hostname\' OR ' .
                'isys_settings__key = \'security.passwort.minlength\' OR ' .
                'isys_settings__key = \'maxlenghts.location.objects\' OR ' .
                'isys_settings__key = \'maxlenghts.location.path\' OR ' .
				'isys_settings__key = \'qrcode.config\') AND isys_settings__isys_mandator__id = 0;';
			$l_dao->update($l_delete);

			$l_sql = 'SELECT isys_mandator__id FROM isys_mandator';
			$l_res = $l_dao->retrieve($l_sql);

			while($l_row = $l_res->get_row())
			{
				$l_mandator_id = $l_row['isys_mandator__id'];

				$l_insert = 'INSERT INTO isys_settings (isys_settings__key, isys_settings__value, isys_settings__isys_mandator__id) '.
					'VALUES '.
					'(\'barcode.type\', \'qr\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'qrcode.config\', '.$l_dao->convert_sql_text($l_qr_code_config).', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'gui.empty_values\', \'-\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'cmdb.unique.hostname\', '.$l_dao->convert_sql_text($l_unique_hostname).', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'cmdb.unique.ip-address\', '.$l_dao->convert_sql_text($l_unique_ip).', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'cmdb.unique.layer-2-net\', '.$l_dao->convert_sql_text($l_unique_layer_2).', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'cmdb.unique.object-title\', '.$l_dao->convert_sql_text($l_unique_obj_title).', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'maxlength.dialog_plus\', \'110\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'maxlength.object.lists\', \'55\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'maxlength.location.path\', \'40\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'maxlength.location.objects\', \'16\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'logbook.changes\', \'1\', '.$l_dao->convert_sql_id($l_mandator_id).'), ' .
					'(\'minlength.login.password\', \'4\', '.$l_dao->convert_sql_id($l_mandator_id).') ' .
					';';
				$l_dao->update($l_insert);
			} // while

			$l_dao->apply_update();
		}
		// Set flag in db
		if($l_set_migration_flag){
			$this->migration_done($g_migration_identifier);
		} // if

		$g_migration_log[] = "<span class=\"bold indent\">Migration successful.</span>";

		$g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);
	} catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
}