<?php

/**
 * i-doit
 *
 * CMDB DAO class for the WAN category.
 *
 * @package     i-doit
 * @subpackage  CMDB_Categories
 * @version     Leonard Fischer <lfischer@i-doit.com>
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_cmdb_dao_category_g_wan extends isys_cmdb_dao_category_global
{
	/**
	 * Category's name. Will be used for the identifier, constant, main table, and many more.
	 * @var  string
	 */
	protected $m_category = 'wan';

	/**
	 * Category entry is purgable.
	 * @var  boolean
	 */
	protected $m_is_purgable = true;


	/**
	 * Method for returning the properties.
	 *
	 * @return  array
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function properties ()
	{
		return array(
			'title' => array_replace_recursive(isys_cmdb_dao_category_pattern::text(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__TITLE',
					C__PROPERTY__INFO__DESCRIPTION => 'Title'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__title'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__TITLE'
				)
			)),
			'role' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog_plus(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__ROLE',
					C__PROPERTY__INFO__DESCRIPTION => 'Role'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__isys_wan_role__id',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_role',
						'isys_wan_role__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__ROLE',
					C__PROPERTY__UI__PARAMS => array(
						'p_strTable' => 'isys_wan_role',
						'p_strClass' => 'input-small'
					)
				)
			)),
			'type' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog_plus(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__TYPE',
					C__PROPERTY__INFO__DESCRIPTION => 'Type'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__isys_wan_type__id',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_type',
						'isys_wan_type__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__TYPE',
					C__PROPERTY__UI__PARAMS => array(
						'p_strTable' => 'isys_wan_type',
						'p_strClass' => 'input-small'
					)
				)
			)),
			'channels' => array_replace_recursive(isys_cmdb_dao_category_pattern::int(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CHANNELS',
					C__PROPERTY__INFO__DESCRIPTION => 'Channels'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__channels'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CHANNELS'
				)
			)),
			'call_numbers' => array_replace_recursive(isys_cmdb_dao_category_pattern::textarea(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CALL_NUMBERS',
					C__PROPERTY__INFO__DESCRIPTION => 'Call numbers'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__call_numbers'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CALL_NUMBERS'
				),
				C__PROPERTY__CHECK => array(
					C__PROPERTY__CHECK__SANITIZATION => null // This is necessary to keep linebreaks
				)
			)),
			'connection_location' => array_replace_recursive(isys_cmdb_dao_category_pattern::object_browser(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CONNECTION_LOCATION',
					C__PROPERTY__INFO__DESCRIPTION => 'Connection location'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__connection_location',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_obj',
						'isys_obj__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CONNECTION_LOCATION',
					C__PROPERTY__UI__PARAMS => array(
						'p_strPopupType' => 'browser_location'
					)
				),
				C__PROPERTY__PROVIDES => array(
					C__PROPERTY__PROVIDES__SEARCH => false
				)
			)),
			'capacity_up' => array_replace_recursive(isys_cmdb_dao_category_pattern::int(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CAPACITY_UP',
					C__PROPERTY__INFO__DESCRIPTION => 'Capacity up'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__capacity_up'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CAPACITY_UP',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-large'
					)
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__UNIT => 'capacity_up_unit'
				)
			)),
			'capacity_up_unit' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CAPACITY_UP_UNIT',
					C__PROPERTY__INFO__DESCRIPTION => 'Capacity up unit'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__capacity_up_unit',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_capacity_unit',
						'isys_wan_capacity_unit__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CAPACITY_UP_UNIT',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-small',
						'p_bInfoIconSpacer' => 0,
						'p_strTable' => 'isys_wan_capacity_unit',
						'p_bSort' => false
					)
				)
			)),
			'capacity_down' => array_replace_recursive(isys_cmdb_dao_category_pattern::int(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CAPACITY_DOWN',
					C__PROPERTY__INFO__DESCRIPTION => 'Capacity down'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__capacity_down'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CAPACITY_DOWN',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-large'
					)
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__UNIT => 'capacity_down_unit'
				)
			)),
			'capacity_down_unit' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CAPACITY_DOWN_UNIT',
					C__PROPERTY__INFO__DESCRIPTION => 'Capacity down unit'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__capacity_down_unit',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_capacity_unit',
						'isys_wan_capacity_unit__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CAPACITY_DOWN_UNIT',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-small',
						'p_bInfoIconSpacer' => 0,
						'p_strTable' => 'isys_wan_capacity_unit',
						'p_bSort' => false
					)
				)
			)),
			'max_capacity_up' => array_replace_recursive(isys_cmdb_dao_category_pattern::int(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__MAX_CAPACITY_UP',
					C__PROPERTY__INFO__DESCRIPTION => 'Max capacity up'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__max_capacity_up'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__MAX_CAPACITY_UP',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-large'
					)
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__UNIT => 'max_capacity_up_unit'
				)
			)),
			'max_capacity_up_unit' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__MAX_CAPACITY_UP_UNIT',
					C__PROPERTY__INFO__DESCRIPTION => 'Max capacity up unit'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__max_capacity_up_unit',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_capacity_unit',
						'isys_wan_capacity_unit__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__MAX_CAPACITY_UP_UNIT',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-small',
						'p_bInfoIconSpacer' => 0,
						'p_strTable' => 'isys_wan_capacity_unit',
						'p_bSort' => false
					)
				)
			)),
			'max_capacity_down' => array_replace_recursive(isys_cmdb_dao_category_pattern::int(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__MAX_CAPACITY_DOWN',
					C__PROPERTY__INFO__DESCRIPTION => 'Max capacity down'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__max_capacity_down'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__MAX_CAPACITY_DOWN',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-large'
					)
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__UNIT => 'max_capacity_down_unit'
				)
			)),
			'max_capacity_down_unit' => array_replace_recursive(isys_cmdb_dao_category_pattern::dialog(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__MAX_CAPACITY_DOWN_UNIT',
					C__PROPERTY__INFO__DESCRIPTION => 'Max capacity down unit'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__max_capacity_down_unit',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_wan_capacity_unit',
						'isys_wan_capacity_unit__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__MAX_CAPACITY_DOWN_UNIT',
					C__PROPERTY__UI__PARAMS => array(
						'p_strClass' => 'input-dual-small',
						'p_bInfoIconSpacer' => 0,
						'p_strTable' => 'isys_wan_capacity_unit',
						'p_bSort' => false
					)
				)
			)),
			'project_no' => array_replace_recursive(isys_cmdb_dao_category_pattern::text(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__PROJECT_NO',
					C__PROPERTY__INFO__DESCRIPTION => 'Project number'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__project_no'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__PROJECT_NO'
				)
			)),
			'vlan_id' => array_replace_recursive(isys_cmdb_dao_category_pattern::object_browser(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__VLAN_ID',
					C__PROPERTY__INFO__DESCRIPTION => 'VLAN-ID'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__vlan',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_obj',
						'isys_obj__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__VLAN_ID',
					C__PROPERTY__UI__PARAMS => array(
						isys_popup_browser_object_ng::C__CAT_FILTER => 'C__CATS__LAYER2_NET;C__CATS__LAYER2_NET_ASSIGNED_PORTS;C__CATS__LAYER2_NET_ASSIGNED_LOGICAL_PORTS',
					)
				),
				C__PROPERTY__PROVIDES => array(
					C__PROPERTY__PROVIDES__SEARCH => false
				)
			)),
			'shopping_cart_no' => array_replace_recursive(isys_cmdb_dao_category_pattern::text(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__SHOPPING_CART_NO',
					C__PROPERTY__INFO__DESCRIPTION => 'Shopping cart number'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__shopping_cart_no'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__SHOPPING_CART_NO'
				)
			)),
			'ticket_no' => array_replace_recursive(isys_cmdb_dao_category_pattern::text(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__TICKET_NO',
					C__PROPERTY__INFO__DESCRIPTION => 'Ticket number'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__ticket_no'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__TICKET_NO'
				)
			)),
			'customer_no' => array_replace_recursive(isys_cmdb_dao_category_pattern::text(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__CUSTOMER_NO',
					C__PROPERTY__INFO__DESCRIPTION => 'Customer number'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__customer_no'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__CUSTOMER_NO'
				)
			)),
			'router' => array_replace_recursive(isys_cmdb_dao_category_pattern::object_browser(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__ROUTER',
					C__PROPERTY__INFO__DESCRIPTION => 'Connected routers'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__id',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_catg_wan_list_2_router',
						'isys_catg_wan_list_2_router__isys_catg_wan_list__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__ROUTER',
					C__PROPERTY__UI__PARAMS => array(
						isys_popup_browser_object_ng::C__CAT_FILTER => 'C__CATS__ROUTER',
						isys_popup_browser_object_ng::C__MULTISELECTION => true
					)
				),
				C__PROPERTY__PROVIDES => array(
					C__PROPERTY__PROVIDES__SEARCH => false,
					C__PROPERTY__PROVIDES__LIST => false
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__CALLBACK => array() // @todo
				)
			)),
			'net' => array_replace_recursive(isys_cmdb_dao_category_pattern::object_browser(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CATG__WAN__NET',
					C__PROPERTY__INFO__DESCRIPTION => 'Connected nets'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__id',
					C__PROPERTY__DATA__REFERENCES => array(
						'isys_catg_wan_list_2_net',
						'isys_catg_wan_list_2_net__isys_catg_wan_list__id'
					)
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CATG__WAN__NET',
					C__PROPERTY__UI__PARAMS => array(
						isys_popup_browser_object_ng::C__CAT_FILTER => 'C__CATS__NET',
						isys_popup_browser_object_ng::C__MULTISELECTION => true
					)
				),
				C__PROPERTY__PROVIDES => array(
					C__PROPERTY__PROVIDES__SEARCH => false,
					C__PROPERTY__PROVIDES__LIST => false
				),
				C__PROPERTY__FORMAT => array(
					C__PROPERTY__FORMAT__CALLBACK => array() // @todo
				)
			)),
			'description' => array_replace_recursive(isys_cmdb_dao_category_pattern::commentary(), array(
				C__PROPERTY__INFO => array(
					C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__DESCRIPTION',
					C__PROPERTY__INFO__DESCRIPTION => 'Description'
				),
				C__PROPERTY__DATA => array(
					C__PROPERTY__DATA__FIELD => 'isys_catg_wan_list__description'
				),
				C__PROPERTY__UI => array(
					C__PROPERTY__UI__ID => 'C__CMDB__CAT__COMMENTARY_' . C__CMDB__CATEGORY__TYPE_GLOBAL . C__CATG__WAN
				)
			))
		);
	} // function


	/**
	 * Create a new entity.
	 *
	 * @param   array    $p_data
	 * @return  mixed
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function create_data ($p_data)
	{
		$p_data['capacity_up'] = isys_convert::speed_wan($p_data['capacity_up'], $p_data['capacity_up_unit']);
		$p_data['capacity_down'] = isys_convert::speed_wan($p_data['capacity_down'], $p_data['capacity_down_unit']);
		$p_data['max_capacity_up'] = isys_convert::speed_wan($p_data['max_capacity_up'], $p_data['max_capacity_up_unit']);
		$p_data['max_capacity_down'] = isys_convert::speed_wan($p_data['max_capacity_down'], $p_data['max_capacity_down_unit']);

		$l_result = parent::create_data($p_data);

		// If the result is not false, we connect the routers and nets.
		if ($l_result && is_numeric($l_result))
		{
			if (isset($p_data['router']) && isys_format_json::is_json_array($p_data['router']))
			{
				$this->set_connect_routers($l_result, array_filter(isys_format_json::decode($p_data['router'])));
			} // if

			if (isset($p_data['net']) && isys_format_json::is_json_array($p_data['net']))
			{
				$this->set_connect_nets($l_result, array_filter(isys_format_json::decode($p_data['net'])));
			} // if
		} // if

		return $l_result;
	} // function


	/**
	 * Updates existing entity.
	 *
	 * @param   integer  $p_category_data_id
	 * @param   array    $p_data
	 * @return  boolean
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function save_data ($p_category_data_id, $p_data)
	{
		$p_data['capacity_up'] = isys_convert::speed_wan($p_data['capacity_up'], $p_data['capacity_up_unit']);
		$p_data['capacity_down'] = isys_convert::speed_wan($p_data['capacity_down'], $p_data['capacity_down_unit']);
		$p_data['max_capacity_up'] = isys_convert::speed_wan($p_data['max_capacity_up'], $p_data['max_capacity_up_unit']);
		$p_data['max_capacity_down'] = isys_convert::speed_wan($p_data['max_capacity_down'], $p_data['max_capacity_down_unit']);

		$l_result = parent::save_data($p_category_data_id, $p_data);

		// If the result is not false, we connect the routers and nets.
		if ($l_result)
		{
			if (isset($p_data['router']) && isys_format_json::is_json_array($p_data['router']))
			{
				$this->set_connect_routers($p_category_data_id, array_filter(isys_format_json::decode($p_data['router'])));
			} // if

			if (isset($p_data['net']) && isys_format_json::is_json_array($p_data['net']))
			{
				$this->set_connect_nets($p_category_data_id, array_filter(isys_format_json::decode($p_data['net'])));
			} // if
		} // if

		return $l_result;
	} // function


	/**
	 * Method for retrieving all connected routers.
	 *
	 * @param   integer  $p_cat_entry_id
	 * @return  isys_component_dao_result
	 * @throws  Exception
	 * @throws  isys_exception_database
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function get_connected_routers ($p_cat_entry_id)
	{
		$l_sql = 'SELECT * FROM isys_catg_wan_list_2_router
			LEFT JOIN isys_obj ON isys_obj__id = isys_catg_wan_list_2_router__isys_obj__id
			LEFT JOIN isys_obj_type ON isys_obj_type__id = isys_obj__isys_obj_type__id
			WHERE isys_catg_wan_list_2_router__isys_catg_wan_list__id = ' . $this->convert_sql_id($p_cat_entry_id) . '
			AND isys_obj__status = ' . $this->convert_sql_int(C__RECORD_STATUS__NORMAL) . ';';

		return $this->retrieve($l_sql);
	} // function


	/**
	 * Method for setting certain routers to a given WAN-category entry.
	 *
	 * @param   integer  $p_cat_entry_id
	 * @param   mixed    $p_routers
	 * @return  boolean
	 * @throws  isys_exception_dao
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function set_connect_routers ($p_cat_entry_id, $p_routers)
	{
		if (! is_numeric($p_cat_entry_id) || !$p_cat_entry_id)
		{
			return true;
		} // if

		if (! is_array($p_routers))
		{
			$p_routers = array($p_routers);
		} // if

		$l_items = array();
		$l_sql = 'INSERT INTO isys_catg_wan_list_2_router (isys_catg_wan_list_2_router__isys_catg_wan_list__id, isys_catg_wan_list_2_router__isys_obj__id) VALUES ';

		foreach ($p_routers as $l_router)
		{
			$l_items[] = '(' . $this->convert_sql_id($p_cat_entry_id) . ', ' . $this->convert_sql_id($l_router) . ')';
		} // foreach

		if (count($l_items))
		{
			$this->update('DELETE FROM isys_catg_wan_list_2_router WHERE isys_catg_wan_list_2_router__isys_catg_wan_list__id = ' . $this->convert_sql_id($p_cat_entry_id) . ';');

			return $this->update($l_sql . implode(', ', $l_items) . ';') && $this->apply_update();
		} // if

		return true;
	} // function


	/**
	 * Method for retrieving all connected nets.
	 *
	 * @param   integer  $p_cat_entry_id
	 * @return  isys_component_dao_result
	 * @throws  Exception
	 * @throws  isys_exception_database
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function get_connected_nets ($p_cat_entry_id)
	{
		$l_sql = 'SELECT * FROM isys_catg_wan_list_2_net
			LEFT JOIN isys_obj ON isys_obj__id = isys_catg_wan_list_2_net__isys_obj__id
			LEFT JOIN isys_obj_type ON isys_obj_type__id = isys_obj__isys_obj_type__id
			WHERE isys_catg_wan_list_2_net__isys_catg_wan_list__id = ' . $this->convert_sql_id($p_cat_entry_id) . '
			AND isys_obj__status = ' . $this->convert_sql_int(C__RECORD_STATUS__NORMAL) . ';';

		return $this->retrieve($l_sql);
	} // function


	/**
	 * Method for setting certain nets to a given WAN-category entry.
	 *
	 * @param   integer  $p_cat_entry_id
	 * @param   mixed    $p_nets
	 * @return  boolean
	 * @throws  isys_exception_dao
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function set_connect_nets ($p_cat_entry_id, $p_nets)
	{
		if (! is_numeric($p_cat_entry_id) || !$p_cat_entry_id)
		{
			return true;
		} // if

		if (! is_array($p_nets))
		{
			$p_nets = array($p_nets);
		} // if

		$l_items = array();
		$l_sql = 'INSERT INTO isys_catg_wan_list_2_net (isys_catg_wan_list_2_net__isys_catg_wan_list__id, isys_catg_wan_list_2_net__isys_obj__id) VALUES ';

		foreach ($p_nets as $l_net)
		{
			$l_items[] = '(' . $this->convert_sql_id($p_cat_entry_id) . ', ' . $this->convert_sql_id($l_net) . ')';
		} // foreach

		if (count($l_items))
		{
			$this->update('DELETE FROM isys_catg_wan_list_2_net WHERE isys_catg_wan_list_2_net__isys_catg_wan_list__id = ' . $this->convert_sql_id($p_cat_entry_id) . ';');

			return $this->update($l_sql . implode(', ', $l_items) . ';') && $this->apply_update();
		} // if

		return true;
	} // function
} // class