<?php
define("C__PAGE__LOGBOOK_VIEW", 1);
define("C__PAGE__LOGBOOK_ARCHIVE", 2);
define("C__PAGE__LOGBOOK_RESTORE", 3);
define("C__PAGE__LOGBOOK_CONFIGURATION", 4);

/**
 * i-doit
 *
 * Logbook list.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Bluemer <dbluemer@i-doit.org>
 * @version     0.9.4
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_module_logbook extends isys_module implements isys_module_interface, isys_module_authable
{
	/**
	 * @var bool
	 */
	protected static $m_licenced = true;

	// Define, if this module shall be displayed in the named menus.
	const DISPLAY_IN_MAIN_MENU   = true;
	const DISPLAY_IN_SYSTEM_MENU = false;

    /**
     * Objects with their names and identifiers
     *
     * @see fetch_object_name_by_id()
     * @var array
     */
    protected $m_objects = array();


	/**
	 * This method builds the tree for the menu.
	 *
	 * @param   isys_component_tree  $p_tree
	 * @param   boolean              $p_system_module
	 * @param   integer              $p_parent
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 * @since   0.9.9-7
	 * @see     isys_module::build_tree()
	 */
	public function build_tree(isys_component_tree &$p_tree, $p_system_module = true, $p_parent = null)
	{
		global $g_comp_template_language_manager, $g_dirs;

		$l_parent    = -1;
		$l_submodule = '';

		if ($p_system_module) {
			$l_parent    = $p_tree->find_id_by_title('Modules');
			$l_submodule = '&' . C__GET__MODULE_SUB_ID . '=' . C__MODULE__LOGBOOK;
		} // if

		if (null !== $p_parent && is_int($p_parent))
		{
			$l_root = $p_parent;
		}
		else
		{
			$l_root = $p_tree->add_node(
				C__MODULE__LOGBOOK . '0',
				$l_parent,
				$g_comp_template_language_manager->get('LC__CMDB__CATG__LOGBOOK'));
		} // if

		$p_tree->add_node(
			C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_VIEW,
			$l_root,
			$g_comp_template_language_manager->get('LC__CMDB__LOGBOOK__LIST_CONTENT_TITLE'),
			'?' . C__GET__MODULE_ID . '=' . $_GET[C__GET__MODULE_ID] .
					$l_submodule .
					'&' . C__GET__TREE_NODE . '=' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_VIEW .
					'&' . C__GET__SETTINGS_PAGE . '=' . C__PAGE__LOGBOOK_VIEW .
					'&' . C__GET__MAIN_MENU__NAVIGATION_ID . '=6',
			'', 'images/icons/silk/book_open.png',
			0,
			'',
			'',
			isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_VIEW)
		);


		$p_tree->add_node(
			C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_CONFIGURATION,
			$l_root,
			$g_comp_template_language_manager->{"LC__MODULE__CMDB__LOGBOOK_CONFIGURATION"},
			'?' . C__GET__MODULE_ID . '=' . $_GET[C__GET__MODULE_ID] .
				$l_submodule .
				'&' . C__GET__TREE_NODE . '=' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_CONFIGURATION .
				'&' . C__GET__SETTINGS_PAGE . '=' . C__PAGE__LOGBOOK_CONFIGURATION .
				'&' . C__GET__MAIN_MENU__NAVIGATION_ID . '=6',
			null,
			$g_dirs["images"]."/icons/silk/book_edit.png",
			0,
			'',
			'',
			isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_CONFIGURATION)
		);

		$p_tree->add_node(
			C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_ARCHIVE,
			$l_root,
			$g_comp_template_language_manager->get('LC__NAVIGATION__NAVBAR__ARCHIVE'),
			'?' . C__GET__MODULE_ID . '=' . $_GET[C__GET__MODULE_ID] .
					$l_submodule .
					'&' . C__GET__TREE_NODE . '=' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_ARCHIVE .
					'&' . C__GET__SETTINGS_PAGE . '=' . C__PAGE__LOGBOOK_ARCHIVE .
					'&' . C__GET__MAIN_MENU__NAVIGATION_ID . '=6',
			null,
			$g_dirs["images"]."/icons/silk/door_in.png",
			0,
			'',
			'',
			isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_ARCHIVE)
		);

		$p_tree->add_node(
			C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_RESTORE,
			$l_root,
			$g_comp_template_language_manager->get('LC__UNIVERSAL__RESTORE'),
			'?' . C__GET__MODULE_ID . '=' . $_GET[C__GET__MODULE_ID] .
					$l_submodule .
					'&' . C__GET__TREE_NODE . '=' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_RESTORE .
					'&' . C__GET__SETTINGS_PAGE . '=' . C__PAGE__LOGBOOK_RESTORE .
					'&' . C__GET__MAIN_MENU__NAVIGATION_ID . '=6',
			null,
			$g_dirs["images"]."/icons/silk/arrow_refresh.png",
			0,
			'',
			'',
			isys_auth_logbook::instance()->is_allowed_to(isys_auth::EXECUTE, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_RESTORE)
		);
	} // function


	/**
	 * Method for handling the module request.
	 *
	 * @global  isys_component_template  $g_comp_template
	 *
	 */
	public function start()
	{
		if (isys_glob_get_param("ajax") && !isys_glob_get_param("call"))
		{
			$this->processAjaxRequest();
			die;
		} // if

        /* Set memory limit */
        if (($l_memlimit = isys_settings::get('system.memory-limit.search', '768M')))
        {
            ini_set('memory_limit', $l_memlimit);
        }

		global $g_comp_template, $index_includes;

		// Build up the tree
		if ($_GET[C__GET__MODULE_ID] != C__MODULE__SYSTEM)
		{
			$l_tree = isys_module_request::get_instance()->get_menutree();
			$this->build_tree($l_tree, false);
			$g_comp_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));
		} // if

		try{

			if (!isset($_GET[C__GET__SETTINGS_PAGE]))
			{
				if (isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_VIEW))
				{
					$_GET[C__GET__SETTINGS_PAGE] = C__PAGE__LOGBOOK_VIEW;
				}
				elseif (isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_CONFIGURATION))
				{
					$_GET[C__GET__SETTINGS_PAGE] = C__PAGE__LOGBOOK_CONFIGURATION;
				}
				elseif (isys_auth_logbook::instance()->is_allowed_to(isys_auth::VIEW, 'LOGBOOK/' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_ARCHIVE))
				{
					$_GET[C__GET__SETTINGS_PAGE] = C__PAGE__LOGBOOK_ARCHIVE;
				}
				elseif (isys_auth_logbook::instance()->is_allowed_to(isys_auth::EDIT, 'LOGBOOK/' . C__MODULE__LOGBOOK . C__PAGE__LOGBOOK_RESTORE))
				{
					$_GET[C__GET__SETTINGS_PAGE] = C__PAGE__LOGBOOK_RESTORE;
			} // if
			} // if

			// Handle request.
			switch ($_GET[C__GET__SETTINGS_PAGE])
			{
				case C__PAGE__LOGBOOK_ARCHIVE:
					isys_auth_logbook::instance()->check(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_ARCHIVE);
					$this->processArchive();
					break;
				case C__PAGE__LOGBOOK_RESTORE:
					isys_auth_logbook::instance()->check(isys_auth::EXECUTE, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_RESTORE);
					$this->processRestore();
					break;
				case C__PAGE__LOGBOOK_CONFIGURATION:
					isys_auth_logbook::instance()->check(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_CONFIGURATION);
					$this->processConfiguration();
					break;
				case C__PAGE__LOGBOOK_VIEW:
				default:
					isys_auth_logbook::instance()->check(isys_auth::VIEW, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_VIEW);
					$this->processView();
			} // switch

		} catch(isys_exception_general $e){
			throw $e;
		} catch(isys_exception_auth $e){
			$g_comp_template->assign("exception", $e->write_log());
			$index_includes['contentbottomcontent'] = "exception-auth.tpl";
		}
	} // function

	/**
	 * Process an AJAX request on the logbook
	 *
	 */
	private function processAjaxRequest()
	{
		switch (isys_glob_get_param("request")) {
			case "expandLogbookEntry":
				$this->expandLogbookEntry();
				break;

			case "executeRestore":
				$this->restoreLogbook();
				break;

			default:
				global $g_comp_template, $index_includes;
				$l_navbar = isys_component_template_navbar::getInstance();

                $l_settings_page = (int) $_GET[C__GET__SETTINGS_PAGE];
				switch ($l_settings_page) {
					case C__PAGE__LOGBOOK_ARCHIVE:
                        $this->processArchive();

                        $g_comp_template->assign("index_includes", $index_includes);
                        $g_comp_template->assign("navbar", $l_navbar->show_navbar());

                        $g_comp_template->display("content/main_groups.tpl");
						break;
					case C__PAGE__LOGBOOK_CONFIGURATION:
						$this->processConfiguration();
						$g_comp_template->assign("index_includes", $index_includes);
						$g_comp_template->assign("navbar", $l_navbar->show_navbar());

						$g_comp_template->display("content/main_groups.tpl");
						break;
					case C__PAGE__LOGBOOK_VIEW:
					default:
						if ($_POST[C__GET__NAVMODE] == C__NAVMODE__FORWARD || $_POST[C__GET__NAVMODE] == C__NAVMODE__BACK || isys_glob_get_param("navPageStart") != "") {
							$this->processListView();

							$g_comp_template->assign("index_includes", $index_includes);
							$g_comp_template->assign("navbar", $l_navbar->show_navbar());

							$g_comp_template->display("content/main_groups.tpl");
						}
						else {
							echo isys_glob_utf8_encode("Error processing AJAX request");
						}
						break;
				}

		}
	}


	/**
	 * Echo the description of a logbook entry given by its id.
	 * This is an AJAX request
	 *
	 */
	private function expandLogbookEntry()
	{
		if (! $_GET["log_id"])
		{
			echo "Error";
		}
		else
		{
			global $g_comp_database, $g_comp_template, $g_db_system;

			$l_cmdb_ui = new isys_cmdb_ui_category_g_logb($g_comp_template);

			if (isset($_GET["inArchive"]))
			{

				$l_daoLogbook = new isys_component_dao_logbook($g_comp_database);
				if (is_object($l_daoLogbook))
				{
					$l_settings = $l_daoLogbook->getArchivingSettings();
					try
					{
						if ($l_settings["dest"] == 0)
						{
							$l_db = $g_comp_database;
						}
						else
						{
							$l_db = isys_component_database::get_database(
								$g_db_system["type"],
								$l_settings["host"],
								$l_settings["port"],
								$l_settings["user"],
								$l_settings["pass"],
								$l_settings["db"]);
						} // if

						$l_daoLogbook = new isys_component_dao_archive($l_db);
					}
					catch (Exception $e)
					{
						die ($e->getMessage());
					} // try
				} // if
			}
			else
			{
				$l_daoLogbook = new isys_component_dao_logbook($g_comp_database);
			} // if

			try
			{
				$l_desc = $l_daoLogbook->get_changes_utf8($_GET["log_id"]);

				$l_changes_ar = $l_cmdb_ui->get_changes_as_array($l_desc);

				if (count($l_changes_ar) > 0)
				{
					echo $l_cmdb_ui->get_changes_as_html_table($l_changes_ar, "");
				}
				else
				{
					echo "No changes assigned to this logbook entry.";
				} // if
			}
			catch (Exception $e)
			{
				echo $e->getMessage();
			} // try
		} // if
	} // function

	/**
	 * Restore logbook data from the archive. Connection information are passed by POST
	 *
	 */
	private function restoreLogbook()
	{
		global $g_comp_database, $g_db_system;

		try {
			if ($_POST["archiveSource"] == "0")
			{
				// Use the local database
				$l_db = $g_comp_database;
			}
			else
			{
				// Use a remote database
				$l_db = isys_component_database::get_database($g_db_system["type"],
				                                              $_POST["archiveHost"],
				                                              $_POST["archivePort"],
				                                              $_POST["archiveUser"],
				                                              $_POST["archivePass"],
				                                              $_POST["archiveDB"]);
			}
		}
		catch (Exception $e) {
			echo isys_glob_utf8_encode("<p style=\"color:#ff0000;\">Could not connect to " . $_POST["archiveHost"] . "</p>");
			die;
		}

		$l_daoArchive = new isys_component_dao_archive($l_db);

		// Calculate the date to restore from. Number of days in the past passed by POST
		$l_arDate = getdate(time() - $_POST["restoreFrom"] * isys_convert::DAY);
		$l_date   = $l_arDate["year"] . "-" . $l_arDate["mon"] . "-" . $l_arDate["mday"] . " " . $l_arDate["hours"] . ":" . $l_arDate["minutes"] . ":" . $l_arDate["seconds"];

		try {
			if (!$l_db->is_table_existent("isys_archive_logbook"))
				throw new Exception("No archive data found");

			// start restoring
			$l_daoArchive->restore(new isys_component_dao($g_comp_database), $l_date);
			echo isys_glob_utf8_encode("Restore successful");
		}
		catch (Exception $e) {
			echo isys_glob_utf8_encode("<p>" . $e->getMessage() . "</p>");
			die;
		}
	}

	private function processView()
	{
		if (isys_glob_get_param("id"))
		{
			$this->processDetailView();
		}
		else
		{
			try
			{
				$this->processListView();
			}
			catch (Exception $e)
			{
				;
			} // try
		} // if
	} // function

	private function processConfiguration()
	{
		global $index_includes, $g_comp_template, $g_comp_session, $g_comp_database;

		$l_comp_logb  = isys_factory::get_instance('isys_component_dao_logbook', $g_comp_database);
		$l_user_id    = $g_comp_session->get_user_id();
		$l_config_res = $l_comp_logb->get_logbook_config($l_user_id);
		$l_config     = null;
		if ($l_config_res->num_rows() > 0)
		{
			$l_config = $l_config_res->__to_array();
		} else
		{
			$l_comp_logb->create_logbook_config();
			$l_config['isys_logbook_configuration__type'] = 0;
		}

		switch ($_POST[C__GET__NAVMODE])
		{
			case C__NAVMODE__EDIT:
				isys_component_template_navbar::getInstance()->set_active(true, C__NAVBAR_BUTTON__SAVE)
					->set_active(true, C__NAVBAR_BUTTON__CANCEL);
				$index_includes['contentbottomcontent'] = "modules/cmdb/logbook.tpl";
				break;
			case C__NAVMODE__SAVE:
				if(!empty($l_config))
				{
					$l_comp_logb->save_logbook_config(
						$l_config['isys_logbook_configuration__id'],
						$_POST['C__MODULE__CMDB__LOGBOOK_CONFIG__TYPE'],
						$_POST['C__MODULE__CMDB__LOGBOOK_CONFIG__PLACEHOLDER']
					);
				} else{
					$l_comp_logb->create_logbook_config(
						$_POST['C__MODULE__CMDB__LOGBOOK_CONFIG__TYPE'],
						$_POST['C__MODULE__CMDB__LOGBOOK_CONFIG__PLACEHOLDER']
					);
				}
				isys_settings::set('logbook.changes.multivalue-threshold', $_POST['C__MODULE__CMDB__LOGBOOK_CONFIGURATION__MULTIVALUE_THRESHOLD']);

				$l_config = $l_comp_logb->get_logbook_config($l_user_id)->get_row();
			default:
				isys_component_template_navbar::getInstance()->set_active(isys_auth_logbook::instance()->is_allowed_to(isys_auth::EDIT, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_CONFIGURATION), C__NAVBAR_BUTTON__EDIT)
									   ->set_visible(true, C__NAVBAR_BUTTON__EDIT);
				$index_includes['contentbottomcontent'] = "modules/cmdb/logbook.tpl";
				break;
		}

		$g_comp_template
			->assign('multivalue_threshold', isys_settings::get('logbook.changes.multivalue-threshold', 25))
			->assign('content_title', _L('LC__MODULE__CMDB__LOGBOOK_CONFIGURATION'))
			->assign('default_checked', false)
            ->assign('advanced_checked', false)
            ->assign('disabled_on', (($_POST[C__GET__NAVMODE] == C__NAVMODE__EDIT)? false: true))
            ->assign('logbook_type', $l_config['isys_logbook_configuration__type']);

		if(!empty($l_config))
		{
			$g_comp_template->assign('default_checked', (($l_config['isys_logbook_configuration__type'] == 0)? true: false))
							->assign('advanced_checked', (($l_config['isys_logbook_configuration__type'] == 1)? true: false))
							->assign('placeholder_string', (!empty($l_config['isys_logbook_configuration__placeholder_string'])? $l_config['isys_logbook_configuration__placeholder_string']: _L('LC__NAVIGATION__BREADCRUMB__NO_TITLE')));
		}

		isys_component_template_navbar::getInstance()
			->set_active(false, C__NAVBAR_BUTTON__NEW)
			->set_active(false, C__NAVBAR_BUTTON__PURGE);
	}

	/**
	 * Process the archiving settings page.
	 *
	 */
	private function processArchive()
	{
		global $index_includes;
		global $g_comp_template;
		global $g_comp_template_language_manager;
		global $g_comp_database;

		$l_navbar = isys_component_template_navbar::getInstance();

		$l_rules = array();

		$l_daoLogbook = new isys_component_dao_logbook($g_comp_database);

		$l_rules["archiveDest"]["p_arData"] = serialize(array(
		                                                     "0" => $g_comp_template_language_manager->get("LC__UNIVERSAL__DATABASE_LOCAL"),
		                                                     "1" => $g_comp_template_language_manager->get("LC__UNIVERSAL__DATABASE_REMOTE")
		                                                ));

		if ($_POST["navMode"] == C__NAVMODE__SAVE) {
			try {
				$l_daoLogbook->archiveAccessible();
				$l_daoLogbook->saveArchivingSettings();
			}
			catch (Exception $e) {
				switch ($e->getMessage()) {
					case "Error 1":
						$l_rules["archiveInterval"]["p_strInfoIconError"] = $g_comp_template_language_manager->{"LC___UNIVERSAL__ERROR_INFO_ICON"};
						break;

					default:
						// echo $e->getMessage();
						$l_rules["archiveError"]["p_strValue"] = "Connection failed";
				}

				$_POST["navMode"] = C__NAVMODE__EDIT;
			}

			$l_rules["archiveInterval"]["p_strValue"]  = $_POST["archiveInterval"];
			$l_rules["archiveHost"]["p_strValue"]      = $_POST["archiveHost"];
			$l_rules["archivePort"]["p_strValue"]      = $_POST["archivePort"];
			$l_rules["archiveDB"]["p_strValue"]        = $_POST["archiveDB"];
			$l_rules["archiveUser"]["p_strValue"]      = $_POST["archiveUser"];
			$l_rules["archivePass"]["p_strValue"]      = $_POST["archivePass"];
			$l_rules["archiveDest"]["p_strSelectedID"] = $_POST["archiveDest"];
			$g_comp_template->assign("archiveDest", $_POST["archiveDest"]);
		}
		else {
			$l_ad                                      = $l_daoLogbook->getArchivingSettings();
			$l_rules["archiveInterval"]["p_strValue"]  = $l_ad["interval"];
			$l_rules["archiveHost"]["p_strValue"]      = $l_ad["host"];
			$l_rules["archivePort"]["p_strValue"]      = $l_ad["port"];
			$l_rules["archiveDB"]["p_strValue"]        = $l_ad["db"];
			$l_rules["archiveUser"]["p_strValue"]      = $l_ad["user"];
			$l_rules["archivePass"]["p_strValue"]      = $l_ad["pass"];
			$l_rules["archiveDest"]["p_strSelectedID"] = $l_ad["dest"];
			$g_comp_template->assign("archiveDest", $l_ad["dest"]);
		}

        if (isset($_POST['navMode']) && ($_POST["navMode"] == C__NAVMODE__EDIT || $_POST["navMode"] == C__NAVMODE__NEW)) {
            $l_navbar->set_active(true, C__NAVBAR_BUTTON__SAVE)
					 ->set_active(true, C__NAVBAR_BUTTON__CANCEL);
			//$l_navbar->set_js_onclick("remove_action_parameter('ajax'); document.isys_form.navMode.value='" . C__NAVMODE__SAVE . "'; form_submit();", C__NAVBAR_BUTTON__SAVE);
            //$l_navbar->set_js_onclick("remove_action_parameter('ajax'); document.isys_form.navMode.value='" . C__NAVMODE__CANCEL . "'; form_submit();", C__NAVBAR_BUTTON__CANCEL);
        } else {
            $l_navbar->set_active(false, C__NAVBAR_BUTTON__SAVE)
            		 ->set_active(false, C__NAVBAR_BUTTON__CANCEL);
        }

		$g_comp_template->assign("content_title", $g_comp_template_language_manager->get("LC__NAVIGATION__NAVBAR__ARCHIVE"));

		$l_navbar->set_active(isys_auth_logbook::instance()->is_allowed_to(isys_auth::EDIT, 'LOGBOOK/'.C__MODULE__LOGBOOK.C__PAGE__LOGBOOK_ARCHIVE), C__NAVBAR_BUTTON__EDIT)
				 ->set_visible(true, C__NAVBAR_BUTTON__EDIT);
		$g_comp_template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
		$index_includes["contentbottomcontent"] = "content/bottom/content/module__logbook__archive.tpl";
	}


	/**
	 * Process the restore page
	 */
	private function processRestore ()
	{
		global $g_comp_template, $g_comp_database, $index_includes;

		$l_rules = array();

		$l_rules["archiveSource"]["p_arData"] = serialize(array(
			"0" => _L("LC__UNIVERSAL__DATABASE_LOCAL"),
			"1" => _L("LC__UNIVERSAL__DATABASE_REMOTE")
		));

		$l_daoLogbook = new isys_component_dao_logbook($g_comp_database);
		$l_ad = $l_daoLogbook->getArchivingSettings();

		$l_rules["restoreFrom"]["p_strValue"] = $l_ad["interval"];
		$l_rules["archiveHost"]["p_strValue"] = $l_ad["host"];
		$l_rules["archivePort"]["p_strValue"] = $l_ad["port"];
		$l_rules["archiveDB"]["p_strValue"] = $l_ad["db"];
		$l_rules["archiveUser"]["p_strValue"] = $l_ad["user"];
		$l_rules["archivePass"]["p_strValue"] = $l_ad["pass"];
		$l_rules["archiveSource"]["p_strSelectedID"] = $l_ad["dest"];

		// Removed: isys_rs_system
		$l_rules["buttonRestore"]["p_bDisabled"] = 0;

		$g_comp_template
			->assign("btnLabelExecute", _L("LC__UNIVERSAL__EXECUTE"))
			->assign("archiveDest", $l_ad["dest"])
			->assign("content_title", _L("LC__UNIVERSAL__RESTORE"))
			->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1")
			->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);

		$index_includes["contentbottomcontent"] = "content/bottom/content/module__logbook__restore.tpl";
	} // function


	/**
	 * Assign the logbook detail view to the templates
	 */
	protected function processDetailView()
	{
		global $index_includes;
		global $g_comp_template;
		global $g_comp_database;
		global $g_comp_template_language_manager;

		$l_ui_logbook = new isys_cmdb_ui_category_g_logb($g_comp_template);

		// if id is connected to an object redirect browser to logbook
		// category of the object
		$l_objDAOLogbook = new isys_component_dao_logbook($g_comp_database);

		$l_lbID = isys_glob_get_param("id");

		$l_lbRes = $l_objDAOLogbook->get_result_by_logbook_id($l_lbID);
		$l_lbRow = $l_lbRes->get_row();

		$l_nObjID = $l_objDAOLogbook->get_object_id_by_logbook_id($l_lbID);

		/**
		 * @todo set $l_nObjID to ISYS_NULL if object is deleted!
		 */
		if ($l_nObjID) {
			$l_objCMDBDAO = new isys_cmdb_dao($g_comp_database);
			$l_nObjTypeID = $l_objCMDBDAO->get_objTypeID($l_nObjID);
			$l_nTreeMode  = C__CMDB__VIEW__TREE_OBJECT;
			$l_nViewMode  = C__CMDB__VIEW__CATEGORY_GLOBAL;
			$l_CatgID     = C__CATG__LOGBOOK;
			$l_nCatID     = $l_lbRow["isys_catg_logb_list__id"];

			$l_strURL = "index.php" .
					"?moduleID=" . C__MODULE__CMDB .
					"&objTypeID=$l_nObjTypeID" .
					"&viewMode=$l_nViewMode" .
					"&tvMode=$l_nTreeMode" .
					"&objID=$l_nObjID" .
					"&catgID=$l_CatgID" .
					"&cateID=$l_nCatID";

			header("Location: $l_strURL"); //redirect browser
			die;
		}

		$l_objDAOUser = new isys_component_dao_user($g_comp_database);

		// title
		$l_mod_event_manager = isys_event_manager::getInstance();

		$l_strTitle = $l_mod_event_manager->translateEvent($l_lbRow["isys_logbook__event_static"],
		                                                   $l_lbRow["isys_logbook__obj_name_static"],
		                                                   $l_lbRow["isys_logbook__category_static"],
		                                                   $l_lbRow["isys_logbook__obj_type_static"].
                                                           $l_lbRow["isys_logbook__entry_identifier_static"],
														   $l_lbRow["isys_logbook__changecount"]);


		/* Assign and retrieve changes */
		$l_changes_ar = $l_ui_logbook->get_changes_as_array($l_lbRow["isys_logbook__changes"]);

		$l_rules["C__CMDB__LOGBOOK__CHANGED_FIELDS"]["p_strValue"] = count($l_changes_ar);

		if (($l_changes = $l_ui_logbook->get_changes_as_html_table($l_changes_ar))) {
			$g_comp_template->assign("changes", $l_changes);
		}

		// Make rules
		$l_rules["C__CMDB__LOGBOOK__TITLE"]["p_strValue"]       = $l_strTitle;
		$l_rules["C__CMDB__LOGBOOK__DESCRIPTION"]["p_strValue"] = stripslashes($l_lbRow["isys_logbook__description"]);
		$l_rules["C__CMDB__LOGBOOK__DATE"]["p_strValue"]        = $l_lbRow["isys_logbook__date"];
		$l_rules["C__CMDB__LOGBOOK__LEVEL"]["p_strValue"]       = $g_comp_template_language_manager->get($l_lbRow["isys_logbook_level__title"]);
		$l_rules["C__CMDB__LOGBOOK__COMMENT"]["p_strValue"]     = $l_lbRow["isys_logbook__comment"];
        $l_rules["C__CMDB__LOGBOOK__REASON"]["p_strSelectedID"]     = $l_lbRow["isys_logbook__isys_logbook_reason__id"];

		//is there a name?
		if ($l_lbRow["isys_logbook__isys_obj__id"] > 0) {
			$l_strUsertitle = $l_objDAOUser->get_user_title($l_lbRow["isys_logbook__isys_obj__id"]);
		} else {
			$l_strUsertitle = $l_lbRow["isys_logbook__user_name_static"];
		}

		if (empty($l_strUsertitle)) {
			$l_strUsertitle = isys_tenantsettings::get('gui.empty_value', '-');
		}

		$l_rules["C__CMDB__LOGBOOK__USER"]["p_strValue"] = $l_strUsertitle;

		// Apply rules
		$g_comp_template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);


		$index_includes["contenttop"] = '';
		$index_includes["contentbottomcontent"] = "content/bottom/content/catg__logbook.tpl";
	}


	/**
	 * Assign the logbook list view to the templates
	 */
	protected function processListView()
	{
		global $g_comp_template;
		global $g_comp_database;
		global $index_includes;
		global $g_db_system;

        try
        {
            $l_listdao = new isys_component_dao_logbook($g_comp_database);

            // Removed "new" and "save" state, according to #4780
            $g_comp_template->smarty_tom_add_rule("tom.content.top.filter.p_strValue=" . isys_glob_get_param("filter"));

            /**
             * Get all logbook entries
             */
            $l_navPageCount = $l_listdao->count();

            /* Hotfix for too many entries in page select dropdown */
            global $g_page_limit;
            if ($g_page_limit < 250 && $l_navPageCount > 250000)
            {
                $g_page_limit = 250;
            }

            if (isys_glob_get_param("filter_archive") == "1")
            {
                try
                {
                    $l_settings = $l_listdao->getArchivingSettings();

                    if ($l_settings["dest"] == 0)
                    {
                        $l_db = $g_comp_database;
                    }
                    else
                    {
                        $l_db = isys_component_database::get_database($g_db_system["type"], $l_settings["host"], $l_settings["port"], $l_settings["user"], $l_settings["pass"], $l_settings["db"]);
                    } // if

                    $l_daoArchive = new isys_component_dao_archive($l_db);
                    $l_listres    = $l_daoArchive->get_result();

                    $l_objList = new isys_component_list_logbook(ISYS_NULL, $l_listres, $l_daoArchive);
                }
                catch (Exception $e)
                {
                    $g_comp_template
                        ->assign("content_title", _L("LC__CMDB__LOGBOOK__LIST_CONTENT_TITLE"))
                        ->assign("bNavbarFilter", "1")
                        ->assign("archiveBrowser", "1")
                        ->assign("objectTableList", "<h3>" . $e->getMessage() . "</h3>")
                        ->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1");

                    $this->setupFilter($l_listdao);
                    $index_includes['contentbottomcontent'] = "content/bottom/content/module__logbook__list.tpl";
                    $index_includes["navbar"]               = "content/navbar/logbook.tpl";

                    throw new Exception($e->getMessage());
                }
            }
            else
            {
                $l_objList = new isys_component_list_logbook(NULL, NULL, $l_listdao);
            }

            $l_strRowLink = "document.location.href='?moduleID=" . C__MODULE__LOGBOOK . "&id=[{isys_logbook__id}]';";

            // array with table header titles
            $l_arTableHeader = array(
                "+"                              => "",
                "isys_logbook__title"            => _L('LC__CMDB__LOGBOOK__TITLE'),
                "isys_logbook__user_name_static" => _L('LC__CMDB__LOGBOOK__SOURCE__USER'),
                "isys_logbook__obj_name_static"  => _L('LC__CMDB__LOGBOOK__SOURCE__OBJECT'),
                "isys_logbook__date"             => _L('LC__CMDB__LOGBOOK__DATE'),
                "isys_logbook_level__title"      => _L('LC__CMDB__LOGBOOK__LEVEL'),
                "isys_logbook_source__title"     => _L('LC__CMDB__LOGBOOK__SOURCE')
            );

            $l_objList->config($l_arTableHeader, $l_strRowLink);
            isys_component_template_navbar::getInstance()->set_nav_page_count($l_navPageCount);

            $g_comp_template
                ->assign("content_title", _L("LC__CMDB__LOGBOOK__LIST_CONTENT_TITLE"))
                ->assign("LogbookList", $l_objList->getTempTableHtml($_POST))
                ->assign("bNavbarFilter", "1")
                ->assign("archiveBrowser", "1")
                ->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1");

            $this->setupFilter($l_listdao);

            $index_includes['contentbottomcontent'] = "content/bottom/content/module__logbook__list.tpl";
            $index_includes["navbar"]               = "content/navbar/logbook.tpl";
        }
        catch (Exception $e)
        {
            global $g_error;
            $g_error = $e->getMessage();
        }
	} // function

	/**
	 * Set up the filter for the logbook
	 */
	private function setupFilter ($p_daoLogbook)
	{
		global $g_comp_template;

		$l_alertFilter = $p_daoLogbook->getAlertlevels();

		$l_sourceFilter = $p_daoLogbook->getSources();

		$l_typeFilter = array(
			"0" => _L("LC__CMDB__CATG__SYSTEM"),
			"1" => _L("LC__NAVIGATION__MENUTREE__BUTTON_OBJECT_VIEW")
		);

		$l_rules = array(
			'filter_source' => array(
				'p_arData' => serialize($l_sourceFilter),
				'p_strClass' => 'input-mini'
			),
			'filter_alert' => array(
				'p_arData' => serialize($l_alertFilter),
				'p_strClass' => 'input-mini'
			),
			'filter_type' => array(
				'p_arData' => serialize($l_typeFilter),
				'p_strClass' => 'input-mini'
			),
			'filter_archive' => array(
				'p_arData' => serialize(array(
					"0" => _L("LC__RECORD_STATUS__NORMAL"),
					"1" => _L("LC__UNIVERSAL__ARCHIVE")
				)),
				'p_strClass' => 'input-mini'
			)
		);

		if (isset($_POST["filter_source"]))
		{
			$l_rules["filter_source"]["p_strSelectedID"] = $_POST["filter_source"];
		}
		else
		{
			$l_rules["filter_source"]["p_strSelectedID"] = "-1";
		} // if

		if (isset($_POST["filter_alert"]))
		{
			$l_rules["filter_alert"]["p_strSelectedID"] = $_POST["filter_alert"];
		}
		else
		{
			$l_rules["filter_alert"]["p_strSelectedID"] = "-1";
		} // if

		if (isset($_POST["filter_type"]))
		{
			$l_rules["filter_type"]["p_strSelectedID"] = $_POST["filter_type"];
		}
		else
		{
			$l_rules["filter_type"]["p_strSelectedID"] = "-1";
		} // if

		if (isset($_POST["filter_from__HIDDEN"]))
		{
			$l_rules["filter_from"]["p_strValue"] = $_POST["filter_from__HIDDEN"];
		} // if

		if (isset($_POST["filter_to__HIDDEN"]))
		{
			$l_rules["filter_to"]["p_strValue"] = $_POST["filter_to__HIDDEN"];
		} // if

		if (isset($_POST["filter_archive"]))
		{
			$l_rules["filter_archive"]["p_strSelectedID"] = $_POST["filter_archive"];
		}
		else
		{
			$l_rules["filter_archive"]["p_strSelectedID"] = "0";
		} // if

		if (isset($_POST["filter_user__HIDDEN"]))
		{
			$l_rules["filter_user"]["p_strSelectedID"] = $_POST["filter_user__HIDDEN"];
		} // if

		$g_comp_template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
	} // function


	/**
	 * Prepares an array of the changes in a category for the loogbook.
	 *
	 * @param   object  $p_dao
	 * @param   array   $p_dataset_from
	 * @param   array   $p_category_values
	 * @return  array
	 * @author  Van Quyen Hoang <qhoang@synetics.de>
	 */
	public function prepare_changes($p_dao, $p_dataset_from = null, $p_category_values = null)
	{
		global $g_convert;

		// Properties of category.
		$l_properties = $p_dao->get_properties();

		$l_changes_array = array();

		$l_category_values = $p_category_values[isys_import_handler_cmdb::C__PROPERTIES];
		// For mass changes we have to go through each property so that it can also document empty fields.
		foreach($l_properties as $l_key => $l_prop_info)
		{
			if ((!$l_prop_info[C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__IMPORT] && isset($l_category_values[$l_key])) ||
				!isset($l_category_values[$l_key]))
				continue;


			$l_arData = null;
			$l_from = null;
			$l_to = null;
			$l_unit_property = null;
			$l_has_callback = false;

			$l_field = ((isset($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS]))?
						$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS]: $l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]);

			// Some properties needs a special handling.
			if (isset($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1]))
			{
				switch ($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1])
				{
                    case 'dialog_multiselect':
                        /**
                         * @todo make dialog_multiselect work in changes array
                         */
                        if (isset($l_prop_info[C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']))
                        {

                        }
                        break;
					case 'dialog':
					case 'dialog_plus':
					case 'get_yes_or_no':
						if (isset($l_prop_info[C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']))
						{
							$l_arData = $l_prop_info[C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData'];
							if($l_arData instanceof isys_callback){
								$l_arData = $l_arData->execute();
							} // if

							if(is_string($l_arData)){
								$l_arData = unserialize($l_arData);
							} // if
						}
						else if (isset($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES]))
						{
							$l_field =	((isset($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS]))?
								$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS]: $l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '__title');
						} // if
						break;
					case 'location':
					case 'object':
						$l_from = $this->fetch_object_name_by_id($p_dataset_from[$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]], $p_dao);
						$l_to = $this->fetch_object_name_by_id($l_category_values[$l_key]['value'], $p_dao);
						break;
					case 'connection':
						if (isset($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS]))
						{
							$l_from = $this->fetch_object_name_by_id($p_dataset_from[$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS].'__object'], $p_dao);
							$l_to = $this->fetch_object_name_by_id($l_category_values[$l_key]['value'], $p_dao);
						}
						else
						{
							$l_from = $this->fetch_object_name_by_id($p_dataset_from['isys_connection__isys_obj__id'], $p_dao);
							$l_to = $this->fetch_object_name_by_id($l_category_values[$l_key]['value'], $p_dao);
						} // if

						$l_has_callback = true;
						break;
					case 'contact':
						$l_res_from = $p_dao->retrieve('SELECT isys_contact_2_isys_obj__isys_obj__id FROM isys_contact_2_isys_obj WHERE isys_contact_2_isys_obj__isys_contact__id = '.$p_dao->convert_sql_id($p_dataset_from[$l_field]).' ORDER BY isys_contact_2_isys_obj__isys_obj__id ASC');
						while ($l_row_from = $l_res_from->get_row())
						{
							$l_from .= $this->fetch_object_name_by_id($l_row_from['isys_contact_2_isys_obj__isys_obj__id'], $p_dao).',';
						} // while
						$l_from = rtrim($l_from, ',');

						$l_res_to = $p_dao->retrieve('SELECT isys_contact_2_isys_obj__isys_obj__id FROM isys_contact_2_isys_obj WHERE isys_contact_2_isys_obj__isys_contact__id = '.$p_dao->convert_sql_id($l_category_values[$l_key]['value']).' ORDER BY isys_contact_2_isys_obj__isys_obj__id ASC');
						while ($l_row_to = $l_res_to->get_row())
						{
							$l_to .= $this->fetch_object_name_by_id($l_row_to['isys_contact_2_isys_obj__isys_obj__id'], $p_dao).',';
						} // while

						$l_to = rtrim($l_to, ',');

						$l_has_callback = true;
						break;
					case 'convert':
						// Checks whether an unit property exists
						if (isset($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__UNIT]))
						{
							$l_unit_property = $l_properties[$l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__UNIT]];
						} // if
						break;
					default:
						$l_helper = new $l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][0](
							$p_dataset_from,
							$p_dao->get_database_component(),
							$l_prop_info[C__PROPERTY__DATA],
							$l_prop_info[C__PROPERTY__FORMAT],
							$l_prop_info[C__PROPERTY__UI]);

						$l_method = $l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1];

						if (isset($p_dataset_from))
						{
							$l_helper_value = $l_helper->$l_method($p_dataset_from[$l_field]);

							if (is_object($l_helper_value))
							{
								$l_helper_value_data = $l_helper_value->get_data();
							}
							else
							{
								$l_helper_value_data = $l_helper_value;
							} // if

							if (is_array($l_helper_value_data) > 0)
							{
								if (is_array($l_helper_value_data[0]))
								{
									$l_helper_value_data = $l_helper_value_data[0];
								} // if

								if (isset($l_helper_value_data['ref_id']))
								{
									if ($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'get_ip_reference')
									{
										$l_from = $l_helper_value_data['ref_title'];
									}
									else
									{
										$l_from = $l_helper_value_data['ref_id'];
									} // if
								}
								elseif(isset($l_helper_value_data['title']))
								{
									$l_from = $l_helper_value_data['title'];
								}
								else
								{
									$l_from = $l_helper_value_data['id'];
								} // if

							}
							else
							{
								$l_from = $l_helper_value_data;
							} // if
						} // if

						if (isset($l_category_values[$l_key]['ref_title']))
						{
							$l_to = $l_category_values[$l_key]['ref_title'];
						}
						elseif (isset($l_category_values[$l_key]['value']))
						{
							if(is_numeric($l_category_values[$l_key]['value']))
							{

								$l_helper_value = $l_helper->$l_method($l_category_values[$l_key]['value']);

								if (is_object($l_helper_value))
								{
									$l_helper_value_data = $l_helper_value->get_data();
								}
								else
								{
									$l_helper_value_data = $l_helper_value;
								} // if

								if (is_array($l_helper_value_data) > 0)
								{
									if (is_array($l_helper_value_data[0]))
									{
										$l_helper_value_data = $l_helper_value_data[0];
									} // if

									if (isset($l_helper_value_data['ref_id']))
									{
										if ($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'get_ip_reference')
										{
											$l_to = $l_helper_value_data['ref_title'];
										}
										else
										{
											$l_to = $l_helper_value_data['ref_id'];
										} // if
									}
									elseif(isset($l_helper_value_data['title']))
									{
										$l_to = $l_helper_value_data['title'];
									}
									else
									{
										$l_to = $l_helper_value_data['id'];
									} // if

								}
								else
								{
									$l_to = $l_helper_value_data;
								} // if
							}
							else
							{
								$l_to_val = $l_category_values[$l_key]['value'];
								if (is_array($l_to_val))
								{
									if (isset($l_to_val['ref_title']))
									{
										$l_to = $l_to_val['ref_title'];
									}
									else
									{
										foreach ($l_to_val as $l_to_part)
										{
											$l_to .= $l_to_part . ',';
										} // foreach
										$l_to = rtrim($l_to, ',');
									} // if
								}
								else
								{
									$l_to = $l_to_val;
								} // if
							}
						}

						if (isset($l_category_values[$l_key]['reference']) && (!empty($l_from) || !empty($l_to)))
						{
							$l_query = 'SELECT ' . $l_category_values[$l_key]['reference'] . '__title FROM ' . $l_category_values[$l_key]['reference'] .
								' WHERE ' . $l_category_values[$l_key]['reference'] . '__id = ';
							if (isset($l_from))
							{
								// Retrieve title only if value is numeric
								if(is_numeric($l_from))
								{
									$l_from = $p_dao->retrieve($l_query . $p_dao->convert_sql_id($l_from) . ';')->get_row_value($l_category_values[$l_key]['reference'].'__title');
								} // if
							} // if

							if (isset($l_to))
							{
								// Retrieve title only if value is numeric
								if(is_numeric($l_to))
								{
									$l_to = $p_dao->retrieve($l_query . $p_dao->convert_sql_id($l_to) . ';')->get_row_value($l_category_values[$l_key]['reference'].'__title');
								} // if
							} // if
						} // if

						// @todo think how to solve this problem
						if (($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'get_net_dns_server' ||
								$l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'get_ip_reference')
							&& (! empty($l_from) || ! empty($l_to)))
						{
							$l_query = 'SELECT isys_cats_net_ip_addresses_list__title FROM isys_cats_net_ip_addresses_list ' .
								'INNER JOIN isys_catg_ip_list ON isys_catg_ip_list__isys_cats_net_ip_addresses_list__id = isys_cats_net_ip_addresses_list__id ' .
								'WHERE isys_catg_ip_list__id = ';

							if (isset($l_from) && is_numeric($l_from))
							{
								$l_data_from2 = $p_dao->retrieve($l_query . $p_dao->convert_sql_id($l_from))->get_row();
								$l_from = $l_data_from2['isys_cats_net_ip_addresses_list__title'];
							} // if

							if (isset($l_to) && is_numeric($l_to))
							{
								$l_data_to2 = $p_dao->retrieve($l_query . $p_dao->convert_sql_id($l_to))->get_row();
								$l_to = $l_data_to2['isys_cats_net_ip_addresses_list__title'];
							} // if
						} // if

						$l_has_callback = true;

						unset($l_helper);
						break;
				} // switch

			}
			else
			{
				if (isset($l_prop_info[C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData']))
				{
					$l_arData = $l_prop_info[C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_arData'];

					if (is_object($l_arData))
					{
						$l_arData = $l_arData->execute();
					} // if

					if (is_string($l_arData))
					{
						$l_arData = unserialize($l_arData);
					} // if
				} // if
			} // if



			// Checks whether data comes from the callback in the ui property
			if (isset($l_arData))
			{
                if (isset($p_dataset_from[$l_field]) && isset($l_arData[$p_dataset_from[$l_field]])) {
                    $l_from = $l_arData[$p_dataset_from[$l_field]];
                }

				if(is_array($l_category_values[$l_key]['value']))
				{
					if(isset($l_category_values[$l_key]['value']['value'])){
						$l_category_values[$l_key] = $l_category_values[$l_key]['value'];
					} // if
				} // if

                if (isset($l_key) && isset($l_category_values[$l_key]['value']) && isset($l_arData[$l_category_values[$l_key]['value']])) {
                    $l_to = $l_arData[$l_category_values[$l_key]['value']];
                }
			}
			else if (!isset($l_from) && !isset($l_to))
			{
				if (isset($l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1]) && (
                    $l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'dialog' ||
                    $l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'dialog_plus'))
				{

					if (isset($p_dataset_from) && !array_key_exists($l_field, $p_dataset_from))
					{
						if (is_numeric($p_dataset_from[$l_field]))
						{
							$l_from_data = $p_dao->get_dialog($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], $p_dataset_from[$l_field])->__to_array();
							$l_from = $l_from_data[$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0].'__title'];
						} // if
					}
					else
					{
						$l_from = $p_dataset_from[$l_field];
					} // if
                    if ($l_category_values[$l_key]['title_lang']) {
                        $l_to = $l_category_values[$l_key]['title_lang'];
                    }
					elseif(isset($l_category_values[$l_key]['value']) && is_numeric($l_category_values[$l_key]['value']))
					{
						$l_to_data = $p_dao->get_dialog($l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], $l_category_values[$l_key]['value'])->__to_array();
						$l_to = $l_to_data[$l_prop_info[C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0].'__title'];
					}
				}
				else if (!$l_has_callback)
				{
					$l_from = $p_dataset_from[$l_field];
                    if (isset($l_category_values[$l_key]['value'])) {
                        $l_to = $l_category_values[$l_key]['value'];
                    }
				} // if
			} // if

			// Replaces the raw data with the converted data
			if (isset($l_unit_property))
			{
				$l_unit_field_from = $p_dataset_from[$l_unit_property[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]];

				$l_method = $l_prop_info[C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0];

				$l_from = $g_convert->$l_method($l_from, $l_unit_field_from, C__CONVERT_DIRECTION__BACKWARD);
				$l_to = $l_category_values[$l_key]['value'];
			} // if

			// Check
			if (!is_array($l_from) && !is_array($l_to) && (!empty($l_from) || !empty($l_to)))
			{
                $l_formatted_from = trim(_L($l_from));
                $l_formatted_to = trim(_L($l_to));

				if (strtolower($l_formatted_from) != strtolower($l_formatted_to))
				{
					$l_changes_array[get_class($p_dao).'::'.$l_key] = array(
						'from' => $l_formatted_from,
						'to' => $l_formatted_to
					);
				} // if
			} // if
		} // foreach

		return $l_changes_array;
	} // function


    /**
     * Fetches object name by its identifier.
     *
     * @param   integer         $p_id
     * @param   isys_cmdb_dao   $p_dao
     * @return  integer
     */
	protected function fetch_object_name_by_id ($p_id, $p_dao)
	{
		if (!is_array($this->m_objects) || !array_key_exists((int) $p_id, $this->m_objects))
		{
			$this->m_objects[$p_id] = $p_dao->get_obj_name_by_id_as_string($p_id);
		} //if

		return $this->m_objects[$p_id];
	} // function


	/**
	 * Initialize method.
	 *
	 * @param   isys_module_request  $p_req
	 * @return  boolean
	 */
	public function init (isys_module_request &$p_req)
	{
		if (is_object($p_req))
		{
			return true;
		} // if

		return false;
	} // function

    /**
     * Get related auth class for module
     *
     * @author Selcuk Kekec <skekec@i-doit.com>
     * @return isys_auth
     */
    public static function get_auth()
    {
        return isys_auth_logbook::instance();
    } // function
} // class