/**
  * i-doit ajax implementation
  * author: dennis stuecken <dstuecken@synetics.de>
  */

/* Extend prototype browser detection */
if (!('IE8' in Prototype.Browser)) {
  Prototype.Browser.IEVersion = (navigator.appVersion.split(';').length > 1) ? parseFloat(navigator.appVersion.split(';')[1].strip().split(' ')[1]) : 0;
  Prototype.Browser.IE6 =  Prototype.Browser.IEVersion == 6;
  Prototype.Browser.IE7 =  Prototype.Browser.IEVersion == 7;
  Prototype.Browser.IE8 =  Prototype.Browser.IEVersion == 8;
}

var isys_ajax_queue = {
	_sim_requests: 15,
	_urls: [],
	_targets: [],
	_options: [],

	set_sim_requests: function(reqs) {
		if (!isNaN(reqs)) this._sim_requests = reqs;
	},

	push: function(p_url, p_options, p_target){
		this._urls.push(p_url);
		this._options.push(p_options);

		if(p_target == null || typeof p_target == 'undefined') this._targets.push("__REQUEST_ONLY");
		else this._targets.push(p_target);

		this._next();
	},

	clear: function() {
		this._urls = [];
		this._targets = [];
		this._options = [];
	},

	_next: function() {
		if(Ajax.activeRequestCount < isys_ajax_queue._sim_requests) {
			if(isys_ajax_queue._targets.first() == "__REQUEST_ONLY") {
				var junk = isys_ajax_queue._targets.shift();

				return new Ajax.Request(isys_ajax_queue._urls.shift(),
										isys_ajax_queue._options.shift());

			} else
				return new Ajax.Updater(isys_ajax_queue._targets.shift(),
										isys_ajax_queue._urls.shift(),
										isys_ajax_queue._options.shift());
		}
	}
};

isys_ajax_ = Class.create(Ajax.Base, {
	_method:'get',
	_asynchronous:true,
	_form:'isys_form',
	_url:'',
	_last:'',

	_ajax: function() {
		return (this._url.match(/^.*[\?].*$/))?'&ajax=1':'?ajax=1';
	},

	initialize: function(p_method, p_encoding, p_async) {
		if (p_method) this._method = p_method;
		if (p_async) this._asynchronous = p_async;

		this._url = String(document.location);
	},

	initOptions: function(p_options) {
		var options = {
			history: {
	            cache : true,
	            onStateChange: function(state) {
	                //History.setTitle(History.getTitle());
	            }
	        },
			asynchronous:this._asynchronous,
			evalScripts:true,
			onSuccess:this.onSuccess.bind(this),
			onFailure:this.onFailure.bind(this),
			onComplete:this.onComplete.bind(this)
		};
		Object.extend(options, p_options || { });

		return options;
	},

	/*
	 * @param string p_method ('post','get') - request method
	 */
	method: function(p_method) {
		_method = p_method;
	},

	add: function(p_url, p_target) {
		this._url = p_url;

		if (!Object.isUndefined(History)) {
			History.add([p_url, p_target, new Date()]);
		}
	},

	last: function() {
		if (!Object.isUndefined(History)) {
			return History.last();
		} else return false;
	},

	_updater: function(p_target, p_url, p_options) {
		if (!p_url || p_url == '') {
			p_url = String(document.location);
		}

		this._target = p_target;
		return isys_ajax_queue.push(p_url + this._ajax(), p_options, this._target);
	},

	_request: function(p_url, p_options) {
		if (!p_url || p_url == '') {
			p_url = String(document.location);
		}

		return isys_ajax_queue.push(p_url + this._ajax(), p_options, null);
	},

	call: function(p_url, p_options) {
		this._request(p_url, p_options);
	},

	/**
	  * History processing (Used to attach ajax calls into the browser history)
	  *
	  */
	_history: function(p_url, p_target, p_options) {

		// History id
        if(Object.isUndefined(p_options.history.id)) {
            var id = (Object.isString(p_target)) ? p_target : p_target.id;
        } else {
            var id = p_options.history.id;
        }

        var tmpOpt = Object.clone(Ajax.History.Features);
        Object.extend(tmpOpt, p_options.history);
        options.history = tmpOpt;
        options.history.__url = p_url+this._ajax();

        options.container = p_target;

        History.set(id, Ajax.History.observe('Updater', id, p_options.history.__url, p_options));
	},

	onSuccess: function() {

		/* Reset form */
		if (p_reset_form && $(p_form)) {
			$(p_form).reset();
		}

		/* Fade out something after sending successfull
		 request (The submit button for example) */
		if ($(p_fade_out)) {
			$(p_fade_out).disabled = 'disabled';
			new Effect.Fade($(p_fade_out), {duration: 0.4});
		}

	},

	onComplete: Prototype.emptyFunction,

	onFailure: function() {
		//hideLightbox();
		$(this._target).update('Error.');
	}

});

/**
 * Submit an ajax request
 *
 * @author dennis stuecken <dstuecken@synetics.de>
 *
 * @param string p_url ('', 'whatever') - location
 * @param string p_target ('id-of-element') - target element (e.g. div)
 * @param string p_form ('', 'id-of-form') - id of form, leave empty for get requests
 */
Ajax.Submit = Class.create(isys_ajax_, {
	initialize: function (p_url, p_target, p_form, p_options) {
		/* Check for p_url existence. If empty, use the current document location */
		if (!p_url || p_url == '') p_url = this._url;
		this.add(p_url);

		if (!p_form || p_form == '') p_form = this._form;

		/* Show loading overlay */
		//showLightbox(null, true);

		var l_params = {};

		if (p_form && $(p_form)) {
			if (!p_options.parameters) p_options.parameters = {};

			/* Prepare all form values to submit them */
			l_params = Object.extend(p_options.parameters, $(p_form).serialize(true));
		}

		if (typeof p_options == 'object' && typeof p_options.method != 'undefined') {
			Object.extend(p_options, { method:'post', history:false });
		}

		var options = this.initOptions(p_options);
		options.parameters = l_params;

		return this._updater(p_target, p_url, options);
	}
});

Ajax.Call = Class.create(isys_ajax_, {
	initialize: function(p_url, p_options) {
		var options = this.initOptions(p_options);
        this.add(p_url);

		options.history = false;
        return this._request(p_url, options);
	}
});

Ajax.Get = Class.create(isys_ajax_, {
    initialize: function (p_target, p_url, p_options) {
        options = this.initOptions(p_options);
        Object.extend(options, p_options);
        this._url = p_url;

        if (console && console.log)
            console.log('Ajax.Get is deprecated. remove this call: ' + p_target + ': ' + p_url);

        /* IE6 compatibility issue */
        if (Prototype.Browser.IE6) {
            options.onSuccess = null;

            return new Ajax.Updater(p_target, p_url + this._ajax(), options);
        }

        // Set historing only if option is active
        if (options.history) {

            return this._history(p_url, p_target, options);

        }
        else {
            if (Object.isArray(p_target)) {
                this._updater(p_target[1], p_target[0], options);
                if (!Object.isUndefined(History)) History.back();
            }
            else {
                return this._updater(p_target, p_url, options);
            }
        }

    }
});

Ajax.Effect = Class.create(isys_ajax_, {
	initialize: function(p_effect, p_target, p_url, p_options) {

		this.add(p_url, p_target);

		this._options = {
			duration:0.2
		};
		Object.extend(this._options, p_options || { });

		if (Object.isString(p_effect)) {
			this._effect = p_effect;
		}
		else return false;

		if (Object.isString(p_target) && $(p_target))
			this._target = p_target;

		options = this.initOptions(p_options);

		this._updater(p_target, p_url, options);
	},

	onComplete: function(transport) {
		if (this._target) {
			if (!$(this._target).visible()) {
				new Effect[this._effect](this._target, this._options);
			}
		}
	}
});


/* Ajax history: based on prototypextensions.com's Ajax.History extension */

var History = {
    __altered: false,
    __currentHash: null,
    __previousHash: null,
    __iframe: false,
    __title: false,
    hash : new Hash(),
    maximum:25,

	setMaximum: function(p_max) {
		if (Object.isNumber(p_max))
			this.maximum = p_max;
	},

	back: function() {
		if (this._urls.length > 2) {
			this._urls.pop();
			this._urls.pop();

			return true;
		} else return false;
	},

	all: function() {
		return this._urls;
	},

	get: function(i)	{
		return this._urls[i];
	},

	go: function(i) {
		var l_go = (this._urls.length >= i)?this._urls[this._urls.length + i]:false;
		if (l_go) {
			return new Ajax.Get(l_go);
		} else return false;
	},

	prev: function() {
		return (this._urls.length >= 2)?this._urls[this._urls.length - 2]:false;
	},

	last: function() {
		return this._urls.last();
	},

	add: function(req) {
		if (Object.isArray(req) && Object.isString(req[0])) {
			this._urls.push(req);

			if (this.maximum && this.num() > this.maximum) {
				this._urls.splice(0,1);
			}
		}

	},

	num: function() {
		return this._urls.size();
	},

    /**
     * init()
     * @desc Initialize the hash. Call this method in first
     */
    init: function() {
        var inst  = this;
        var hash  = location.hash.substring(1);
        this.hash = $H(hash.toQueryParams());
        this.__currentHash  = hash;
        this.__previousHash = hash;
		this._urls = [];

        this.__title = document.title;

        if(Prototype.Browser.IE) {
            document.observe('dom:loaded', function(e) {
                if(!$('px-historyframe')) {
                    History.__iframe = new Element('iframe', {
                        name   : 'px-historyframe',
                        id     : 'px-historyframe',
                        src    : '',
                        width  : '0',
                        height : '0',
                        style  : {
                            visibility: 'hidden'
                        }
                    });

                    document.body.appendChild(History.__iframe);

                    History.setHashOnIframe(inst.hash.toQueryString());
                }
            });
        }
    },

	add: function() {

    },

    /**
     * set( string $name, string $value )
     *
     * @desc Set new value $value for parameter $name
     */
    set: function($name, $value) {
        this.__previousHash = this.hash.toQueryString();
        this.hash.set($name, $value);
        this.apply();
    },

    /**
     * get( string $name )
     *
     * @desc Get value parameter $name
     */
    get: function($name) {
        return this.hash.get($name);
    },

    /**
     * unset( string $name )
     *
     * @desc Unset parameter $name
     */
    unset: function($name) {
        this.hash.unset($name);
        this.apply();
    },

    /**
     * update()
     *
     * @desc Updates this.hash with the current hash
     */
    update: function() {
        this.__previousHash = this.hash.toQueryString();
        var hash = window.location.hash.substring(1);

        // If IE, look in the iframe if the hash is updated
        if(Prototype.Browser.IE && this.__iframe ) {
            var hashInFrame = this.getHashOnIframe();

            if(hashInFrame != hash) {
                hash = hashInFrame;
            }
        }

        this.hash = $H(hash.toQueryParams());
        this.__currentHash = hash;
    },

    /**
     * apply()
     *
     * @desc Apply this.hash to location.hash
     */
    apply: function() {
        var newHash = this.hash.toQueryString();

        // set new hash
        window.location.hash = newHash;

        // If IE, apply new hash to frame for history
        if(Prototype.Browser.IE && this.__iframe) {
            if(this.__currentHash != newHash)
            {
                this.setHashOnIframe(newHash);
            }
            else if(newHash != this.getHashOnIframe())
            {
                this.setHashOnIframe(newHash);
            }
        }
    },

    /**
     * isAltered()
     *
     * @desc Return true if current hash is different of previous hash.
     * this.__altered allows to force the dispatch.
     */
    isAltered: function() {
        if(this.__altered == true) {
            return true;
        }
        this.__altered = false;

        return (History.__currentHash != History.__previousHash);
    },

    /**
     * setHashOnIframe()
     *
     * @use  For IE compatibility
     * @desc Set hash value on iframe
     */
    setHashOnIframe: function(hash) {
        try {
            var doc = History.__iframe.contentWindow.document;
            doc.open();
            doc.write('<html><body id="history">' + hash + '</body></html>');
            doc.close();
        } catch(e) {}
    },

    /**
     * getHashOnIframe()
     *
     * @use  For IE compatibility
     * @desc Get hash value on iframe
     */
    getHashOnIframe: function() {
        var doc = this.__iframe.contentWindow.document;
        if (doc && doc.body.id == 'history') {
            return doc.body.innerText;
        } else {
            return this.hash.toQueryString();
        }
    },

    /**
     * setTitle()
     *
     * @desc Set a new title for window
     */
    setTitle: function(title) {
        if(document.title) {
            document.title = title;
        }
    },

    /**
     * getTitle()
     *
     * @desc Return current window title
     */
    getTitle: function() {
        return this.__title;
    }
};

/**
 * History.Registry
 *
 * @desc Used to register a callback for a parameter
 */
History.Registry =
{
    /**
     * @desc Hash
     */
    hash : new Hash(),

    /**
     * set( string $config )
     *
     * @desc Set new value $historyId for parameter $config
     */
    set: function($config) {

        if(typeof($config) != 'object') {
            throw('History.Registry.set : $config must be a javascript object');
        }

        // id
        if(!$config.id || !Object.isString($config.id)) {
            throw('History.Registry.set : $config.id must be a string');
        }

        // onStateChange
        if(!$config.onStateChange || !Object.isFunction($config.onStateChange)) {
            throw('History.Registry.set : $config.onStateChange '
                + 'must be a javascript callback function');
        }

        // defaultValue
        if(!$config.defaultValue || !Object.isString($config.defaultValue)) {
            $config.defaultValue = '';
        }

        this.hash.set($config.id, $config);
    },

    /**
     * get( string $id )
     *
     * @desc Get value parameter $id
     */
    get: function($id) {
        return this.hash.get($id);
    },

    /**
     * unset( string $id )
     *
     * @desc Unset parameter $id
     */
    unset: function($id) {
        this.hash.unset($id);
    }
};

/**
 * History.Observer
 *
 * @desc Used to perform actions defined in the registry,
 * according to the hash of the url.
 */
History.Observer = {

    /**
     * @desc Interval delay in seconds
     */
    delay : 0.2,

    /**
     * @desc Interval timer instance
     */
    interval : null,

    /**
     * @desc If interval is started : true, else false
     */
    started : false,

    /**
     * start()
     *
     * @desc Start an interval timer
     */
    start: function() {
        if(this.started) return;
        this.interval = new PeriodicalExecuter(History.Observer.dispatch, this.delay);
        this.started = true;
    },

    /**
     * stop()
     *
     * @desc Stop the interval timer
     */
    stop: function() {
        if(!this.started) return;
        this.interval.stop();
        this.started = false;
    },

    /**
     * dispatch()
     *
     * @desc This method is called each time interval,
     * the dispatch of the registry is implemented only if
     * the hash has been amended (optimisiation)
     */
    dispatch: function() {

        // Update the hash
        History.update();

        // Dispatch only if location.hash has been altered
        if(History.isAltered()) {

            //if(console) console.log('pass');
            History.hash.each(function(pair)  {
                var registry = History.Registry.get(pair.key);
                if(registry) {
                   registry.onStateChange.bind(History)( pair.value );
                }
            });
        }
    }
};

/**
 * Ajax.History
 *
 * @desc Provides core methods to easily manage browsing history
 * with Ajax.History.Request / Updater.
 */
Ajax.History = {

    /**
     * @desc Allowed Ajax.History prefix (for validation)
     */
    types : ['Request', 'Updater', 'Appear'],

    /**
     * observe( string $type, string $id, string $url, object $options )
     *
     * @desc This method helps manage the browsing history
     */
    observe: function($type, $id, $url, $options) {

        var getter         = Ajax.History.Registry.get($id);
        var currentVersion = 0;
        var output         = false;

        // Type validation
        if(this.types.indexOf($type) == -1) {
			throw('Ajax.History.observer() -> type ' + $type + ' is invalid !');
        }

        // Registry management
		if(getter == undefined) {
            currentVersion = ($options.history.state) ? $options.history.state : 0;
            var hash = new Hash();
            hash.set(currentVersion, $options);
            Ajax.History.Registry.set($id, hash);
		} else {
            currentVersion = ($options.history.state)
                ? $options.history.state : this.getCurrentVersion($id);
            getter.set(currentVersion, $options);
		}

        // add handler on registry
		this.addCallback($type, $id);

        return currentVersion;
    },

    /**
     * addCallback( string $type, string $id )
     *
     * @desc This method adds a state for request on History.Registry
     */
    addCallback: function($type, $id) {

        History.Observer.start();

        // Set history altered state to true : force dispatch
        History.__altered = true;

        // Return void if registry is already set
        if(!Object.isUndefined(History.Registry.get($id))) return;

        // Add this id to history registry
        History.Registry.set({
            id: $id,
            onStateChange: function(state) {
                var options = Ajax.History.Registry.get($id).get(state.toString());
                var request = null;

                if(Object.isUndefined(options)) return;

                if(options.history.cache == true && options.history.__request != null) {
                    new Ajax.Cache(options.history.__request);
                } else {

                    if($type == 'Request') {
						request = isys_ajax_queue.push(options.history.__url, options, null);
                    } else if($type == 'Updater') {
                    	request = isys_ajax_queue.push(options.history.__url, options, options.container);
                    } else if($type == 'Appear') {
                        request = new Ajax.Effect('appear', options.container, options.history.__url, options);
                    }

                    options.history.__request = request;
                }

                History.__altered = false;

                if (Object.isFunction(options.history.onStateChange)) {
                    options.history.onStateChange(state);
                }
            }
        });
    },

    /**
     * getCurrentVersion( string $id )
     *
     * @desc This method returns the current state in history
     * (if the state is not defined)
     */
    getCurrentVersion: function($id) {
        var getter = Ajax.History.Registry.get($id);
        return Object.isUndefined(getter) ? 0 : getter.keys().length;
    }
};

/**
 * Ajax.History.Registry
 *
 * @desc Simple Hash to manage request history
 */
Ajax.History.Registry = new Hash();

/**
 * Ajax.History.Features
 *
 * @desc It's default request options for Ajax.History.Request/Updater
 */
Ajax.History.Features = {
    id              : null,    // set custom history value for this instance
    state           : false,   // set custom state value for this instance
    cache           : false,   // enable/disable history cache
    onStateChange   : null,    // this handler is called when history change
    __url           : null,
    __request       : null
};

/**
 * Ajax.Cache
 *
 * @desc Ajax.Cache can "simulate" an Ajax request from an
 * Ajax.Request/Updater made beforehand.
 */
Ajax.Cache = Class.create(Ajax.Base, {
    _complete: false,
    initialize: function($super, request) {
        $super(request.options);
        request._complete = false;
        this.transport = request.transport;
        this.request(request.url);
        return this;
    },

    request: function(url) {
        this.url = url;
        this.method = this.options.method;
        var params = Object.clone(this.options.parameters);

        try {
            var response = new Ajax.Response(this);

            if (this.options.onCreate) this.options.onCreate(response);
            Ajax.Responders.dispatch('onCreate', this, response);

            if (this.options.asynchronous) this.respondToReadyState.bind(this).defer(1);

            this.onStateChange();
        }
        catch (e) {
            this.dispatchException(e);
        }
    }
});

Object.extend(Ajax.Cache.prototype, {
    respondToReadyState : Ajax.Request.prototype.respondToReadyState,
    onStateChange       : Ajax.Request.prototype.onStateChange,
    success             : Ajax.Request.prototype.getStatus,
    getStatus           : Ajax.Request.prototype.getStatus,
    isSameOrigin        : Ajax.Request.prototype.isSameOrigin,
    getHeader           : Ajax.Request.prototype.getHeader,
    evalResponse        : Ajax.Request.prototype.evalResponse,
    dispatchException   : Ajax.Request.prototype.dispatchException
});


/**
 * Ajax.Request Extended
 *
 * @desc Just a small change: now Ajax.Request return self scope.
 * It is required by Ajax.Cache
 */
Ajax.Request = Class.create(Ajax.Request, {
    initialize: function($super, url, options) {
        $super(url, options);
        return this;
    }
});

Ajax.Request.Events = ['Uninitialized', 'Loading', 'Loaded', 'Interactive', 'Complete'];

/**
 * Ajax.Updater Extended
 *
 * @desc Just a small change: now Ajax.Updater return self scope
 * It is required by Ajax.Cache
 */
Ajax.Updater = Class.create(Ajax.Updater, {
    initialize: function($super, container, url, options) {
        $super(container, url, options);
        return this;
    }
});

Ajax.Responders.register({
  onComplete: isys_ajax_queue._next
});


/* ----------------------------------------------------------------------------------------------------------- */
/* ----------------------------------------------------------------------------------------------------------- */

/**
  * Submit an ajax request
  *
  * @author dennis stuecken <dstuecken@synetics.de>
  *
  * @deprecated This function is deprecated. Use Ajax.Get or Ajax.Submit instead .
  *
  * @param string p_url ('', 'whatever') - location
  * @param string p_method ('post','get') - request method
  * @param string p_target ('id-of-element') - target element (e.g. div)
  * @param string p_form ('', 'id-of-form') - id of form, leave empty for get requests
  * @param string p_fade_out ('', 'id-of-element') - id of element to fade out (e.g. submit button), empty for no fade out
  * @param int p_reset_form (1,0) - reset form after submit? leave empty for get requests
  * @param function p_success - onSuccess callback
  * @param function p_complete - onComplete callback
  * @param function p_failure - onFailure callback
  */
function aj_submit(p_url, p_method, p_target, p_form, p_fade_out, p_reset_form, p_success, p_complete, p_failure) {
  var l_params = '';

  /* Check for p_url existence. If empty, use the current one */
  if (!p_url || p_url == '') p_url = String(document.location);

  if (!p_method) p_method = 'get';

  /* Show loading overlay */
  //showLightbox(null, true);

  if ($(p_form) || p_method == 'get') {
    if ($(p_form)) {
      /* Prepare all form values to submit them */
      l_params = $(p_form).serialize(true);
    }

    if (p_url.search('^.*[\?].*$') != -1) l_ajax = '&ajax=1';
    else l_ajax = '?ajax=1';

    /* Send the request and update the layer */
    isys_ajax_queue.push(
    		p_url+l_ajax,
			{
				method:p_method,
				evalScripts:true,
				evalJS:true,
				onComplete: p_complete,
				onSuccess: function(transport) {
					/* Evaluate p_success */
					if (typeof p_success != 'undefined') p_success(transport);

					/* Show p_target */
					if (!$(p_target).visible()) $(p_target).show();

					/* Hide the overlay*/
					//hideLightbox();

					/* Reset form */
					if (p_reset_form && $(p_form)) {
						$(p_form).reset();
					}

					/* Fade out something after sending successfull
					 request (The submit button for example) */
					if ($(p_fade_out)) {
						$(p_fade_out).disabled = 'disabled';
						new Effect.Fade($(p_fade_out), {duration: 0.4});
					}

					},
				onFailure: function() {
					/* Evalute p_failure */
					if (p_failure != 'undefined') p_failure();

					//hideLightbox();
					$(p_target).innerHTML = 'Error.';
				},
				parameters:l_params
			},
			p_target);
  }

}