/* Inserting a "IEVersion" variable to the "Prototype.Browser" object */
if (Prototype.Browser.IE) {
	Prototype.Browser.IEVersion = parseFloat(navigator.appVersion.split(';')[1].strip().split(' ')[1]);
	Prototype.Browser.IE6 =  Prototype.Browser.IEVersion == 6;
	Prototype.Browser.IE7 =  Prototype.Browser.IEVersion == 7;
}

/* Adding "getOwnPropertyNames" function to the Object class, only for IE <9 */
if (Prototype.Browser.IEVersion < 9) {
	Object.getOwnPropertyNames = function (obj) {
		var arr = [];
		for (var k in obj) {
			if (obj.hasOwnProperty(k))
				arr.push(k);
		}
		return arr;
	}
}

/* Extends prototypes utility functions to getElementsByName */
function $N(name, doc) {
	return (doc || document).getElementsByName(name);
}

/**
 * Fancy "in_array" method for javascript.
 * Use like: [1,2,3,4].in_array(3);
 *
 * @param    needle
 * @returns  boolean
 * @author   Leonard Fischer <lfischer@i-doit.com>
 */
Array.prototype.in_array = function(needle) {
	for(var i=0; i < this.length; i++) if(this[i] === needle) return true;

	return false;
};

/**
 * Small "sum" implementation to sum up an array of numbers.
 * Use like: [1,2,3,4,5].sum(); => 15
 *
 * @returns  integer
 * @see      http://stackoverflow.com/questions/3762589/fastest-javascript-summation
 */
if (!Array.prototype.hasOwnProperty('sum')) {
	Array.prototype.sum = function () {
		for (var total = 0, l = this.length; l--; total += this[l]);

		return total;
	}
}

/**
 * Method for retrieving all "data-*" attributes of an element.
 * Use like: $('element').getDataAttributes(); => {'const': 'C__ABC', 'action': 'xyz'}
 *
 * @returns  object
 * @see      http://stackoverflow.com/questions/3762589/fastest-javascript-summation
 */
if (!Element.prototype.hasOwnProperty('getDataAttributes')) {
    Element.prototype.getDataAttributes = function () {
        var i, attributes = {};

        for (i in this.attributes) {
            if (Object.isString(this.attributes[i].localName) && this.attributes[i].localName.startsWith('data-')) {
                attributes[this.attributes[i].localName.substring(5)] = this.attributes[i].nodeValue;
            }
        }

        return attributes;
    }
}

/**
 * Function to repeat a given string n-times.
 *
 * @param   count
 * @return  string
 */
if (!String.prototype.hasOwnProperty('repeat')) {
	String.prototype.repeat = function (count) {
		if (count < 1) return '';
		var result = '', pattern = this.valueOf();
		while (count > 1) {
			if (count & 1) result += pattern;
			count >>= 1, pattern += pattern;
		}
		return result + pattern;
	};
}

if (!String.prototype.hasOwnProperty('encodeHTML')) {
	String.prototype.encodeHTML = function () {
		return this
			.replace(/</g, '&lt;').replace(/>/g, '&gt;')
			.replace(/ä/g, '&auml;').replace(/ö/g, '&ouml;').replace(/ü/g, '&uuml;')
			.replace(/Ä/g, '&Auml;').replace(/Ö/g, '&Ouml;').replace(/Ü/g, '&Uuml;');
	};
}

if (!String.prototype.hasOwnProperty('decodeHTML')) {
	String.prototype.decodeHTML = function () {
		return this
			.replace(/&lt;/g, '<').replace(/&gt;/g, '>')
			.replace(/&auml;/g, 'ä').replace(/&ouml;/g, 'ö').replace(/&uuml;/g, 'ü')
			.replace(/&Auml;/g, 'Ä').replace(/&Ouml;/g, 'Ö').replace(/&Uuml;/g, 'Ü');
	};
}

if (!String.prototype.hasOwnProperty('toInt')) {
    String.prototype.toInt = function (base) {
        return parseInt(this, base || 10);
    };
}

if (!String.prototype.hasOwnProperty('toFloat')) {
    String.prototype.toFloat = function () {
        return parseFloat(this);
    };
}

function move_arrow_to(element) {
	if (element) {
		$('downarrow').morph('left: ' + ((parseInt(element.positionedOffset().left) + parseInt($('mainMenu').up().offsetLeft)) + 45) + 'px', {
			duration: 0.2
		});
	}
}

function load_ndo_state_in_list(el) {
	var obj_id = el.up('tr').readAttribute('data-objectid'),
		url = el.readAttribute('data-url'),
		td = el.up('td').update(new Element('img', {className:'vam mr5', src: window.dir_images + 'ajax-loading.gif'}));

	new Ajax.Request(url, {
		method: "post",
		parameters: {
			objID:obj_id
		},
		onSuccess: function(transport) {
			var json = transport.responseJSON;

			if (json.success && json.data) {
				td.update(display_monitoring_state(json.data));
			} else {
				td.addClassName('text-red').update(json.message || '-');
			}
		}.bind(this)
	});
}

/**
 * This method loads the Check_MK livestatus state of a single object.
 *
 * @param   el
 * @author  Leonard Fischer <lfischer@i-doit.com>
 */
function load_livestatus_state_in_list(el) {
	var obj_id = el.up('tr').readAttribute('data-objectid'),
		url = el.readAttribute('data-url'),
		td = el.up('td').update(new Element('img', {className:'vam mr5', src: window.dir_images + 'ajax-loading.gif'}));

	new Ajax.Request(url, {
		method: "post",
		parameters: {
			objID:obj_id
		},
		onSuccess: function(transport) {
			var json = transport.responseJSON,
				img = new Element('img', {className: 'ml5 vam'});

			if (json.success && json.data) {
				td.update(display_monitoring_state(json.data));
			} else {
				td.addClassName('text-red').update(json.message || '-');
			}
		}.bind(this)
	});
}

/**
 * This function shall be used for a unified monitoring state output.
 *
 * @param    data
 * @param    el
 * @returns  {HTMLElement}
 */
function display_monitoring_state (data, el) {
	var $el = el;

	if (Object.isUndefined($el)) {
		$el = new Element('p');
	}

	if (data === null) {
		return $el
	}

	// "host_state" has a higher priority than the normal "state".
	if (data.hasOwnProperty('host_state')  && data.host_state !== null) {
		$el.addClassName(data.host_state.color || '')
			.update(new Element('span', {className:'vam'}).update(data.host_state.state));

		if (data.host_state.hasOwnProperty('icon')) {
			$el.insert({top:new Element('img', {className:'vam mr5', src: window.dir_images + data.host_state.icon})});
		}
	} else if (data.hasOwnProperty('state') && data.state !== null) {
		$el.addClassName(data.state.color || '')
			.update(new Element('span', {className:'vam'}).update(data.state.state));

		if (data.state.hasOwnProperty('icon')) {
			$el.insert({top:new Element('img', {className:'vam mr5', src: window.dir_images + data.state.icon})});
		}
	} else if (data.state === null || data.host_state === null) {
		$el.update(new Element('img', {className:'vam mouse-help', src: window.dir_images + 'icons/silk/help.png', title:'No data found...'}));
	}

	return $el;
}

/**
 * Checks if the given response is actually a JSON response.
 *
 * @param    response
 * @param    notify
 * @returns  boolean
 * @author   Leonard Fischer <lfischer@i-doit.com>
 */
function is_json_response(response, notify) {
	var is_json = response.hasOwnProperty('responseJSON') && ! Object.isUndefined(response.responseJSON);

	// Display a i-doit "notify" box, when the second parameter is set to something true and the response is not a json response!
	if (!! notify && !is_json) {
		idoit.Notify.error('The request did not answer with proper JSON:' + response.responseText, {sticky:true});
	}

	return is_json;
}

/**
 * i-doit
 *
 * JavaScript function onlyNumeric
 *
 * @author     Dennis Stücken <dstuecken@i-doit.org>
 * @version    0.9
 * @copyright  synetics GmbH
 * @license    http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @return     string  With nothing else than numbers
 */

function onlyNumeric(sText){
   return sText.replace(/[^0-9:]/g, "");
}

function isNumeric(x) {
	return (x!=null && !isNaN(x));
}


/**
 * Javascript to test an IPv6 address for proper format, and to
 * present the "best text representation" according to IETF Draft RFC at
 * http://tools.ietf.org/html/draft-ietf-6man-text-addr-representation-04
 *
 * 8 Feb 2010 Rich Brown, Dartware, LLC
 * Please feel free to use this code as long as you provide a link to
 * http://www.intermapper.com
 *
 * @return  boolean
 */
function checkipv6(str) {
	return (/^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$/.test(str));
}


/**
 * Disables the save button.
 *
 * @author Benjamin Heisig <bheisig@i-doit.org>
 */
function isys_glob_disable_save() {
    var e = $('navbar_item_C__NAVMODE__SAVE');
	if (e) {
        e.className = 'navbar_item_inactive';
        var onClick = e.readAttribute('onclick');
        if (onClick) {
            e.writeAttribute('data-onclick', onClick);
            e.writeAttribute('onclick', null);
        }
    }
}

/**
 * Enables the save button.
 *
 * @author Benjamin Heisig <bheisig@i-doit.org>
 */
function isys_glob_enable_save() {
    var e = $('navbar_item_C__NAVMODE__SAVE');
	if (e) {
        var onClick = e.readAttribute('data-onclick');
        if (onClick) {
            e.className = 'navbar_item';
            e.writeAttribute('onclick', onClick);
            e.writeAttribute('data-onclick', null);
        }
    }
}

/**
 * Converts ip address to numeric value
 *
 * @deprecated  Please use "IPv4.ip2long();"
 * @param   ip_address
 * @return  long
 * @author  Van Quyen Hoang <qhoang@i-doit.org>
 */
function ip2long(ip_address) {
    var ip_arr = ip_address.split('.'),
	    iplong = 0;

    with (Math) {
        iplong = ip_arr[0] * pow(256, 3) + ip_arr[1] * pow(256, 2) + ip_arr[2] * pow(256, 1) + ip_arr[3] * pow(256, 0);
    }

    return iplong;
}

/**
 * Converts numeric value to ip address
 *
 * @deprecated  Please use "IPv4.long2ip();"
 * @param   iplong
 * @return  string
 * @author  Van Quyen Hoang <qhoang@i-doit.org>
 */
function long2ip(iplong) {

    with (Math) {
        var ip1 = floor(iplong / pow(256,3));
        var ip2 = floor((iplong % pow(256,3)) / pow(256,2));
        var ip3 = floor(((iplong % pow(256,3)) % pow(256,2)) / pow(256,1));
        var ip4 = floor((((iplong % pow(256,3)) % pow(256,2)) % pow(256,1)) / pow(256,0));
    }
    return ip1 + '.' + ip2 + '.' + ip3 + '.' + ip4;
}

/**
 * This function validates a given IPv4 address.
 *
 * @deprecated  Please use "IPv4.validate();"
 * @param   ip
 * @return  boolean
 * @author  Leonard Fischer <lfischer@i-doit.com
 */
function validate_ipv4 (ip) {
	var parts = ip.split('.'),
		bad_ip = false;

	if (parts.length != 4) bad_ip = true;
	if (! (parts[0] >= 0 && parts[0] <= 255)) bad_ip = true;
	if (! (parts[1] >= 0 && parts[1] <= 255)) bad_ip = true;
	if (! (parts[2] >= 0 && parts[2] <= 255)) bad_ip = true;
	if (! (parts[3] >= 0 && parts[3] <= 255)) bad_ip = true;

	return ! bad_ip;
}

/**
 * This method handles the purging action. If a single object shall be purged, an AJAX request will check if it's got any relations.
 * If your purge more than one object, there will be a javascript "confirm" with a certain message.
 *
 * @author  Leonard Fischer <lfischer@i-doit.org>
 * @return  boolean
 */
function purge_object(navmode, ajax, message) {
	var objects = list_selection();

	// If we selected no objects, we return immediately.
	if (objects.length == 0) {
		return false;
	}

	// When we have got one or more objects, we display a fancy little list with all the objects and their relations.
	var jsonClient = new idoit.JSONClient();
	jsonClient.getRelationsByObjectId(
		objects.join(';'),
		function(response) {
			var json = response.responseJSON,
				output = '';

			if (response.responseText != '[]') {
				output += "\n" + json.join("\n");

				if (confirm(message + "\n" + output)) {
					document.isys_form.navMode.value = navmode;
					replace_listSelection();
					if (ajax) {
						form_submit();
					} else {
						$('isys_form').submit();
					}
				}
			} else {
				document.isys_form.navMode.value = navmode;
				replace_listSelection();
				if (ajax) {
					form_submit();
				} else {
					$('isys_form').submit();
				}
			}
		},
		{});
}

/**
 * updates DialogList hidden field
 *
 * @param   p_fromID
 * @param   p_toID
 * @return  void
 * @author  Selcuk Kekec <skekec@i-doit.org>
 */
function updateDialogList(p_fromID, p_toID) {
    $(p_toID).value = $(p_fromID).getValue().join(',');
}

/**
 * This method triggers overwriteIDs() of the object_list
 * to replace the values of the checkboxes by hidden fields
 *
 * @author Selcuk Kekec
 * @author Leonard Fischer <lfischer@i-doit.com>
 * @returns array
 */
function list_selection() {
    return $A($N('table_ids[]')).filter(function ($input) {
        return $input.checked
    }).invoke('getValue');
    // return $('scroller').select('[name="table_ids[]"]:checked').invoke('getValue');
    //return $('scroller').select('.mainTableCheckbox input').filter(function($input) {return $input.checked;}).invoke('getValue');
}

/**
 * This Method unchecks all checkboxes in
 * the existing table and replace it by
 * hidden fields.
 *
 * @param p_id ID of container element
 * @returns String HiddenFields
 */
function replace_listSelection(id) {
    var $container = $(id || 'scroller'),
        selection = list_selection(), i;

    $container.select('.entry-selection').invoke('remove');

    for (i in selection) {
        if (selection.hasOwnProperty(i)) {
            $container.insert(new Element('input', {type:'hidden', className:'entry-selection', name:'id[]', value:selection[i]}));
        }
    }
}

/**
 * Store the List-Selection in
 * an object for merging
 *
 * @see form_submit() the last parameter
 *
 * @returns {get_listSelection4Submit.l_selection}
 */
function get_listSelection4Submit() {
    var l_result = [];
    var l_selection = list_selection();
    var l_selection4Submit =  {};
    if (l_selection.length) l_result['id[]'] = l_selection;

    return l_result;
}

/**
 * Gets current page from the list
 */
function get_current_page() {
    if($('current_page'))
    {
        return $('current_page').value;
    }
    if($('data-grid-mainList-page-current'))
    {
        return $('data-grid-mainList-page-current').value;
    }
    return 1;
}

/**
 * Gets current filter from the list
 */
function get_current_filter(){
    if($('filter'))
    {
        return $('filter').value;
    }
    if($('data-grid-mainList-filter-data'))
    {
        return $('data-grid-mainList-filter-data').value;
    }
    return '';
}

/**
 * Delays a function call by the given amount of milliseconds
 *
 * @param fn callback
 * @param int ms
 */
var delay = (function () {
	var timer = 0;
	return function (callback, ms) {
		clearTimeout(timer);
		timer = setTimeout(callback, ms);
	};
})();

/**
 * This little function lets you build up a queue for "asynchronus functions".
 * Example:
 *
 * queue([
 *   function(callback) {
 *     // Logic...
 *     setTimeout(function() {
 *       callback();
 *     }, 200);
 *   },
 *   function(callback) {
 *     // Logic...
 *     callback();
 *   }
 * ]);
 *
 * @param   funcs
 * @param   scope
 * @author  Krasimir Tsonev
 * @see     http://krasimirtsonev.com/blog/article/7-lines-JavaScript-library-for-calling-asynchronous-functions
 */
var queue = function (funcs, scope) {
	(function next() {
		if (funcs.length > 0) {
			funcs.shift().apply(scope || {}, [next].concat(Array.prototype.slice.call(arguments, 0)));
		}
	})();
};

/**
 * This function is beeing used by "share access" and the multiedit feature.
 *
 * @param  obj_id
 * @param  receiver
 */
window.get_shares_from_object = function get_shares_from_object (obj_id, receiver) {
	if (obj_id.blank()) {
		$(receiver).update(new Element('option', {value: '-1', selected: true}).insert('-'));
	}

	new Ajax.Request('?call=share&func=get_shares_by_obj&ajax=1', {
			parameters: {'objID': obj_id},
			method: "post",
			onComplete: function (response) {
				var i,
					json = response.responseJSON,
					share = $(receiver).update(new Element('option', {value: '-1', selected: true}).insert('-'));

				if (json.success && is_json_response(response)) {
					for (i in json.data) {
						if (json.data.hasOwnProperty(i)) {
							share.insert(new Element('option', {value: json.data[i].id}).update(json.data[i].val));
						}
					}
				} else {
					idoit.Notify.error(json.message || response.responseText, {sticky:true});
				}
			}
		}
	);
};

/**
 * This function is beeing used by "nagios service assignment" for toggling the automated service inheritance.
 *
 * @param  $button
 * @param  host_obj_id
 * @param  service_obj_id
 */
window.toggle_nagios_service_inheritance = function ($button, host_obj_id, service_obj_id) {
	$button
		.removeClassName('text-red')
		.removeClassName('text-green')
		.down('img').writeAttribute('src', window.dir_images + 'ajax-loading.gif')
		.next('span').update(idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__LOADING'));

	new Ajax.Request('?call=nagios&func=toggle_inherited_service&ajax=1', {
			parameters: {"host":host_obj_id, "service":service_obj_id},
			method: "post",
			onComplete: function (response) {
				var json = response.responseJSON;

				if (json.success && is_json_response(response)) {
					if (json.data.will_be_exported) {
						$button.addClassName('text-green')
							.down('img').writeAttribute('src', window.dir_images + 'icons/silk/tick.png')
							.next('span').update(idoit.Translate.get('LC__UNIVERSAL__YES'));
					} else {
						$button.addClassName('text-red')
							.down('img').writeAttribute('src', window.dir_images + 'icons/silk/cross.png')
							.next('span').update(idoit.Translate.get('LC__UNIVERSAL__NO'));
					}
				} else {
					idoit.Notify.error(json.message || response.responseText, {sticky:true});
				}
			}
		}
	);
};

window.toggle_primary_contact = function ($button, cat_row_id, obj_id) {
	$button.up('table').select('button.primary-button')
		.invoke('removeClassName', 'text-green')
		.invoke('addClassName', 'text-red')
		.invoke('down', 'img').invoke('writeAttribute', 'src', window.dir_images + 'icons/silk/cross.png')
		.invoke('next', 'span').invoke('update', idoit.Translate.get('LC__UNIVERSAL__NO'));

	$button.removeClassName('text-red').removeClassName('text-green')
		.down('img').writeAttribute('src', window.dir_images + 'ajax-loading.gif')
		.next('span').update(idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__LOADING'));

	new Ajax.Request('?call=update_contact_primary&ajax=1', {
		parameters: {"id":cat_row_id, "objID":obj_id},
		method: 'post',
		onSuccess: function(transport) {
			var json = transport.responseJSON;

			if (json.success) {
				if (json.data.is_primary) {
					$button.removeClassName('text-red').addClassName('text-green')
						.down('img').writeAttribute('src', window.dir_images + 'icons/silk/tick.png')
						.next('span').update(idoit.Translate.get('LC__UNIVERSAL__YES'));
				} else {
					$button.removeClassName('text-green').addClassName('text-red')
						.down('img').writeAttribute('src', window.dir_images + 'icons/silk/cross.png')
						.next('span').update(idoit.Translate.get('LC__UNIVERSAL__NO'));
				}
			} else {
				$button
					.addClassName('text-red')
					.down('img').writeAttribute('src', window.dir_images + 'icons/silk/error.png')
					.next('span').update('-');

				idoit.Notify.error(json.message);
			}
		}});
};

/**
 * Function for testing on several Fullscreen-API methods.
 * @param   obj
 * @param   method
 * @returns {*}
 */
function run_prefix_method (obj, method) {
	var p = 0, m, t, browser_prefixes = ["webkit", "moz", "ms", "o", ""];

	while (p < browser_prefixes.length && !obj[m]) {
		m = method;

		if (browser_prefixes[p] == "") {
			m = m.substr(0, 1).toLowerCase() + m.substr(1);
		}

		m = browser_prefixes[p] + m;
		t = typeof obj[m];

		if (t != "undefined") {
			browser_prefixes = [browser_prefixes[p]];
			return (t == "function" ? obj[m]() : obj[m]);
		}

		p++;
	}
}

/**
 * This function will try to open the given element in fullscreen mode.
 * @param  $el
 */
function request_fullscreen ($el) {
	if (run_prefix_method(document, "FullScreen") || run_prefix_method(document, "IsFullScreen")) {
		run_prefix_method(document, "CancelFullScreen");
	} else {
		run_prefix_method($el, "RequestFullScreen");
	}
}

/**
 * This method will prepare an JS object (recursively) for preventing assignment errors (obj.x.y.z is undefined).
 * Example:
 *   fillObject({}, 'cmdb.obj.id', 1337);
 * Returns:
 *   {cmdb: {obj: {id: 1337}}}
 *
 * @param    {object}        obj
 * @param    {Array|String}  keys
 * @param    {*}             value
 * @returns  {object}
 * @author   Leonard Fischer <lfischer@i-doit.com>
 */
function fillObject (obj, keys, value) {
	var currentKey;

	if (Object.isString(keys)) {
		keys = keys.split('.');
	}

	currentKey = keys.shift();

	if (! obj.hasOwnProperty(currentKey)) {
		obj[currentKey] = {};
	}

	if (keys.length > 0) {
		obj[currentKey] = fillObject(obj[currentKey], keys, value);
	} else {
		obj[currentKey] = value;
	}

	return obj;
}

/**
 * This function should only be used to display fullscreen-popups (like the NAG-Screen).
 *
 * @param tpl
 * @param params
 */
function openFullscreenPopup (tpl, params) {
	var $popup = $('popup'),
		$overlay = $('overlay'),
		width = $overlay.getWidth() - 100,
		height = $overlay.getHeight() - 100;

	if (width < 200) {
		width = 200;
	}

	if (height < 200) {
		height = 200;
	}

	if (! $popup.visible()) {
		get_popup('fullscreen', '', width, height, {tpl:tpl, parameters:Object.toJSON(params)}, null);
	}
}