/**
 * Static IPv4 class for calculating IPv4s, IPv4 ranges, IPv4 subnets, etc.
 *
 * @author  Leonard Fischer <lfischer@i-doit.com>
 * @author  Van Quyen Hoang <qhoang@i-doit.com>
 */
var IPv4 = {
	/**
	 * Validates the given IP.
	 *
	 * @param   ip
	 * @return  boolean
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	valid_ip: function (ip) {
		if (Object.isString(ip)) {
			var parts = ip.split('.'),
				bad_ip = false;

			if (parts.length != 4) bad_ip = true;
			if (!(parts[0] >= 0 && parts[0] <= 255)) bad_ip = true;
			if (!(parts[1] >= 0 && parts[1] <= 255)) bad_ip = true;
			if (!(parts[2] >= 0 && parts[2] <= 255)) bad_ip = true;
			if (!(parts[3] >= 0 && parts[3] <= 255)) bad_ip = true;

			return !bad_ip;
		}

		return false;
	},

	/**
	 * Converts numeric value to ip address
	 *
	 * @param   iplong
	 * @return  string
	 * @author  Van Quyen Hoang <qhoang@i-doit.com>
	 */
	long2ip: function (iplong) {
		var pow = Math.pow,
			floor = Math.floor;

		return floor(iplong / pow(256, 3)) + '.' +
			floor((iplong % pow(256, 3)) / pow(256, 2)) + '.' +
			floor(((iplong % pow(256, 3)) % pow(256, 2)) / pow(256, 1)) + '.' +
			floor((((iplong % pow(256, 3)) % pow(256, 2)) % pow(256, 1)) / pow(256, 0));
	},

	/**
	 * Converts ip address to numeric value
	 *
	 * @param   ip_address
	 * @return  Number
	 * @author  Van Quyen Hoang <qhoang@i-doit.com>
	 */
	ip2long: function (ip_address) {
		if (this.valid_ip(ip_address)) {
			var ip_arr = ip_address.split('.');

			return (parseInt(ip_arr[0]) * Math.pow(256, 3)) + (parseInt(ip_arr[1]) * Math.pow(256, 2)) + (parseInt(ip_arr[2]) * 256) + parseInt(ip_arr[3]);
		} else {
			return 0;
		}
	}
};