/**
 * i-doit Lists
 */
if (!Lists) var Lists = {};

Lists.Objects = Class.create(TableOrderer, {
	initialize:function (element, options) {
		this.element = element;
		this.options = options;

		this.options = Object.extend({
			data:false, // array of data
			url:false, // url to a JSON application containing the data
			allowMultiselect:true, // don't work yet
			unsortedColumn:[], // array of column you don't want to sort
			dateFormat:'d', // d|m ; d => dd/mm/yyyy; m => mm/dd/yyyy
			filter:false, // show Filter Option. false | 'top' | 'bottom'
			pageCount:5, // Number of items by table
			paginate:false, // show Paginate Option. false | 'top' | 'bottom'
			search:false, // show Gloabal Search . falses | 'top' | 'bottom'
			classPrefix:'mainTable',
			draggable:false, // Define, if the single rows shall be draggable.
			checkboxes:true, // Define, if the single rows shall get an checkbox for selecting.
			checkedBoxes: [], // IDs of the checked checkboxes
			instanceName:'', // The name of the actual instance, this is used by .addButton() and .removeButton().
			// The following attributes are used for the ajax paging.
			max_pages:0,
			ajax_pager:false, // Define, if the pager shall be activated.
			ajax_pager_url:'', // Define the URL for the ajax pager.
			ajax_pager_preload:10, // Define the URL for the ajax pager.
			// The following attribut defines, if the "tr"-click shall be activated.
			tr_click:true,
			// The following attribut is needed for ndo state calls.
			ndo_state_url:'',
			ndo_state_field:'',
			// The following attribut is needed for livestatus state calls.
			livestatus_state_url:'',
			livestatus_state_field:'',
            order_direction:'',
            order_field:'',
			// This will be used for saving the "filter" and "filter-field" in the SESSION (via ajax).
			filter_save_url:''
		}, options || {});

		// saves tool state data for pagination, filtering and searching.
		this.tools = {
			page:1, // for pagination
			pages:1, // for pagination
			filterCol:'', // for filtering
			filterData:'', // for filtering
			searchData:'', // for global searching
			requested_offsets:[] // This array will save the previously loaded offset-blocks (only for ajax-pager).
		};

		// separates text messages out
		this.msgs = {
			loading:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__LOADING'),
			emptyResults:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__EMPTY_RESULTS'),
			errorURL:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__ERROR_URL'),
			errorData:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__ERROR_DATA'),
			searchLabel:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__SEARCH_LABEL'),
			filterLabel:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__FILTER_LABEL'),
			paginationOf:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__PAGINATEN_OF'),
			paginationPages:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__PAGINATEN_PAGES'),
			paginationFirst:'&nbsp;&nbsp;&nbsp;&nbsp;',
			paginationPrev:'&nbsp;&nbsp;&nbsp;&nbsp;',
			paginationNext:'&nbsp;&nbsp;&nbsp;&nbsp;',
			paginationLast:'&nbsp;&nbsp;&nbsp;&nbsp;'
		};

		this.cache = [];			// for caching capabilities
		this.isCached = false;		// for caching capabilities
		this.container = $(element);
		this.orderField = false;
		this.order = 'asc';

		this.thClickbfx = this.thClick.bindAsEventListener(this);
		this.thOverbfx = this.thOver.bindAsEventListener(this);
		this.thOutbfx = this.thOut.bindAsEventListener(this);

		this.ndo_loaded = [];
		this.livestatus_loaded = [];

		this.setData();
	},
	// createTable -- creates the inital table being ran just once. It writes out the HTML elements for the table and tools interface (i.e. pagination).
	createTable:function () {
		this.container.update();
		this.container.insert({top:'<table cellspacing="0" cellpadding="0" id="data-grid-' + this.element + '" class="' + this.options.classPrefix + ' mainTableHover"></table>'});
		this.table = $('data-grid-' + this.element);
		this.createTools();
		this.createRows();
		this.addToolsObserver();
		this.addTableObserver();
		this.makeUnsort();

		this.updateNDO();
		this.updateLivestatus();

        /* Focus on the filter-data input for navigating */
        if(this.options.filter)
        {
            $(this.table.id+'-filter-data').focus();
        }

        if (this.options.tr_click) {
            this.table.addClassName('mouse-pointer');
        }
	},

	// Minor addition to the native "addToolsObserver" method.
	addToolsObserver : function($super) {
		var instance = this;

		$super();

		if (this.options.filter && !this.options.filter_save_url.empty()) {
			$(this.table.id + '-filter-data').on('change', this.filterData.bindAsEventListener(this));

			$(this.table.id + '-filter-data').on('blur', function () {
				instance.save_filter_to_session();
			});

			$(this.table.id + '-filter-column').on('change', function () {
				instance.save_filter_to_session();
			});
		}
	},

	// New method to save the current filter data to the session.
	save_filter_to_session: function () {
		new Ajax.Request(this.options.filter_save_url, {
			parameters: {
				field: $F(this.table.id + '-filter-column'),
				value: $F(this.table.id + '-filter-data')
			},
			onComplete: function () {
				 // Do nothing.
            }
		});
	},

	createRow:function (obj, index) {
		var values = Object.values(obj);
		var tr = new Element('tr', {'class':'data line' + (index % 2), id:this.table.id + '-' + index}).writeAttribute('data-objectid', values[0]).writeAttribute('data-listindex', index);
        var checkedStr = (this.options.checkedBoxes.indexOf(parseInt(values[0])) !== -1) ? "checked='checked'" : '';
		// We don't want to display the ID field.
		this.tableColumnsName.each(function (s, index) {
			if (s == '__id__' && this.options.checkboxes) {
				var dragger = '';

				if (this.options.draggable) {
					dragger = '<span class="drag-handle"></span>';
				}

				tr.insert(new Element('td', {
					'className':this.table.id + '-column-' + s,
					'style':'width:30px; text-align:center;'
				}).update(dragger + '<input class="checkbox obj-selector" type="checkbox" value="' + values[index] + '" name="id[]" title="' + values[index] + '" '+checkedStr+'/>'));
			} else {
				tr.insert(new Element('td', {
					'className':this.table.id + '-column-' + s
				}).update(values[index]));
			}
		}.bind(this));

		if (Prototype.Browser.IE && tr.outerHTML) {
			return tr.outerHTML;
		} else {
			return tr;
		}
	},
	createFirstRow:function (obj) {
		var row = '<thead><tr class="no-drag">';

		this.tableColumnsName.each(function (i) {
			if (i == '__id__' && this.options.checkboxes) {
				var checkbox_style = '';
				if (this.options.draggable) {
					checkbox_style = 'style="margin-left:12px;"';
				}

				row += '<th id="' + this.table.id + '-' + i + '" class="no-sort" style="text-align:center;">' +
					'<input class="no-sort" id="complete-list-selector" type="checkbox" ' + checkbox_style + ' /></th>';
			} else {
				if (i == '__id__') {
					i = 'ID';
				}
				row += '<th id="' + this.table.id + '-' + i + '">' + i + '</th>';
			}
		}.bind(this));

		row += '</tr></thead>';
		return row;
	},
	changePage:function (ev) {
		var page = ev.element().value;
		if (page <= this.tools.pages && page > 0) {
			this.tools.page = page;

			if (this.options.ajax_pager) {
				this.load_data();
			}
			this.updateTable();
		} else {
            if (page != '')
            {
                // Revert the page number, if the input was not existent.
                this.updatePager();
            }
		}
		delete page;
	},
	load_data: function() {
		var offset = 0;

		// Calculate the offset.
		offset = parseInt((this.tools.page - 1) / this.options.ajax_pager_preload);

		// We check if this offset has been requested, so we don't load data twice.
		if (! this.tools.requested_offsets.in_array(offset) && offset > 0) {
			this.tools.requested_offsets.push(offset);

			new Ajax.Request(this.options.ajax_pager_url, {
				parameters: {
					'offset_block': offset
				},
				method: "post",
				onSuccess: function(transport) {
					var json = transport.responseJSON,
						index_offset = (offset * this.options.pageCount * this.options.ajax_pager_preload);

					// Faster than the "each" method.
					for (var i = 0, len = json.length; i < len; ++i) {
						this.data[(index_offset + i)] = json[i];
					}

					this.perform();
				}.bind(this)
			});
		}

	},
	createPager:function () {
		if ($(this.table.id + '-pager')) {
			$(this.table.id + '-pager')
				.insert({bottom:new Element('a', {'id':this.table.id + '-page-first', 'class':'first-page-button'}).update(this.msgs.paginationFirst)})
				.insert({bottom:new Element('a', {'id':this.table.id + '-page-prev', 'class':'prev-page-button'}).update(this.msgs.paginationPrev)})
				.insert({
					bottom:'<span id="' + this.table.id + '-page-amount" style="margin-right:6px">' +
						'<input id="' + this.table.id + '-page-current' + '" class="currentpage input" value="' + this.tools.page + '" />' +
						this.msgs.paginationOf + '<span id="' + this.table.id + '-page-total' + '" class="totalpages">' + this.tools.pages + '</span>' + this.msgs.paginationPages +
						'</span>'
				})
				.insert({bottom:new Element('a', {'id':this.table.id + '-page-next', 'class':'next-page-button'}).update(this.msgs.paginationNext)})
				.insert({bottom:new Element('a', {'id':this.table.id + '-page-last', 'class':'last-page-button'}).update(this.msgs.paginationLast)})
			;
		}
		$(this.table.id + '-page-current').on('keydown', function (ev) {
			if (ev.keyCode == Event.KEY_RETURN)
            {
				ev.preventDefault();
			}
		});
		$(this.table.id + '-page-current').on('keyup', this.changePage.bindAsEventListener(this))
	},
	pagerData : function(e, element) {
		var tid = this.table.id;
		var caller = element || Event.element(e);

		switch (caller.id) {
			case tid+'-page-next':
				this.tools.page = ((++this.tools.page) > this.tools.pages) ? --this.tools.page : this.tools.page;
				break;

			case tid+'-page-prev':
				this.tools.page = ((--this.tools.page) > this.tools.pages) ? ++this.tools.page : this.tools.page;
				break;

			case tid+'-page-last':
				this.tools.page = this.tools.pages;
				break;

			default:
				this.tools.page = 1;
		}

		if (this.options.ajax_pager) {
			this.load_data();
		}
		this.updateTable();
	},
	updatePager:function () {
		$(this.table.id + '-page-current').value = this.tools.page;
	},
	updateTable : function() {
		this.table = $('data-grid-'+this.element);
		$(this.table.id).update();
		this.createRows();
		this.addTableObserver();
		this.makeUnsort();
		this.updatePager();

		this.updateNDO();
		this.updateLivestatus();
	},
	updateNDO : function () {
		// At first we check, if the ndo state shall be displayed at all.
		if ($(this.table.id).down('span.ndo_state')) {
			var obj_ids = $$('#' + this.table.id + ' tr').invoke('readAttribute', 'data-objectid').compact();

			// Now we remove all object-IDs which are currently loading or already have loaded: "obj_ids - this.ndo_loaded"
			obj_ids = Array.prototype.without.apply(obj_ids, this.ndo_loaded);

			new Ajax.Request(this.options.ndo_state_url, {
				method: "post",
				parameters:{
					obj_ids: Object.toJSON(obj_ids)
				},
				onSuccess: function(transport) {
					var json = transport.responseJSON,
						item,
						index,
						row_index,
						el;

					if (json.success && json.data !== null) {
						for (index in json.data) {
							if (json.data.hasOwnProperty(index)) {
								item = json.data[index];

								el = $$('tr[data-objectid="' + item.obj_id + '"]');

								if (el[0]) {
									this.ndo_loaded.push(item.obj_id);

									row_index = (this.tools.page - 1) * this.options.pageCount + parseInt(el[0].readAttribute('data-listindex'));

									el = el[0].down('.ndo_state');

									if (! el) {
										continue;
									}

									el = el.up().update(display_monitoring_state(item));

									this.data[row_index][this.options.ndo_state_field] = el.innerHTML;
								}
							}
						}

						$(this.table.id).select('span.ndo_state.loading').invoke('addClassName', 'red').invoke('update', '?');
					} else {
						$(this.table.id).select('span.ndo_state').invoke('addClassName', 'red').invoke('update', (json.message || ''));
					}
				}.bind(this)
			});
		}
	},
	updateLivestatus : function () {
		// At first we check, if the ndo state shall be displayed at all.
		if ($(this.table.id).down('span.livestatus_state')) {
			var obj_ids = $$('#' + this.table.id + ' tr').invoke('readAttribute', 'data-objectid').compact();

			// Now we remove all object-IDs which are currently loading or already have loaded: "obj_ids - this.livestatus_loaded"
			obj_ids = Array.prototype.without.apply(obj_ids, this.livestatus_loaded);

			new Ajax.Request(this.options.livestatus_state_url, {
				method: "post",
				parameters:{
					obj_ids: Object.toJSON(obj_ids)
				},
				onSuccess: function(transport) {
					var json = transport.responseJSON,
						item,
						index,
						row_index,
						el;

					if (json.success && json.data !== null) {
						for (index in json.data) {
							if (json.data.hasOwnProperty(index)) {
								item = json.data[index];

								el = $$('tr[data-objectid="' + item.obj_id + '"]');

								if (el[0]) {
									this.livestatus_loaded.push(item.obj_id);

									row_index = (this.tools.page - 1) * this.options.pageCount + parseInt(el[0].readAttribute('data-listindex'));

									el = el[0].down('.livestatus_state');

									if (! el) {
										continue;
									}

									el = el.up().update(display_monitoring_state(item));

									this.data[row_index][this.options.livestatus_state_field] = el.innerHTML;
								}
							}
						}

						$(this.table.id).select('span.livestatus_state.loading').invoke('addClassName', 'red').invoke('update', '?');
					} else {
						$(this.table.id).select('span.livestatus_state').invoke('addClassName', 'red').invoke('update', (json.message || ''));
					}
				}.bind(this)
			});
		}
	},
	createFilter:function () {
		var option = '',
			objectlink_index;

		this.tableColumnsName.each(function (i) {
			if (i != '__id__') {
				option += '\t<option value="' + i + '">' + i + '</option>\n';
			}
		});

        var inputFilter = new Element('input', {'id':this.table.id + '-filter-data', className:'input input-mini'});

        inputFilter.on('keydown', function (ev) {
			if (ev.keyCode == Event.KEY_RETURN)
            {
				ev.preventDefault();
			}
		});

		if (! this.options.ajax_pager) {
			$(this.table.id + '-options')
				.insert({bottom:this.msgs.filterLabel})
				.insert({bottom:'<select id="' + this.table.id + '-filter-column" class="input input-mini">' + option + '</select>'})
				.insert({bottom:inputFilter});
		} else {
			$(this.table.id + '-options')
				.insert({bottom:this.msgs.filterLabel})
				.insert({bottom:'<select id="' + this.table.id + '-filter-column" disabled="disabled" class="input input-mini">' + option + '</select>'})
				.insert({bottom:new Element('input', {'id':this.table.id + '-filter-data', className:'input input-mini', 'disabled': 'disabled'})});
		}

		// Have we got the "object-link" column?
		if (this.tableColumnsName.in_array(idoit.Translate.get('LC__UNIVERSAL__TITLE_LINK'))) {
			// We use it as default column for the filter.
			objectlink_index = this.tableColumnsName.indexOf(idoit.Translate.get('LC__UNIVERSAL__TITLE_LINK'));

			// If we find the "__id__" column, we have to reduce the index by one.
			if (this.tableColumnsName.in_array('__id__')) objectlink_index --;

			// Now we set the index.
			$(this.table.id + '-options').down('select').selectedIndex = objectlink_index;
		}


		this.tools.filterCol = $F(this.table.id + '-filter-column');
		this.tools.filterData = $F(this.table.id + '-filter-data');
	},
	// addTableObserver -- binds event handlers to the events associated with the table generated.
	addTableObserver:function () {
		var tid = this.table.id,
			that = this;

		if (! this.options.ajax_pager) {
			$(tid).select('th')
				.invoke('observe', 'click', this.thClickbfx)
				.invoke('observe', 'mouseover', this.thOverbfx)
				.invoke('observe', 'mouseout', this.thOutbfx);
		}

		$(tid).select('tr.data')
			.invoke('observe', 'click', this.trClick.bindAsEventListener(this))
			.invoke('observe', 'mouseover', this.trOver.bindAsEventListener(this))
			.invoke('observe', 'mouseout', this.trOut.bindAsEventListener(this));

		// Adding the observer to select all visible checkboxes.
		if ($('complete-list-selector')) {
			$('complete-list-selector').observe('change', function (e) {
				$(tid).select('tr input[name="id[]"]').each(function(es) {es.checked = e.element().checked});

                // Delete all available id's
				that.options.checkedBoxes = [];

                // Check all available table rows
                if (e.target.checked) {
                    // this marks only the current page
                    $(tid).select('tr.data').each(function(ele){
	                    that.options.checkedBoxes.push(parseInt(ele.down('.checkbox').value));
                    });
                }
			});
		}

		if (this.options.draggable) {
			// Enable dragging.
			$(this.table.id).select('tr').each(function (el) {
				if (el.hasClassName('no-drag')) {
					return;
				}

				new Draggable(el, {
					scroll:$('contentWrapper'),
					handle:el.down('span.drag-handle'),
					zindex:500,
					revert:true,
					onStart:function () {
						// Is "dragging" used for something? I took it from the old logic.
						dragging = true;

						// We hide the current table-row, because we are displaying a ghost.
						el.addClassName('draggable');

						// If we already have a ghosting div, remove it!
						if ($('ghost')) {
							$('ghost').remove();
						}

						// We create a new ghosting element.
						var ghost = new Element('table', {id:'ghost', className:that.options.classPrefix})
							.setStyle({position:'absolute', opacity:0.7, width:'50%', zIndex:1000})
							.update(el.clone(true).writeAttribute({id:'ghos-tr'}));

						$('body').insert(ghost);
					},
					onDrag:function () {
						// We are manageing our own ghosting, because prototype fails here.
						if ($('ghost')) {
							$('ghost').setStyle({left: Position.xcomp + 'px', top: Position.ycomp + 'px'});
						}
					},
					onEnd:function () {
						// Is "dragging" used for something? I took it from the old logic.
						dragging = false;

						// We show the current table-row, because our ghost is being remove.
						el.removeClassName('draggable');

						// Finally remove the ghosting div.
						if ($('ghost')) {
							$('ghost').remove();
						}
					}
				});
			});
		}
	},

    // trClick -- event handler for when user clicks on a table row, highlighting the row
    // param e: event that triggered handler
	trClick: function (e) {
		if (this.options.tr_click) {
			var clicked_element = e.findElement(),
				clicked_tag = clicked_element.tagName,
				$tr = clicked_element.up('tr');

			// We don't want to navigate to the object, when we click the drag-handle, the checkbox or any link.
			if (clicked_tag != 'A' && clicked_tag != 'INPUT' && !clicked_element.hasClassName('drag-handle')) {
				if ($tr && window.list_vars && 'tree_view' in window.list_vars) {
					this.objectLink(
						$tr.readAttribute('data-objectid'),
						window.list_vars.tree_view,
						window.list_vars.view_mode,
						'');
				}
			}
		}
	},
	// thClick -- event handler for when the user clicks on the th cell of the table. It orders the table data by that column's field.
	// param e: event that triggered handler.
	thClick:function (e) {
		// Don't sort, if we click a "no sort" field.
		if (e.element().hasClassName('no-sort')) {
			return;
		}

		this.defineOrderField(e);
		this.defineOrder();
		this.orderData(this.orderField);
		if (this.order == 'desc') {
			this.cache = this.cache.reverse();
		}
		this.updateTable();
	},
	removeDataItem:function (obj_id) {
		var tmp_data = [];

		this.data.each(function(i) {
			var arr = Object.values(i);

			if (arr[0] != obj_id) {
				tmp_data.push(i);
			}
		}.bind(this));

		this.data = tmp_data;
		this.clearCache();
	},
	objectLink:function (obj_id, treeview, view_mode, category_id) {
		get_tree_by_object(obj_id, treeview);
		get_content_by_object(obj_id, view_mode, category_id);
	},
	createRows : function() {
		var line = 1;
		var display, enddisplay, startdisplay, dataView, dat, col, searchStr,row, s;

		// header information
		this.table.insert({top: this.createFirstRow()});

		// data -> {filter} -> dataView -> {paginate} -> display
		dataView = this.cache;

		// if filtering is turned off or not currently being used then skip
		if(this.options.filter && !this.isCached && this.tools.filterData !== '') {
			col = this.tools.filterCol;
			dat = this.tools.filterData;

			dataView = this.cache.inject([], function(array, rec) {
				if(rec[col] && (rec[col].toString().toUpperCase().include(dat.toUpperCase()) || rec[col].toString().toUpperCase().include(dat.encodeHTML().toUpperCase()))) {
					array.push(rec);
				}

				return array;
			});
		}

		if (this.options.search && !this.isCached && this.tools.searchData !== '') {
			dat = this.tools.searchData.toUpperCase();

			dataView = dataView.inject([], function (array, value) {
				searchStr = Object.values(value).inject('', function (acc, n) {
					return acc + " " + n;
				});

				if (searchStr.toUpperCase().include(dat)) {
					array.push(value);
				}

				return array;
			});
		}

		display = dataView;

		if (this.options.paginate) {
			this.tools.pages = Math.ceil(dataView.size() / this.options.pageCount);

			// We use "max_pages" when we dynamically reload content (because at this point we can't trust the dataView size).
			if (this.tools.filterData.blank() && this.options.max_pages > 0) {
				this.tools.pages = this.options.max_pages;
			}

			if (this.tools.page > this.tools.pages) {
				this.tools.page = this.tools.pages;
			}
			if (this.tools.page < 1) {
				this.tools.page = 1;
			}
			if (this.tools.pages === 0) {
				this.tools.page = 0;
			}

			$(this.table.id + '-page-current').setValue(this.tools.page);	// Update current page on tool.
			$(this.table.id + '-page-total').update(this.tools.pages);	// Update total pages on tool.

			// Instead of displaying all just display a "paginate window" to the user.
			startdisplay = this.options.pageCount * (this.tools.page - 1);
			enddisplay = this.options.pageCount * this.tools.page;
			display = dataView.slice(startdisplay, enddisplay);
		}

		display.each(function (i, index) {
			this.table.insert({bottom: this.createRow(i, index)});
			line = (line == 1) ? 2 : 1;

			// Add EventListener to the input[checkbox] of the newest row
			if (this.options.checkboxes) {
				$(this.table.id).select('input:last')[0].observe("change", function (e) {
					this.checkListener(e);
				}.bind(this));
			}
		}.bind(this));

		// if there are no results
		if(display.size() === 0) {
			s = this.tableColumnsName.size();
			row = '<tr class="data line0" id="'+this.table.id+'-0">\n';
			if (this.options.ajax_pager) {
                if (typeof window.list_vars === 'undefined') {
                    window.list_vars = {'image_dir': 'images/'};
                }
				// It's very likely that this message will only be displayed when more pages are being loaded.
				row += '\t<td class="'+this.table.id+'-column" colspan="' + s + '"><img src="' + window.list_vars.image_dir + 'ajax-loading.gif" class="vam" /> ' + this.msgs.loading + '</td>\n';
			} else {
				row += '\t<td class="'+this.table.id+'-column" colspan="' + s + '">' + this.msgs.emptyResults + '</td>\n';
			}
			row += '\n</tr>';
			this.table.insert({bottom: row});
		}

		if (this.orderField) {
			$(this.table.id + '-' + this.orderField).addClassName(this.order);
		}

		// the new dataView is set as the cache
		if (!this.isCached) {
			this.isCached = true;
			this.cache = dataView;
		}
	},
	checkListener: function (e) {
		// Add / Remove id from selected rows
		if (e.target.checked && this.options.checkedBoxes.indexOf(parseInt(e.target.value)) == -1) {
			// Add
			this.options.checkedBoxes.push(parseInt(e.target.value));
		} else {
			// Remove
			this.options.checkedBoxes.splice(this.options.checkedBoxes.indexOf(parseInt(e.target.value)), 1);
		}
	},
    filterData: function (e) {
        var tid = this.table.id;

        /* --------------------------------------------------- */
        /* -                Table-Row-Navigator              - */
        /* --------------------------------------------------- */
        if ($(tid))
        {
            if (e.keyCode == Event.KEY_LEFT || e.keyCode == Event.KEY_RIGHT)
            {
                var l_element = (e.keyCode == Event.KEY_LEFT) ? $(this.table.id+'-page-previous') : $(this.table.id+'-page-next');
                if (e.ctrlKey)
                {
                    this.pagerData(null, l_element);
                }

                return false;
            }

            /* Control-key pressed */
            if (e.keyCode == Event.KEY_RETURN || e.keyCode == Event.KEY_DOWN || e.keyCode == Event.KEY_UP)
            {
                var l_selected = $(tid).select('tr.highlightLine');

                if (e.keyCode == Event.KEY_RETURN && l_selected.length)
                {
                    if (l_selected[0])
                    {
                        if (window.list_vars && 'tree_view' in window.list_vars)
                        {
                            this.objectLink(
                                l_selected[0].readAttribute('data-objectid'),
                                window.list_vars.tree_view,
                                window.list_vars.view_mode,
                                ''
                            );
                        }
                        e.stop();
                    }
                    return false;
                }
                else
                {
                    /* Select the first row */
                    if (!l_selected.length)
                    {
                        $(tid).select('tr.data')[0].addClassName('highlightLine');
                    }
                    else
                    {
                        var data = false;
                        l_selected[0].removeClassName('highlightLine');
                        if (e.keyCode == Event.KEY_DOWN)
                        {
                            data = l_selected[0].next('tr.data');
                        }
                        else if (e.keyCode == Event.KEY_UP)
                        {
                            data = l_selected[0].previous('tr.data');
                        }

                        if (data)
                        {
                            data.addClassName('highlightLine');
                        }
                    }
                }

                return false;
            }
        }
        /* --------------------------------------------------- */

	    // We use the "delay" function, because we don't want the logic to be triggered by every key press.
	    delay(function (e) {
		    var caller = Event.element(e);

            // Update state information for filter tool.
            this.tools.filterData = $F(tid + '-filter-data');
            this.tools.filterCol = $F(tid + '-filter-column');

		    if (caller.id == tid + '-filter-column' && this.tools.filterData.blank()) {
			    // If we are just changing the column option and we had not filtered previously then just update the column info.
			    return;
		    }

		    /*
		    // LF: What for ...?
		    if (caller.id == tid + '-filter-column') {
			    // Clear filter data when changing which column we are filtering on.
			    $(tid + '-filter-data').clear();
			    this.tools.filterCol = $F(tid + '-filter-column');
			    this.tools.filterData = '';
		    }
		    */

            // Anytime we filter there is a good chance our data view will change.
            this.clearCache();

		    if (this.options.paginate) {
			    this.tools.page = 1;
			    this.updateTable();

			    return;
		    }

		    $(tid).select('td.' + tid + '-column-' + this.tools.filterCol).each(function (i) {
			    var str = i.innerHTML.toUpperCase(),
				    search = this.tools.filterData;

			    if (!str.include(search.toUpperCase()) && !str.include(search.encodeHTML().toUpperCase())) {
				    i.ancestors()[1].hide();
			    } else {
				    i.ancestors()[1].show();
			    }
		    });
	    }.bind(this, e), 250);
	},
    // Order rules
    orderRule : function (s) {
        if(typeof(s) != 'number'){
            var dateRE = /^(\d{2})[\/\-\. ](\d{2})[\/\-\. ](\d{4}|\d{2})/;
            var exp=new RegExp(dateRE);
            if (exp.test(s)) {
                s = this.options.dateFormat == 'd' ? s.replace(dateRE,"$3$2$1") : s.replace(dateRE,"$3$1$2");
            } else{
                var ipRE = new RegExp(/(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}/g);
                if(ipRE.test(s))
                {
                    s = IPv4.ip2long(s);
                }
                else
                {
                    // it is not a date or an ip
                    var htmlRE = /<a[^>]*>(.*?)<\/a>/i;
                    var matches = htmlRE.exec(s);

                    if(matches != null && matches.length > 1){
                        return matches[1].toLowerCase();
                    }
                }
            }
        }
        return s;
	},

	overwriteIDs: function () {
		var l_length = this.options.checkedBoxes.length,
			l_selectorContainer = null,
			l_hiddenFields = "";

		// Uncheck all checkboxes.
		while (l_length--) {
			l_selectorContainer = $$('input[type="checkbox"][value="' + this.options.checkedBoxes[l_length] + '"]');

			if (l_selectorContainer.length) {
				l_selectorContainer[0].checked = false;
			}

			l_hiddenFields += "<input type='hidden' name='id[]' value='" + this.options.checkedBoxes[l_length] + "' />";
		}

		$('data-grid-' + object_list.element).select('input:checked').each(function (s) {
			s.checked = false;
		});

		this.options.checkedBoxes = [];

		return l_hiddenFields;
	}
});
