/**
 * Static placeholder class for matching, replacing and generating placeholder.
 *
 * @author  Leonard Fischer <lfischer@i-doit.com>
 */
Placeholder = {
	counter_regex:/##COUNT(:-?\d+)?(:\d+)?##/g,
	iteration: 0,

	/**
	 * Finds and replaces the ##COUNT## placeholder (including parameters)
	 *
	 * @param   string
	 * @param   iteration  Should be the Number, of the current iteration.
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 */
	process_counter_string: function (string, iteration) {
		if (!this.counter_regex.test(string)) {
			return string;
		}

		if (!iteration) {
			iteration = this.iteration++;
		}

		iteration = parseInt(iteration);

		return string.replace(this.counter_regex, function (match, start, zeroes) {
			var result,
				minus;

			if (!start) {
				start = 0;
			} else {
				start = parseInt(start.substr(1));
			}

			result = iteration + start;
			minus = (result < 0);

			if (zeroes) {
				if (minus) {
					result = Math.abs(result);
				}

				zeroes = parseInt(zeroes.substr(1)) + 1;

				if (zeroes > (result + '').length) {
					zeroes = '0'.repeat(zeroes);
					result = (zeroes + result).slice(-zeroes.length);
				}

				if (minus) {
					result = '-' + result;
				}
			}

			return result;
		});
	}
};