"use strict";
if (!idoit) var idoit = {};

idoit.Require = {
    // Configuration
    params: {
        paths: {},     // Modules paths: { moduleName: 'path/to/file.js'  }
        debug: false   // Debug output to console
    },
    // Register of modules that are already loaded
    loaded: [],

    /**
     * Confige paths
     *
     * @param config object
     */
    config: function (config) {
        this.params = config;
        return this;
    },

    /**
     * Register new module
     *
     * @param moduleName string
     * @param script string
     */
    addModule: function(moduleName, script) {
        this.params.paths[moduleName] = script;
    },

    /**
     * Loads modules as they are needed
     *
     * @param modulesToLoad string|array Modules that are required
     * @param onload func Function callback that is applied after modules were loaded
     */
    require: function (modulesToLoad, onload /* = null */) {
        var i;

        // Transform "modules" into array if it's a string
        if (typeof modulesToLoad === "string")
        {
            modulesToLoad = [modulesToLoad];
        }

        // Find modules that aren't loaded already
        for (i = 0; i < modulesToLoad.length; ++i)
        {
            if (this.loaded.indexOf(modulesToLoad[i]) !== -1)
            {
                modulesToLoad.splice(i, 1);
                i -= 1;
            }
        }

        // Modules already loaded?
        if (!this.modulesLoaded(modulesToLoad, onload))
        {
            for (i = 0; i < modulesToLoad.length; ++i)
            {
                var moduleName = modulesToLoad[i];

                if (this.params.paths.hasOwnProperty(moduleName))
                {
                    var e = document.createElement("script");
                    e.setAttribute('data-module', moduleName);
                    e.src = this.params.paths[moduleName];
                    e.async = true;

                    e.onload = this.onLoad.bind(this, moduleName, i, modulesToLoad, onload);

                    document.getElementsByTagName("head")[0].appendChild(e);
                }
            }
        }
    },

    /**
     * Onload helper
     *
     * @param moduleName string
     * @param moduleIndex int
     * @param modulesToLoad array
     * @param onload func
     * @param ev Event
     */
    onLoad: function (moduleName, moduleIndex, modulesToLoad, onload, ev) {
        if (this.params.debug) console.log('Module loaded: ' + moduleName + ' (' + Math.round(ev.timeStamp / 1000) + 'ms; '+ ev.path[0].src +')');

        // Remove module from register
        modulesToLoad.splice(modulesToLoad.indexOf(moduleName), 1);

        // Register loaded module
        this.loaded.push(moduleName);

        this.modulesLoaded(modulesToLoad, onload);
    },

    /**
     * Check if modules are loaded and fire callback
     *
     * @param modulesToLoad array
     * @param onloadCallback func
     * @returns {boolean}
     */
    modulesLoaded: function (modulesToLoad, onloadCallback) {
        if (modulesToLoad.length === 0)
        {
            if (this.params.debug) console.log('Modules loaded:', this.loaded);

            if (onloadCallback)
            {
                // Call callback directly since all modules already loaded
                onloadCallback();
            }

            return true;
        }

        return false;
    }
};