<?php
namespace idoit\Module\Search\Query;

/**
 * i-doit
 *
 * Search index condition
 *
 * @package     idoit\Module\Search\Index
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.7
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class Condition
{

    /**
     * @var string
     */
    protected $condition;

    /**
     * Search keyword
     *
     * @var string
     */
    protected $keyword;

    /**
     * Negate this condition?
     *
     * @var bool
     */
    protected $negation;

    /**
     * Do a fuzzy search on this keyword?
     *
     * Only shows exact matches if set to true.
     *
     * @var bool
     */
    protected $fuzzySearch = false;

    /**
     * @return string
     */
    public function getKeyword()
    {
        return $this->keyword;
    }

    /**
     * @return string
     */
    public function getCondition()
    {
        return $this->condition;
    }

    /**
     * @return bool
     */
    public function isNegation()
    {
        return $this->negation;
    }

    /**
     * @return bool
     */
    public function isFuzzySearch()
    {
        return $this->fuzzySearch;
    }

    /**
     * Condition constructor.
     *
     * @param        $keyword
     * @param string $condition
     * @param bool   $negation
     * @param bool   $fuzzySearch
     */
    public function __construct($keyword, $condition = 'AND', $negation = false, $fuzzySearch = false)
    {
        $this->keyword     = $keyword;
        $this->condition   = $condition;
        $this->negation    = $negation;
        $this->fuzzySearch = $fuzzySearch;
    }

}