<?php
namespace idoit\Module\Search\Query\Engine\Mysql;

use idoit\Model\Dao\Base;
use idoit\Module\Search\Query\Condition;
use idoit\Module\Search\Query\Engine\AbstractQuery;
use idoit\Module\Search\Query\Protocol\Query as QueryProtocol;
use idoit\Module\Search\Query\QueryResult;
use isys_component_database as Database;

/**
 * i-doit
 *
 * MySQL Search Query
 *
 * @package     idoit\Module\Search\Index
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.7
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class Query
    extends AbstractQuery
    implements QueryProtocol
{

    /**
     * @var Base
     */
    private $dao;

    /**
     * @param Condition[] $conditions
     *
     * @return QueryResult
     */
    public function search(array $conditions)
    {
        $matchers        = $matchers2 = [];
        $result          = new QueryResult($conditions);
        $ftsTermSanitize = '+ -><()~*:""&|'; // result of "show global variables like 'ft_boolean_syntax';"

        foreach ($conditions as $condition)
        {
            foreach (explode(' ', $condition->getKeyword()) as $keyword)
            {
                $matchConditions = '';

                // Fixing error "syntax error, unexpected $end, expecting FTS_TERM or FTS_NUMB or '*'" with str_replace and rtrim
                $term = trim(
                    ltrim(
                        rtrim(
                            str_replace(
                                [
                                    '-*', '**'
                                ], '*', $keyword
                            ), $ftsTermSanitize
                        ), $ftsTermSanitize
                    )
                );
                $matchConditions .= $term . '*';

                if ($condition->isNegation() || $keyword[0] === '-')
                {
                    $matchConditions = '-' . $matchConditions . '';
                }
                else
                {
                    $matchConditions = '+' . $matchConditions . '';
                }

                $matchers[] = $matchConditions;

                $matchConditions = ' >"' . $term . '"';

                $matchers2[] = $matchConditions;

            }
        }

        if (count($matchers))
        {
            $matching = '(MATCH(isys_search_idx__value) AGAINST (' .
                $this->dao->convert_sql_text(
                    implode(' ', $matchers) . implode(' ', $matchers2)
                ) .
                ' IN BOOLEAN MODE))';

            $sql = 'SELECT ' .
                'isys_search_idx__type AS type, isys_search_idx__key AS searchKey, ' .
                '(CASE isys_obj__title WHEN isys_search_idx__value THEN isys_search_idx__value ELSE CONCAT(isys_obj__title, ": ", isys_search_idx__value) END) AS searchValue, '.
                'isys_search_idx__reference AS searchReference, ' .
                $matching . ' as searchScore ' .
                'FROM isys_search_idx LEFT JOIN isys_obj ON isys_obj__id = isys_search_idx__reference WHERE ' .
                $matching . ' ' .
                'HAVING searchScore > 0 '.
                'ORDER BY searchScore DESC LIMIT ' . \isys_tenantsettings::get('search.limit', '2500') .
                ';';

            $daoResult = $this->dao->retrieve(
                $sql
            );

            while ($row = $daoResult->get_row())
            {
                foreach ($result->getConditions() as $condition)
                {
                    if (!$condition->isFuzzySearch() && !stristr($row['searchValue'], $condition->getKeyword()))
                    {
                        continue 2;
                    }
                }

                $result->addItem(
                    $this->getQueryItemInstance(
                        $row['type'], $row['searchReference'], $row['searchKey'], $row['searchValue'], $row['searchScore']*1.5, $conditions
                    )
                );
            }
        }

        return $result;
    }

    /**
     * Mysql constructor.
     *
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->dao = new Base($database);
    }

}