<?php
/**
 * i-doit - Updates
 * Cleanup script which removes only duplicate identifiers
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_comp_database_system;

$g_migration_identifier = "migration_cleanup_identifier";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Cleanup has already been executed.</span>";
}
else
{
	try
	{
		$l_dao = new isys_cmdb_dao_category_g_identifier($g_comp_database);

		$l_sql = 'SHOW CREATE TABLE isys_catg_identifier_list;';
		$l_res = $l_dao->retrieve($l_sql);
		$l_row = $l_res->get_row();
		$l_execute_migration = true;

		foreach($l_row AS $l_content)
		{
			if(strpos($l_content, 'identifier_unique_index'))
			{
				// Do not set unique index its already been set
				$l_execute_migration = false;
				break;
			}
		}

		if($l_execute_migration)
		{
			$l_sql = 'SELECT isys_catg_identifier_type__id AS typ_id FROM isys_catg_identifier_type';
			$l_res = $l_dao->retrieve($l_sql);
			$l_id_pairs = array();
			$l_duplicate_amount = 0;

			while ($l_row = $l_res->get_row())
			{
				$l_sql = 'SELECT GROUP_CONCAT(isys_catg_identifier_list__id) AS ids, GROUP_CONCAT(isys_catg_identifier_list__isys_obj__id) AS objids,
				COUNT(isys_catg_identifier_list__value) AS cnt, GROUP_CONCAT(isys_catg_identifier_list__datetime) AS crt FROM isys_catg_identifier_list
				WHERE isys_catg_identifier_list__isys_catg_identifier_type__id = ' .
						 $l_dao->convert_sql_id($l_row['typ_id']) . '
					GROUP BY isys_catg_identifier_list__value HAVING cnt > 1;';

				$l_res2 = $l_dao->retrieve($l_sql);
				$l_amount = $l_res2->num_rows();

				if ($l_amount)
				{
					while ($l_row2 = $l_res2->get_row())
					{
						$l_id_pairs[] = array(explode(',', $l_row2['ids']), explode(',', $l_row2['objids']), explode(',', $l_row2['crt']));
					} // while
				} // if
			} // while

			if (count($l_id_pairs) > 0)
			{
				$g_migration_log[] = '<span class="bold">Starting cleanup of the identifiers.</span>';
				$l_delete = array();
				foreach ($l_id_pairs AS $l_pair)
				{
					$l_objIDs = $l_pair[1];
					$l_identIDs = $l_pair[0];
					$l_dates = $l_pair[2];

					foreach ($l_objIDs AS $l_key => $l_obj_id)
					{
						$l_sql = 'SELECT isys_obj__status FROM isys_obj WHERE isys_obj__id = ' .
								 $l_dao->convert_sql_id($l_obj_id) . ';';
						if ($l_dao->retrieve($l_sql)->get_row_value('isys_obj__status') != C__RECORD_STATUS__NORMAL)
						{
							$l_delete[] = $l_identIDs[$l_key];
							unset($l_identIDs[$l_key]);
							unset($l_dates[$l_key]);
						} // if
					} // foreach

					// if we have more than one identifier for the same value then we have to delete all identifiers from this pair except the newest entry
					if (count($l_identIDs) > 1)
					{
						$l_date = 0;
						$l_ignore_key = 0;
						foreach($l_dates AS $l_key => $l_val)
						{
							if($l_date <= strtotime($l_val))
							{
								$l_date = strtotime($l_val);
								$l_ignore_key = $l_key;
							} // if
						} // foreach

						foreach($l_identIDs AS $l_key => $l_val)
						{
							if($l_ignore_key == $l_key)
							{
								continue;
							}
							else
							{
								$l_delete[] = $l_val;
							} // if
						} // foreach
					} // if
				} // foreach
				// Delete all duplicates
				$l_delete_sql = 'DELETE FROM isys_catg_identifier_list WHERE isys_catg_identifier_list__id IN (' .
								implode(',', $l_delete) . ');';

				if ($l_dao->update($l_delete_sql))
				{

					$g_migration_log[] = '<span class="bold">' . count($l_delete) .
										 ' entries removed from the identifier category.</span>';
					$l_dao->apply_update();
				}
				else
				{
					$g_migration_log[] = "<span class=\"bold red\">Cleanup failed.</span>";
				} // if
				$g_migration_log[] = "<span class=\"bold\">Cleanup successful.</span>";
			}
			else
			{
				$g_migration_log[] = "<span class=\"bold\">No cleanup neccessary skipping migration.</span>";
			}

			// Set Unique Index
			$l_dao->update('ALTER TABLE  isys_catg_identifier_list ADD UNIQUE INDEX `identifier_unique_index`
                    (isys_catg_identifier_list__isys_catg_identifier_type__id, isys_catg_identifier_list__key, isys_catg_identifier_list__value);');
			$l_dao->apply_update();

			$g_migration_log[] = "<span class=\"bold\">Unique Index 'identifier_unique_index' for table 'isys_catg_identifier_list' added.</span>";
		}
		else
		{
			$g_migration_log[] = "<span>No cleanup neccessary skipping migration.</span>";
		}
		// Set flag in db.
		$this->migration_done($g_migration_identifier);
	}
	catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
} // if