<?php
/**
 * i-doit - Updates
 *
 * Migration of stor manufacturers / models.
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Leonard Fischer <lfischer@i-doit.com>
 */

// Initialization
global $g_comp_database;

$g_migration_identifier = "migration_stor_manufacturer_model";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = '<span class="bold">Migration of the stor manufacturers / models has already been executed.</span>';
}
else
{
	try
	{
		$l_mapping = $l_manufacturer_cache = $l_model_cache = array();
		$l_dao = new isys_cmdb_dao($g_comp_database);

		// Collect all stor manufacturers:
		$l_res = $l_dao->retrieve('SELECT * FROM isys_stor_manufacturer;');

		if (count($l_res))
		{
			while ($l_row = $l_res->get_row())
			{
				$l_manufacturer_in_catg_stor = $l_dao->retrieve('SELECT isys_catg_stor_list__id
					FROM isys_catg_stor_list
					WHERE isys_catg_stor_list__isys_stor_manufacturer__id = ' . $l_dao->convert_sql_id($l_row['isys_stor_manufacturer__id']) . '
					LIMIT 1;');

				// We remove unused manufacturers.
				if (! count($l_manufacturer_in_catg_stor))
				{
					$l_dao->update('DELETE FROM isys_stor_manufacturer WHERE isys_stor_manufacturer__id = ' . $l_dao->convert_sql_id($l_row['isys_stor_manufacturer__id']) . ';');
					continue;
				} // if

				$l_manufacturer_cache[$l_row['isys_stor_manufacturer__id']] = $l_row;
			} // while
		} // if

		// Iterate over every model:
		$l_res = $l_dao->retrieve('SELECT * FROM isys_stor_model;');

		if (count($l_res))
		{
			while ($l_row = $l_res->get_row())
			{
				$l_model_in_catg_stor = $l_dao->retrieve('SELECT isys_catg_stor_list__id
					FROM isys_catg_stor_list
					WHERE isys_catg_stor_list__isys_stor_model__id = ' . $l_dao->convert_sql_id($l_row['isys_stor_model__id']) . '
					LIMIT 1;');

				// We remove unused models.
				if (! count($l_model_in_catg_stor))
				{
					$l_dao->update('DELETE FROM isys_stor_model WHERE isys_stor_model__id = ' . $l_dao->convert_sql_id($l_row['isys_stor_model__id']) . ';');
					continue;
				} // if

				$l_model_cache[$l_row['isys_stor_model__id']] = $l_row;
			} // while
		} // if

		// ################################################################################################################################

		// Are there any models without manufacturer?
		$l_sql = 'SELECT isys_catg_stor_list__id, isys_catg_stor_list__isys_stor_model__id
			FROM isys_catg_stor_list
			WHERE isys_catg_stor_list__isys_stor_manufacturer__id IS NULL
			AND isys_catg_stor_list__isys_stor_model__id > 0;';

		$l_res = $l_dao->retrieve($l_sql);

		$g_migration_log[] = '<span class="bold">Looking for devices with missing manufacturers.</span>';

		if (count($l_res))
		{
			$g_migration_log[] = '<span class="indent">Found ' . count($l_res) . ' item(s)!</span>';

			$l_empty_manufacturer_id = $l_dao->retrieve('SELECT isys_stor_manufacturer__id
				FROM isys_stor_manufacturer
				WHERE isys_stor_manufacturer__const = "C__STOR__MANUFACTURER__EMPTY"
				LIMIT 1;')->get_row_value('isys_stor_manufacturer__id');

			if ($l_empty_manufacturer_id === null)
			{
				// Create a "empty" manufacturer and use it's ID to update the "isys_catg_stor_list" table.
				$l_sql = 'INSERT INTO isys_stor_manufacturer
					SET isys_stor_manufacturer__title = "LC__UNIVERSAL__EMPTY",
					isys_stor_manufacturer__const = "C__STOR__MANUFACTURER__EMPTY",
					isys_stor_manufacturer__status = ' . $l_dao->convert_sql_int(C__RECORD_STATUS__NORMAL);

				if ($l_dao->update($l_sql) && $l_dao->apply_update())
				{
					$l_empty_manufacturer_id = $l_dao->get_last_id_from_table('isys_stor_manufacturer');
				} // if
			} // if

			if ($l_empty_manufacturer_id > 0)
			{
				$g_migration_log[] = '<span class="indent">Update item(s) with the newly created "' . _L('LC__UNIVERSAL__EMPTY') . '" manufacturer (#' . $l_empty_manufacturer_id . ').</span>';

				$l_manufacturer_cache[$l_empty_manufacturer_id] = array(
					'isys_stor_manufacturer__title' => 'LC__UNIVERSAL__EMPTY'
				);

				while ($l_row = $l_res->get_row())
				{
					if (!isset($l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']]))
					{
						$l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']] = array();
					} // if

					$l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']][] = $l_empty_manufacturer_id;

					$l_dao->update('UPDATE isys_catg_stor_list
						SET isys_catg_stor_list__isys_stor_manufacturer__id = ' . $l_dao->convert_sql_id($l_empty_manufacturer_id) . '
						WHERE isys_catg_stor_list__id = ' . $l_dao->convert_sql_id($l_row['isys_catg_stor_list__id']));
				} // while
			} // if
		}
		else
		{
			$g_migration_log[] = '<span class="indent grey">None found :)</span>';
		} // if

		// ################################################################################################################################

		// Iterate over all devices, which have assigned models.
		$l_sql = 'SELECT isys_catg_stor_list__id, isys_catg_stor_list__isys_stor_manufacturer__id, isys_catg_stor_list__isys_stor_model__id
			FROM isys_catg_stor_list
			WHERE isys_catg_stor_list__isys_stor_model__id > 0;';

		$l_res = $l_dao->retrieve($l_sql);

		$g_migration_log[] = '<span class="bold">Checking ' . count($l_res) . ' item(s) for manufacturer / model mismatch.</span>';

		if (count($l_res))
		{
			while ($l_row = $l_res->get_row())
			{
				if (! isset($l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']]))
				{
					$l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']] = array();
				} // if

				$l_mapping[$l_row['isys_catg_stor_list__isys_stor_model__id']][] = $l_row['isys_catg_stor_list__isys_stor_manufacturer__id'];
			} // while

			foreach ($l_mapping as $l_model_id => $l_manufacturers)
			{
				$l_manufacturers = array_unique($l_manufacturers);
				$l_manufacturer_id = array_shift($l_manufacturers);

				$l_sql = 'UPDATE isys_stor_model
					SET isys_stor_model__isys_stor_manufacturer__id = ' . $l_dao->convert_sql_id($l_manufacturer_id) . '
					WHERE isys_stor_model__id = ' . $l_dao->convert_sql_id($l_model_id) . '
					LIMIT 1;';

				$g_migration_log[] = '<span class="indent">Setting "' . _L($l_manufacturer_cache[$l_manufacturer_id]['isys_stor_manufacturer__title']) . '" (#' . $l_manufacturer_id . ') > "' . _L($l_model_cache[$l_model_id]['isys_stor_model__title']) . '" (#' . $l_model_id . ').</span>';

				$l_dao->update($l_sql);

				// Add the models to the other manufacturers.
				if (count($l_manufacturers))
				{
					foreach ($l_manufacturers as $l_manufacturer_id)
					{
						$l_sql = 'INSERT INTO isys_stor_model
							SET isys_stor_model__title = ' . $l_dao->convert_sql_text($l_model_cache[$l_model_id]['isys_stor_model__title']) . ',
							isys_stor_model__isys_stor_manufacturer__id = ' . $l_dao->convert_sql_id($l_manufacturer_id) . ',
							isys_stor_model__status = ' . $l_dao->convert_sql_int(C__RECORD_STATUS__NORMAL);

						if ($l_dao->update($l_sql) && $l_dao->apply_update())
						{
							$l_new_model = $l_dao->get_last_id_from_table('isys_stor_model');

							$g_migration_log[] = '<span class="superindent grey">Also setting "' . _L($l_manufacturer_cache[$l_manufacturer_id]['isys_stor_manufacturer__title']) . '" (#' . $l_manufacturer_id . ') > "' . _L($l_model_cache[$l_model_id]['isys_stor_model__title']) . '" (#' . $l_new_model . ').</span>';
						} // if
					} // foreach
				} // if
			} // foreach

			$l_dao->apply_update();
		} // if

		// Set flag in DB.
		$this->migration_done($g_migration_identifier);

		$g_migration_log[] = '<span class="bold indent">Migration successful.</span>';
	}
	catch (Exception $e)
	{
		$g_migration_log[] = '<span class="bold red">' . $e->getMessage() . '</span>';
	} // try
} // if