<?php

/**
 * i-doit
 *
 * DAO: abstraction layer for CMDB global categories.
 *
 * @package        i-doit
 * @subpackage     CMDB_Categories
 * @author         Leonard Fischer <lfischer@i-doit.org>
 * @version        Van Quyen Hoang <qhoang@i-doit.org> 19.05.2014
 * @copyright      synetics GmbH
 * @license        http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_cmdb_dao_category_property_ng extends isys_cmdb_dao_category
{
    /**
     * Constants for grouping list values as comma or as list (<li></li>)
     */
    const C__GROUPING__COMMA = 0;
    const C__GROUPING__LIST  = 1;

    /**
     * @var
     */
    private $m_grouping = 0;

    /**
     * @return mixed
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function getGrouping()
    {
        return $this->m_grouping;
    }

    /**
     * @param int $p_grouping
     *
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function setGrouping($p_grouping)
    {
        $this->m_grouping = $p_grouping;
        return $this;
    } // function

    /**
     * Array which contains references
     *
     * @var array
     */
    private $m_reference = ['root' => 'obj_main.isys_obj__id'];

    /**
     * Helper array for reordering the properties for the joins
     *
     * @var array
     */
    private $m_property_order = [];

    /**
     * Ignore these format callbacks
     *
     * @var array
     */
    public static $m_ignored_format_callbacks = [
        'location_property_pos'
    ];
    /**
     * Alias counter for the report-builder.
     *
     * @var  integer
     */
    protected $m_alias_cnt = 1;
    /**
     * List of aliases for the report-builder.
     *
     * @var  array
     */
    protected $m_aliases = [
        'isys_obj'                        => 'obj_main',
        //'isys_catg_ip_list' => 'ip_list',
        //'isys_cats_net_ip_addresses_list' => 'ip',
        //'isys_catg_location_list' => 'loc',
        //'isys_catg_logical_unit_list' => 'log_unit'
    ];
    /**
     * @var array
     */
    protected $m_aliases_lvls = [];
    /**
     * Variable which determines if the report also displays empty values
     *
     * @var bool
     */
    protected $m_empty_values = true;
    /**
     * This variable is used to define, if all the necessary preparations have been made for creating the generic query.
     *
     * @var  boolean
     */
    protected $m_prepared_data_for_query_construction = false;
    /**
     * This array will hold all the necessary property-data!
     *
     * @var  array
     */
    protected $m_property_rows = [];
    /**
     * This array will hold all the necessary property-data!
     *
     * @var  array
     */
    protected $m_property_rows_lvls = [];
    /**
     * @var bool
     */
    protected $m_query_as_report = false;

    /**
     * Contains all used aliase
     *
     * @var array
     */
    private $m_already_used_aliase = [];
    /**
     * Variable which contains the data field columns which will be used for the custom categories joins only for the
     * main object.
     *
     * @var string
     */
    private $m_parent_custom_field = '';
    /**
     * Contains the referenced fields
     *
     * @var array
     */
    private $m_referenced_fields = [];
    /**
     * Variable which contains all referenced fields which will be deleted after building the query
     *
     * @var array
     */
    private $m_remove_from_selection = [];
    /**
     * Contains the sub joins from the conditions
     *
     * @var array
     */
    private $m_sub_joins = [];

    /**
     * Special array which is used for conditions
     *
     * @var array
     */
    private $m_text_fields = [
        'C__CATG__APPLICATION' => [
            'assigned_variant' => 'isys_cats_app_variant_list__variant'
        ]
    ];

    /**
     * @var array
     */
    private $m_reference_types_selection = [
        C__PROPERTY__INFO__TYPE__OBJECT_BROWSER => 'reference',
    ];

    /**
     * @var array
     */
    private $m_reference_types_condition = [
        C__PROPERTY__INFO__TYPE__DIALOG,
        C__PROPERTY__INFO__TYPE__DIALOG_PLUS,
        C__PROPERTY__INFO__TYPE__DIALOG_LIST,
        C__PROPERTY__INFO__TYPE__OBJECT_BROWSER
    ];

    /**
     * @var array
     */
    private $m_property_key_selections = [];

    /**
     * @var array
     */
    private $m_selects = [];

    /**
     * @return array
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function getSelects()
    {
        return $this->m_selects;
    } // function

    /**
     * @return array
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_property_key_selections()
    {
        return $this->m_property_key_selections;
    } // function

    /**
     * @param array $p_value
     *
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function set_property_key_selections($p_key, $p_value)
    {
        $this->m_property_key_selections[$p_key] = $p_value;
    } // function

    /**
     * Method for setting the "query_as_report" variable from extern.
     *
     * @param   boolean $p_query_as_report
     *
     * @return  isys_cmdb_dao_category_property_ng
     * @author  Leonard Fischer <lfischer@i-doit.com>
     */
    public function set_query_as_report($p_query_as_report)
    {
        $this->m_query_as_report = !!$p_query_as_report;

        return $this;
    } // function

    /**
     * Resets member variables
     *
     * @return  isys_cmdb_dao_category_property_ng
     */
    public function reset()
    {
        $this->m_already_used_aliase   = [
            'ip',
            'ip_list'
        ];
        $this->m_aliases_lvls          = [];
        $this->m_sub_joins             = [];
        $this->m_referenced_fields     = [];
        $this->m_remove_from_selection = [];
        $this->m_parent_custom_field   = '';
        $this->m_aliases               = [
            'isys_obj'                        => 'obj_main',
            'isys_cats_net_ip_addresses_list' => 'ip',
        ];
        $this->m_alias_cnt = 1;
        $this->m_prepared_data_for_query_construction = false;

        return $this;
    } // function

    /**
     * Method for recieving the property-ID's by a given array.
     *
     * @param   array $p_properties
     *
     * @return  array
     * @author  Dennis Stücken <dstuecken@synetics.de>
     * @todo    Enable this method to handle categorie-arrays with more than one property!
     */
    public function format_property_array($p_properties)
    {
        $l_property_array = [];

        foreach ($p_properties as $l_props)
        {
            if ($l_props['g'])
            {
                foreach ($l_props['g'] as $l_cat_id => $l_property)
                {
                    $l_tmp = $this->retrieve_properties(null, $l_cat_id, null, null, ' AND isys_property_2_cat__prop_key = \'' . $l_property[0] . '\'')
                        ->get_row();

                    if ($l_tmp)
                    {
                        $l_property_array[] = $l_tmp['id'];
                    } // if
                } // foreach

                foreach ($l_props['s'] as $l_cat_id => $l_property)
                {
                    $l_tmp = $this->retrieve_properties(null, null, $l_cat_id, null, ' AND isys_property_2_cat__prop_key = \'' . $l_property[0] . '\'')
                        ->get_row();

                    if ($l_tmp)
                    {
                        $l_property_array[] = $l_tmp['id'];
                    } // if
                } // foreach
            } // if
        } // foreach

        return $l_property_array;
    } // function

    /**
     * Creates SQL query from selected properties.
     *
     * @param   mixed   $p_properties JSON array or array
     * @param   array   $p_objects    List of objects ('isys_obj__id' has be to included!).
     * @param   boolean $p_with_object_data
     *
     * @return  string  SQL query
     * @throws  isys_exception_general
     */
    public function create_property_query($p_properties, $p_objects, $p_with_object_data = false, $p_leave_field_identifiers = false)
    {
        try
        {
            if (is_string($p_properties))
            {
                $l_properties = isys_format_json::decode($p_properties);
            }
            else if (is_array($p_properties))
            {
                $l_properties = $p_properties;
            }
            else
            {
                throw new isys_exception_general('Invalid argument.');
            } // if

            $l_smarty_plugin = new isys_smarty_plugin_f_property_selector();
            $l_preselection  = $l_smarty_plugin->handle_preselection($l_properties);

            $l_keys = [];
            foreach ($l_preselection as $l_value)
            {
                $l_keys[] = $l_value['prop_id'];
            } // foreach

            if ($p_with_object_data)
            {
                $l_selects = $this->create_property_query_select($l_keys, false, ['obj_main.*'], $p_leave_field_identifiers);
            }
            else
            {
                $l_selects = $this->create_property_query_select($l_keys, false, [], $p_leave_field_identifiers);
            } // if

            $l_joins = $this->create_property_query_join($l_keys);

            $l_objects = [];

            if (is_array($p_objects) && count($p_objects))
            {
                foreach ($p_objects as $l_object)
                {
                    $l_objects[] = $this->convert_sql_id($l_object['isys_obj__id']);
                } // foreach

                return 'SELECT ' . implode(', ', $l_selects) . ' FROM isys_obj as obj_main ' . implode(' ', $l_joins) . ' WHERE obj_main.isys_obj__id IN (' . implode(
                    ',',
                    $l_objects
                ) . ');';
            } // if

            // This could happen, but shall not throw errors :)
            return 'SELECT ' . implode(', ', $l_selects) . ' FROM isys_obj as obj_main ' . implode(' ', $l_joins) . ' WHERE obj_main.isys_obj__id = NULL;';
        }
        catch (Exception $e)
        {
            throw new isys_exception_general('Failed to create property query: ' . $e->getMessage());
        } //try/catch
    } // function

    /**
     * Method for creating a generic list query based on configured category properties.
     *
     * @param   mixed   $p_properties
     * @param   mixed   $p_object_types Can be either a integer or a array of integers.
     * @param   mixed   $p_object_ids   Can be either a integer or a array of integers.
     * @param   array   $p_queries
     * @param   boolean $p_leave_field_identifiers
     *
     * @return  string
     * @author  Dennis Stücken <dstuecken@synetics.de>
     */
    public function create_property_query_for_lists($p_properties, $p_object_types = null, $p_object_ids = null, $p_queries = [], $p_leave_field_identifiers = true, $p_use_property_ids_as_title = false)
    {
        if (is_string($p_properties))
        {
            $p_properties = isys_format_json::decode($p_properties);
        } // if

        $this->prepare_necessary_tasks($p_properties);

        $l_selects = $this->create_property_query_select(
            $p_properties,
            false,
            $p_queries,
            $p_leave_field_identifiers,
            $p_use_property_ids_as_title,
            true
        );

        $l_joins = $this->create_property_query_join($p_properties, true);

        $l_sql = "SELECT \n" . implode(", \n", $l_selects) . " \n\n" . "FROM isys_obj AS obj_main \n" . implode(" \n", $l_joins) . " \n\n" . "WHERE TRUE \n";

        if ($p_object_types !== null)
        {
            if (is_array($p_object_types))
            {
                $l_sql .= 'AND obj_main.isys_obj__isys_obj_type__id ' . $this->prepare_in_condition($p_object_types) . " \n";
            }
            else
            {
                $l_sql .= 'AND obj_main.isys_obj__isys_obj_type__id = ' . $this->convert_sql_id($p_object_types) . " \n";
            } // if
        } // if

        if ($p_object_ids !== null)
        {
            if (is_array($p_object_ids))
            {
                $l_sql .= 'AND obj_main.isys_obj__id ' . $this->prepare_in_condition($p_object_ids) . " \n";
            }
            else
            {
                $l_sql .= 'AND obj_main.isys_obj__id = ' . $this->convert_sql_id($p_object_ids) . " \n";
            } // if
        } // if
        return $l_sql;
    } // function

    /**
     * Method for creating the dynamic SQL (Originally from the report editor).
     *
     * @param    $p_limit               integer
     * @param    $p_data                String see hidden field *__COMPLETE in property selector
     * @param    $p_empty_values        integer defines if 'LEFT' or 'INNER' join should be used
     * @param    $p_conditions          @todo: make it usable
     *
     * @todo make it work for the new object lists
     * @return  string
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function create_property_query_for_report($p_limit = null, $p_data = null, $p_empty_values = null, $p_conditions = null, $p_relations = false, $p_group_by_object = false)
    {
        $l_conditions = $l_new_lvls = $l_select_columns = $l_selects = $l_lvl_selects = [];
        // Here we go through the selected categories.
        if (isset($_POST['report__HIDDEN_IDS']))
        {
            $l_select_columns        = isys_format_json::decode($_POST['report__HIDDEN_IDS']);
            $this->m_query_as_report = true;

            // We have some object references with some properties
            if (isset($_POST['lvls_raw']) && is_array($_POST['lvls_raw']))
            {
                foreach ($_POST['lvls_raw'] AS $l_lvl => $l_lvl_content)
                {
                    if (is_array($l_lvl_content))
                    {
                        foreach ($l_lvl_content AS $l_lvl_key => $l_lvl_properties)
                        {
                            $l_lvl_properties                           = isys_format_json::decode($l_lvl_properties);

                            $l_lvl_key_arr = explode('--', $l_lvl_key);
                            $l_new_lvl_key = '';
                            foreach($l_lvl_key_arr AS $l_lvl_key_pair)
                            {
                                list($l_cat_const, $l_property_key) = explode('-', $l_lvl_key_pair);
                                $l_condition = 'AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                                        $l_cat_const
                                    ) . ' AND ' . 'isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_property_key) . ' ';
                                $l_new_lvl_key .= $this->retrieve_properties(null, null, null, null, $l_condition)
                                        ->get_row_value('id') . '--';
                            }
                            $l_new_lvl_key = rtrim($l_new_lvl_key, '--');
                            $_POST['lvls_raw__IDS'][$l_lvl][$l_lvl_key] = [];
                            $l_new_lvls[$l_new_lvl_key] = [];

                            if (is_array($l_lvl_properties))
                            {
                                foreach ($l_lvl_properties AS $l_prop)
                                {
                                    $l_prop_arr = [];
                                    foreach ($l_prop AS $l_type => $l_cat_info)
                                    {
                                        foreach ($l_cat_info AS $l_cat_const => $l_property)
                                        {
                                            foreach ($l_property AS $l_title)
                                            {
                                                $l_prop_arr[] = $this->retrieve_properties(
                                                    null,
                                                    null,
                                                    null,
                                                    null,
                                                    'AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_title) .
                                                    ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text($l_cat_const),
                                                    true,
                                                    null
                                                )
                                                    ->get_row_value('id');
                                            } // foreach
                                        } // foreach
                                    } // foreach
                                    $l_new_lvls[$l_new_lvl_key] = $_POST['lvls_raw__IDS'][$l_lvl][$l_lvl_key] = array_merge($_POST['lvls_raw__IDS'][$l_lvl][$l_lvl_key], $l_prop_arr);
                                } // foreach
                            } // if

                            $l_new_lvls[$l_new_lvl_key] = $_POST['lvls_raw__IDS'][$l_lvl][$l_lvl_key] = isys_format_json::encode(
                                $_POST['lvls_raw__IDS'][$l_lvl][$l_lvl_key]
                            );
                        } // foreach
                    } // if
                } // foreach
            } // if
        }
        elseif (isset($p_data))
        {
            $l_lvls_testing = [];

            // JSON String in POST['*__COMPLETE'] from property selector
            if (is_array($p_data))
            {
                $l_lvls_testing = $p_data;
            }
            else if (is_string($p_data))
            {
                $l_lvls_testing = isys_format_json::decode($p_data);
            } // if

            foreach ($l_lvls_testing AS $l_key => $l_lvl_arr)
            {
                $l_lvl_cache = [];
                foreach ($l_lvl_arr AS $l_lvl)
                {
                    $l_cat_type  = key($l_lvl);
                    $l_current   = current($l_lvl);
                    $l_cat_const = key($l_current);
                    $l_prop_key  = $l_current[$l_cat_const][1];

                    if ($l_key == 'root')
                    {
                        $l_select_columns[] = $this->retrieve_properties(
                            null,
                            null,
                            null,
                            null,
                            'AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key) .
                            ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text($l_cat_const),
                            true,
                            null
                        )
                            ->get_row_value('id');
                    }
                    else
                    {
                        $l_lvl_cache[] = $this->retrieve_properties(
                            null,
                            null,
                            null,
                            null,
                            'AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key) .
                            ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text($l_cat_const),
                            true,
                            null
                        )
                            ->get_row_value('id');
                    } // if
                } // foreach
                if (count($l_lvl_cache) > 0)
                {
                    if (strpos($l_key, '--') !== false) $l_lvl_cache_key = substr_count($l_key, '--') + 1;
                    else
                        $l_lvl_cache_key = 1;

                    $l_key_arr    = explode('--', $l_key);
                    $l_key_string = '';
                    foreach ($l_key_arr AS $l_ref_key)
                    {
                        list($l_cat_const, $l_prop_key) = explode('-', $l_ref_key);
                        $l_key_string .= $this->retrieve_properties(
                            null,
                            null,
                            null,
                            null,
                            'AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                                $l_cat_const
                            ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key),
                            true,
                            null
                        )
                            ->get_row_value('id');
                        if ($l_lvl_cache_key > 1)
                        {
                            $l_key_string .= '--';
                        } // if
                    } // foreach

                    $l_key_string              = rtrim($l_key_string, '--');
                    $l_new_lvls[$l_key_string] = isys_format_json::encode($l_lvl_cache);
                } // if
            } // foreach
        } // if

        if (isset($_POST['empty_values']) || $p_empty_values !== null)
        {
            $this->m_empty_values = (isset($_POST['empty_values'])) ? (bool) $_POST['empty_values'] : (bool) $p_empty_values;
        } // if

        // Unset condition template
        if (isset($_POST['querycondition']['#{queryConditionBlock}']))
        {
            unset($_POST['querycondition']['#{queryConditionBlock}']);
        }

        // Workaroud for dozens of checks.
        if (is_array($l_select_columns) !== true)
        {
            $l_select_columns = [];
        } // if

        // This is a list of property-id's to select from the isys_properties_2_cat table.
        $l_property_ids = $l_select_columns;

        // We don't want duplicates.
        $l_property_ids = array_unique($l_property_ids);

        // Prepare the property-rows.
        if (!$this->m_prepared_data_for_query_construction)
        {
            $this->prepare_necessary_tasks($l_property_ids);
        } // if

        // Prepare the select-part of the query.
        $l_selects = $this->create_property_query_select($l_select_columns, false, [], false, false, $p_group_by_object);

        // Prepare the joins for our query.
        $l_joins = $this->create_property_query_join($this->m_property_order);

        if (count($l_new_lvls) > 0)
        {
            $this->m_property_order = array();

            $l_lvls_select = $this->create_property_query_lvls_select($l_new_lvls, $l_selects, false, $p_group_by_object);
            $l_lvls_join   = $this->create_property_query_join_lvls($this->m_property_order, $l_selects, $l_lvls_select);

            $l_new_selection = [];

            // removing refenced field IDs
            if (count($this->m_remove_from_selection) > 0)
            {
                foreach ($this->m_remove_from_selection AS $l_removed_selection)
                {
                    unset($l_lvls_select[$l_removed_selection]);
                } // foreach
            } // if

            foreach ($l_selects AS $l_prop_key => $l_select_field)
            {
                $l_new_selection[] = $l_select_field;
                $l_unsetted        = false;
                foreach ($l_lvls_select AS $l_assigned_prop => $l_lvl_select_field)
                {
                    $l_assigned_prop_key_arr = explode('--', $l_assigned_prop);
                    if ($l_assigned_prop_key_arr[0] == $l_prop_key)
                    {
                        if (!$l_unsetted)
                        {
                            $l_unsetted = true;
                            array_pop($l_new_selection);
                        }
                        $l_new_selection[] = $l_lvl_select_field;
                    }
                }
            }

            foreach ($l_lvls_join AS $l_lvl_sql_join)
            {
                $l_joins = array_merge($l_joins, $l_lvl_sql_join);
            } // foreach
            $l_selects = $l_new_selection;

        } // if

        if (isset($_POST['querycondition__HIDDEN']))
        {
            $_POST['querycondition'] = array_replace_recursive($_POST['querycondition'], $_POST['querycondition__HIDDEN']);
        } // if

        // Prepare all the conditions for the query.
        if (isset($_POST['querycondition']))
        {
            $l_conditions = $this->create_property_query_condition($_POST['querycondition'], $l_joins);

            if (!isset($_POST['display_relations']) || !$_POST['display_relations'])
            {
                unset($_POST['querycondition'][0], $_POST['querycondition'][1]);
            } // if

            // First the normal joins
            if (is_array($l_conditions['joins']) && count($l_conditions['joins']) > 0)
            {
                foreach ($l_conditions['joins'] AS $l_arr_joins)
                {
                    $l_joins = array_merge($l_joins, $l_arr_joins);
                } // if
            } // if
        } // if

        // Secondly the sub joins
        if (count($this->m_sub_joins) > 0)
        {
            foreach ($this->m_sub_joins AS $l_arr_joins)
            {
                $l_joins = array_merge($l_joins, $l_arr_joins);
            } // foreach
        } // if

        if (!isset($_POST['display_relations']) || !$_POST['display_relations'])
        {
            $p_conditions .= ' AND (obj_main.isys_obj__isys_obj_type__id != ' . $this->convert_sql_id(C__OBJTYPE__RELATION) .
                ' AND obj_main.isys_obj__isys_obj_type__id != ' . $this->convert_sql_id(C__OBJTYPE__PARALLEL_RELATION) . ')';
        } // if

        // Returning the SQL-query and the other dara (title, description, ...).
        $l_sql = "SELECT \n" . implode(", \n", $l_selects) . " \n\n" . "FROM isys_obj AS obj_main \n" . implode(
                " \n",
                $l_joins
            ) . " \n\n" . "WHERE TRUE \n" . $p_conditions . " \n" . ((isset($l_conditions['conditions'])) ? rtrim($l_conditions['conditions'], 'AND OR') : '') . "";

        if($p_group_by_object > 0 )
        {
            $l_sql .= ' GROUP BY __id__';
        } // if

        if ($p_limit !== null)
        {
            $l_sql .= ' LIMIT 0, ' . $p_limit;
        }
        return $l_sql . ';';
    } //function

    /**
     * Create the selection for the referenced objects
     *
     * @param array $p_lvls
     * @param       $p_selects
     * @param bool  $p_select_status
     * @param bool  $p_leave_field_identifiers
     * @param bool  $p_group_by_object
     * @todo make it work for the new object lists
     *
     * @return array
     * @author Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function create_property_query_lvls_select(array $p_lvls, $p_selects, $p_leave_field_identifiers = false, $p_group_by_object = false)
    {
        $l_selects = [];

        // Iterate through every level
        foreach ($p_lvls AS $l_assigned_property => $l_properties)
        {
            $l_unformatted_properties = (isys_format_json::is_json($l_properties))? isys_format_json::decode($l_properties): $l_properties;
            $this->prepare_necessary_tasks_for_lvls_select($l_unformatted_properties, $l_assigned_property);
            $l_referenced_title    = '';

            if (strpos($l_assigned_property, '--') === false)
            {
                $l_referenced_field_string = substr($p_selects[$l_assigned_property], strpos($p_selects[$l_assigned_property], ' AS ') + 4);
                $l_referenced_title        = substr($l_referenced_field_string, 1, strpos($l_referenced_field_string, '###') - 1);
            }
            else
            {
                foreach ($l_selects AS $l_select_key => $l_select_string)
                {
                    if ($l_select_key == $l_assigned_property)
                    {
                        $l_referenced_field_string = substr($l_selects[$l_select_key], strpos($l_selects[$l_select_key], ' AS ') + 4);
                        $l_referenced_title        = substr($l_referenced_field_string, 1, strpos($l_referenced_field_string, '###') - 1);
                        break;
                    } // if
                } // foreach
            } // if

            // And add the selected ones from the report-builder.
            foreach ($l_unformatted_properties as $l_select)
            {
                $l_field_name        = '';
                $l_cat               = 'cats';

                $l_table = current(explode('__', $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));

                if ($this->m_property_rows_lvls[$l_select]['catg'] != null)
                {
                    $l_cat = 'catg';
                }
                elseif ($this->m_property_rows_lvls[$l_select]['catg_custom'] != null)
                {
                    $l_cat = 'catg_custom';
                } // if

                if (!isset($this->m_property_rows_lvls[$l_select])) continue;

                if ($l_table == 'isys_catg_custom_fields_list')
                {
                    $l_field_value  = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                    $l_field_alias  = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                    $l_field_type   = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                    $l_alias        = $this->retrieve_alias_lvls($l_field_alias . '#isys_catg_custom_fields_list', null, $l_assigned_property);
                    $l_output_alias = "'" . $this->m_property_rows_lvls[$l_select]['title'] . "#" . $l_referenced_title . "###" . $this->m_property_rows_lvls[$l_select][$l_cat] . "'";

                    switch ($l_field_type)
                    {
                        case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                        case C__PROPERTY__INFO__TYPE__N2M:
                            $l_alias_sec = $this->retrieve_alias_lvls('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj', $l_assigned_property);
                            $l_field     = 'j' . $l_alias_sec . '.isys_obj__title AS ' . $l_output_alias;
                            break;
                        case C__PROPERTY__INFO__TYPE__DIALOG:
                        case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                            $l_alias_sec = $this->retrieve_alias_lvls('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom', $l_assigned_property);
                            $l_field     = 'j' . $l_alias_sec . '.isys_dialog_plus_custom__title AS ' . $l_output_alias;
                            break;
                        default:
                            $l_field = 'j' . $l_alias . '.' . $l_field_value . ' AS ' . $l_output_alias;
                            break;
                    } // switch

                    $l_selects[$l_assigned_property . '--' . $l_select] = $l_field;
                    continue;
                } // if

                $l_alias = 'j' . $this->retrieve_alias_lvls($l_table, null, $l_assigned_property);

                // Then we check for special table-names inside the select.
                if (isset($this->m_aliases_lvls[$l_table]))
                {
                    $l_alias = $this->m_aliases_lvls[$l_table];
                }
                elseif ($l_table == 'isys_logbook')
                {
                    $l_alias = 'j' . $this->retrieve_alias_lvls('isys_catg_logb_list', 'isys_logbook', $l_assigned_property);
                } // if

                $l_selects[$l_assigned_property . '--' . $l_select] = $l_alias . "." . $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] . (!$p_leave_field_identifiers ? " AS '" . $this->m_property_rows_lvls[$l_select]['title'] . "#" . $l_referenced_title . "###" . $this->m_property_rows_lvls[$l_select][$l_cat] . "'" : $l_field_name);
            } // foreach
        } // foreach
        return $l_selects;
    } // function

    /**
     * Create the joins for the referenced objects
     *
     * @param array $p_lvls
     * @param array $p_selects
     * @param array $p_lvl_selects
     * @param bool  $p_from_condition
     * @param null  $p_assigned_field
     * @todo make it work for the new object lists
     *
     * @return array
     * @author Van Quyen Hoang <qhoang@i-doit.org>
     */
    private function create_property_query_join_lvls(array $p_lvls, &$p_selects = [], &$p_lvl_selects = [], $p_from_condition = false, $p_assigned_field = null)
    {
        $l_return = [];

        if ($this->m_empty_values)
        {
            $l_join_type = "LEFT";
        }
        else
        {
            $l_join_type = "INNER";
        } // if

        $l_parent_custom_field = '';
        $l_referenced_field    = '';

        foreach ($p_lvls AS $l_assigned_property => $l_properties)
        {
            $l_unformatted_properties = (isys_format_json::is_json($l_properties)) ? isys_format_json::decode($l_properties) : $l_properties;
            $l_already_joined_tables  = $l_joins = [];

            // We need this array to save "already joined" tables for saving a bit of performance.
            if (isset($this->m_referenced_fields[$l_assigned_property]))
            {
                $l_join_condition_field = $this->m_referenced_fields[$l_assigned_property];
            }
            else if ($p_assigned_field !== null)
            {
                $l_join_condition_field = $p_assigned_field;
            }
            else
            {
                if (strpos($l_assigned_property, '--') === false && !$p_from_condition)
                {
                    $l_join_condition_field = substr($p_selects[$l_assigned_property], 0, strpos($p_selects[$l_assigned_property], ' AS'));
                    if (strpos($l_join_condition_field, 'isys_obj__title') !== false)
                    {
                        $l_join_condition_field = str_replace('title', 'id', $l_join_condition_field);
                    } // if
                }
                else
                {
                    $l_join_condition_field = str_replace('title', 'id', substr($p_lvl_selects[$l_assigned_property], 0, strpos($p_lvl_selects[$l_assigned_property], ' AS')));
                    if (!in_array($l_assigned_property, $this->m_remove_from_selection))
                    {
                        $this->m_remove_from_selection[] = $l_assigned_property;
                    } // if
                } // if
            } // if

            // Now we create the single JOIN's.
            foreach($l_unformatted_properties AS $l_prop_id)
            {
                $l_reference = null;

                if (!isset($this->m_property_rows_lvls[$l_prop_id])) continue;

                $l_prop_data = $this->m_property_rows_lvls[$l_prop_id];

                // We won't handle dynamic properties here.
                if ($l_prop_data['type'] == C__PROPERTY_TYPE__DYNAMIC && !isset($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]))
                {
                    continue;
                } // if

                $l_alias_sec = $l_alias_third = $l_alias_fourth = $l_alias_obj = null;

                if ($l_prop_data['type'] == C__PROPERTY_TYPE__DYNAMIC)
                {
                    $l_table = $l_prop_data['table'];
                }
                else
                {
                    $l_table = current(explode('__', $l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
                }

                if ($l_table == 'isys_catg_custom_fields_list')
                {
                    $l_field_alias = $l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                    $l_field_key   = substr($l_field_alias, strpos($l_field_alias, '_c_') + 1, strlen($l_field_alias));
                    $l_field_type  = $l_prop_data['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                    $l_alias       = 'j' . $this->retrieve_alias_lvls($l_field_alias . '#isys_catg_custom_fields_list', null, $l_assigned_property);

                    if (in_array($l_alias, $this->m_already_used_aliase)) continue;

                    if ($l_field_type == C__PROPERTY__INFO__TYPE__DIALOG || $l_field_type == C__PROPERTY__INFO__TYPE__DIALOG_PLUS)
                    {
                        $l_join_type = 'LEFT';
                    }
                    elseif (!$this->m_empty_values && $l_join_type == 'LEFT')
                    {
                        $l_join_type = "INNER";
                    } // if

                    $l_join_string = $l_join_type . ' JOIN ' . $l_table . ' AS ' . $l_alias . ' ON ' . $l_join_condition_field . ' = ' . $l_alias . '.' . $l_table . '__isys_obj__id AND ' . $l_alias . '.' . $l_table . '__field_key = ' . $this->convert_sql_text(
                            $l_field_key
                        ) . ' ';

                    $l_identifier = $p_assigned_field . '--' . $l_prop_data['const'];

                    if (empty($l_parent_custom_field[$l_identifier]))
                    {
                        $l_description_alias                  = str_replace('-', '_', $l_identifier);
                        $l_parent_custom_field[$l_identifier] = $l_description_alias . '.' . $l_table . '__data__id';

                        $l_join_string = 'LEFT JOIN ' . $l_table . ' AS ' . $l_description_alias . ' ON ' . $l_join_condition_field . ' = ' . $l_description_alias . '.' . $l_table . '__isys_obj__id
                            AND ' . $l_description_alias . '.' . $l_table . '__isysgui_catg_custom__id = (SELECT isysgui_catg_custom__id FROM isysgui_catg_custom WHERE isysgui_catg_custom__const = ' . $this->convert_sql_text($l_prop_data['const']) . ')
                            AND ' . $l_description_alias . '.' . $l_table . '__field_type = ' . $this->convert_sql_text('commentary') . ' ' . $l_join_string . '
                            AND ' . $l_alias . '.' . $l_table . '__data__id = ' . $l_parent_custom_field[$l_identifier] . ' ';
                    }
                    else
                    {
                        $l_join_string .= ' AND ' . $l_alias . '.' . $l_table . '__data__id = ' . $l_parent_custom_field[$l_identifier] . ' ';
                    } // if
                    $l_joins[] = $l_join_string;

                    switch ($l_field_type)
                    {
                        case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                        case C__PROPERTY__INFO__TYPE__N2M:
                            $l_alias_sec = 'j' . $this->retrieve_alias_lvls('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj', $l_assigned_property);
                            if (in_array($l_alias_sec, $this->m_already_used_aliase)) continue;

                            $l_joins[] = $l_join_type . ' JOIN isys_obj AS ' . $l_alias_sec . ' ON ' . $l_alias . '.' . $l_table . '__field_content = ' . $l_alias_sec . '.isys_obj__id ';

                            $l_referenced_field = $l_alias_sec . '.isys_obj__id ';
                            break;
                        case C__PROPERTY__INFO__TYPE__DIALOG:
                        case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                            $l_alias_sec = 'j' . $this->retrieve_alias_lvls(
                                    'isys_catg_custom_fields_list',
                                    $l_field_alias . '#isys_dialog_plus_custom',
                                    $l_assigned_property
                                );

                            $l_joins[] = $l_join_type . ' JOIN isys_dialog_plus_custom AS ' . $l_alias_sec . ' ON ' . $l_alias . '.' . $l_table . '__field_content = ' . $l_alias_sec . '.isys_dialog_plus_custom__id ';
                            break;
                        default:
                            break;
                    } // switch

                    if ($l_alias !== null)
                    {
                        $this->m_already_used_aliase[] = $l_alias;
                    } // if
                    if ($l_alias_sec !== null)
                    {
                        $this->m_already_used_aliase[] = $l_alias_sec;
                    } // if
                    continue;
                } // if

                // We have to check for an existing "predefined" alias.
                if (isset($this->m_aliases_lvls[$l_table]))
                {
                    $l_alias = $this->m_aliases_lvls[$l_table];
                }
                else
                {
                    $l_alias = 'j' . $this->retrieve_alias_lvls($l_table, null, $l_assigned_property);
                } // if

                if(isset($this->m_reference[$l_assigned_property . '-' . $l_table]))
                {
                    $l_reference = $this->m_reference[$l_assigned_property . '-' . $l_table];
                } // if

                if (!in_array($l_table . '#isys_obj#' . $l_assigned_property, $l_already_joined_tables))
                {
                    if (in_array($l_alias, $this->m_already_used_aliase)) continue;

                    $l_already_joined_tables[] = $l_table . '#isys_obj#' . $l_assigned_property;

                    if ($l_table == 'isys_obj')
                    {
                        $l_join = $l_join_type . " JOIN " . $l_table . " AS " . $l_alias . " ON " . $l_alias . "." . $l_table . "__id = " . $l_join_condition_field;
                    }
                    else
                    {
                        // If reference exists from a Sub-Select Join than use it instead of the object id from the joined object property
                        if ($l_reference)
                        {
                            $l_join = $l_join_type . " JOIN " . $l_table . " AS " . $l_alias . " ON " . $l_alias . "." . $l_table . "__id = " . $l_reference;
                        }
                        else
                        {
                            $l_join = $l_join_type . " JOIN " . $l_table . " AS " . $l_alias . " ON " . $l_alias . "." . $l_table . "__isys_obj__id = " . $l_join_condition_field;
                            $this->m_reference[$l_assigned_property . '-' . $l_table] = $l_alias . "." . $l_table . "__id";
                        }
                    }

                    $l_joins[]          = $l_join;
                    $l_referenced_field = $l_join_condition_field;
                } // if

                if (!in_array('isys_obj#' . $l_table . '#' . $l_assigned_property, $l_already_joined_tables))
                {
                    if (in_array($l_alias_sec, $this->m_already_used_aliase)) continue;

                    $l_already_joined_tables[] = 'isys_obj#' . $l_table . '#' . $l_assigned_property;
                    $l_alias_sec               = 'j' . $this->retrieve_alias_lvls($l_table, 'isys_obj', $l_assigned_property);

                    $l_joins[]          = $l_join_type . " JOIN isys_obj AS " . $l_alias_sec . " ON " . $l_alias_sec . ".isys_obj__id = " . $l_join_condition_field;
                    $l_referenced_field = $l_alias_sec . ".isys_obj__id";

                    $this->m_already_used_aliase[] = $l_alias_sec;
                }

                $this->m_already_used_aliase[] = $l_alias;
                if (!isset($this->m_referenced_fields[$l_assigned_property . '--' . $l_prop_data['const'] . '-' . $l_prop_data['key']]))
                {
                    $this->m_referenced_fields[$l_assigned_property . '--' . $l_prop_data['const'] . '-' . $l_prop_data['key']] = $l_referenced_field;
                } // if
            } // foreach

            // Clearing out all duplicate joins.
            $l_return[$l_assigned_property] = array_unique($l_joins);
        } // foreach
        return $l_return;
    } // function

    /**
     * Wrapper method which creates all necessary aliase for the referenced properties
     *
     * @param $p_lvl
     * @param $p_property_ids
     * @param $p_assigned_property
     * @todo make it work for the new object lists
     *
     * @return $this
     * @author Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function prepare_necessary_tasks_for_lvls_select($p_property_ids, $p_assigned_property)
    {
        if (count($p_property_ids) > 0)
        {
            // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
            $l_res = $this->retrieve_properties($p_property_ids, null, null, null, "", true);

            // First we get all the needed data from the isys_property_2_cat table.
            while ($l_row = $l_res->get_row())
            {
                $l_cat_dao    = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                $l_row['prop_count'] = 0;
                $l_cat_properties = $l_cat_dao->get_properties();
                $l_properties = array_merge($l_cat_properties, $l_cat_dao->get_dynamic_properties());

                foreach($l_cat_properties AS $l_prop)
                {
                    if($l_prop[C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__REPORT])
                    {
                        $l_row['prop_count']++;
                    } // if
                } // foreach

                $l_row['data'] = $l_properties[$l_row['key']];

                $l_table_arr = explode('_', $l_row['table']);

                if (array_pop($l_table_arr) !== 'list')
                {
                    $l_row['table'] = $l_row['table'] . '_list';
                } // if

                // We save every row, because we will need them quite often in the upcoming code.
                $this->m_property_rows_lvls[$l_row['id']]                          = $l_row;
                $this->m_property_rows_lvls[$l_row['const'] . '-' . $l_row['key']] = $l_row;

                // Also we create table aliases for each possible join.
                $this->create_alias_lvls_select($l_row, $p_assigned_property);
            } // while

            // Reordered properties so that all Sub-Select Joins are joined first
            if(isset($this->m_property_order[$p_assigned_property]['joins']))
            {
                $l_joins = $this->m_property_order[$p_assigned_property]['joins'];
                unset($this->m_property_order[$p_assigned_property]['joins']);
                $this->m_property_order[$p_assigned_property] = array_merge($l_joins, $this->m_property_order[$p_assigned_property]);
            } // if
        } // if
        return $this;
    } // function

    /**
     * Method for building the property-select query.
     *
     * @param   array   $p_property_ids              Array which holds the property-id's (from "isys_property_2_cat").
     * @param   boolean $p_select_status             Shall the status be selected aswell?
     * @param   array   $p_selects                   You may enter some SELECT-statements here (see $this->create_property_query_for_lists()).
     * @param   boolean $p_leave_field_identifiers   Set to true to keep the original field-names instead of "LANGUACE_CONSTANT###123".
     * @param   boolean $p_use_property_ids_as_title Set to true to retrieve the property-IDs as field-names.
     * @param   boolean $p_group_by_object
     *
     * @return  array
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function create_property_query_select(array $p_property_ids, $p_select_status = true, $p_selects = [], $p_leave_field_identifiers = false, $p_use_property_ids_as_title = false, $p_for_list = false)
    {
        // We need to know if all the necessary stuff has been done!
        if (!$this->m_prepared_data_for_query_construction)
        {
            $this->prepare_necessary_tasks($p_property_ids);
        } // if

        // Now we prepare the SELECT's.
        $l_selects = array_merge(
            $p_selects,
            [
                "obj_main.isys_obj__id AS '__id__'"
            ]
        );

        // And add the selected ones from the report-builder.
        foreach ($p_property_ids as $l_select)
        {
            if (!isset($this->m_property_rows[$l_select])) continue;

            $l_field_name        = '';

            if ($p_use_property_ids_as_title)
            {
                $l_field_name = ' AS \'' . $l_select . '\'';
            } // if


            $l_cat   = 'cats';
            $l_table = current(explode('__', $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));

            if ($this->m_property_rows[$l_select]['catg'] != null)
            {
                $l_cat = 'catg';
            }
            elseif ($this->m_property_rows[$l_select]['catg_custom'] != null)
            {
                $l_cat = 'catg_custom';
            } // if

            if ($l_table == 'isys_catg_custom_fields_list')
            {
                $l_field_value  = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                $l_field_alias  = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                $l_field_type   = $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                $l_alias        = $this->retrieve_alias($l_field_alias . '#isys_catg_custom_fields_list');
                $l_output_alias = $this->convert_sql_text('isys_cmdb_dao_category_g_custom_fields__' . $this->m_property_rows[$l_select]['key']);

                switch ($l_field_type)
                {
                    case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                    case C__PROPERTY__INFO__TYPE__N2M:
                        $l_alias_sec = $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj');
                        $l_field     = 'j' . $l_alias_sec . '.isys_obj__title AS ' . $l_output_alias;
                        break;
                    case C__PROPERTY__INFO__TYPE__DIALOG:
                    case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                        $l_alias_sec = $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom');
                        $l_field     = 'j' . $l_alias_sec . '.isys_dialog_plus_custom__title AS ' . $l_output_alias;
                        break;
                    default:
                        $l_field = 'j' . $l_alias . '.' . $l_field_value . ' AS ' . $l_output_alias;
                        break;
                } // switch

                $l_selects[$l_select] = $l_field;
                $this->set_property_key_selections($l_output_alias, $l_field);
                continue;
            } // if

            if(isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT]))
            {
                $l_alias = (!$p_leave_field_identifiers ? " AS '" . $this->m_property_rows[$l_select]['title'] . "###" . $this->m_property_rows[$l_select][$l_cat] . "'" : $this->convert_sql_text($this->m_property_rows[$l_select]['class'] . '__' . $this->m_property_rows[$l_select]['key']));
                $l_joinedTableAlias = null;
                $l_alias_key = null;

                /**
                 * @var $l_selectSubSelect \idoit\Module\Report\SqlQuery\Structure\SelectSubSelect
                 */
                $l_selectSubSelect = clone $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT];

                if($l_table == 'isys_obj' || $l_selectSubSelect->getSelectTable() == 'isys_obj')
                {
                    $l_joinedTableAlias = 'obj_main';
                }
                else
                {
                    if($l_selectSubSelect->getSelectTable() !== '' && $l_selectSubSelect->getSelectFieldObjectID() !== '')
                    {
                        $l_table = $l_selectSubSelect->getSelectTable();
                    }
                    elseif($l_selectSubSelect->getSelectTable() !== '' && $l_selectSubSelect->getSelectFieldObjectID() === '')
                    {
                        $l_alias_key = $l_selectSubSelect->getSelectTable() . '#' . $this->m_property_rows[$l_select]['table'];
                        $l_joinedTableAlias = (isset($this->m_aliases[$l_alias_key]))? 'j' . $this->m_aliases[$l_alias_key]: null;
                    } // if

                    if($l_joinedTableAlias === null && ($l_alias_num = $this->retrieve_alias($l_table)))
                    {
                        $l_joinedTableAlias = 'j' . $l_alias_num;
                    } // if
                } // if

                $l_selectCondition = clone $l_selectSubSelect->getSelectCondition();
                $l_condition = '';

                if($l_alias_key !== null)
                {
                    // No SubSelect just a field
                    $l_selectSubSelect->setSelectQuery(str_replace($l_selectSubSelect->getSelectTable(), $l_joinedTableAlias . '.' . $l_selectSubSelect->getSelectTable(), $l_selectSubSelect->getSelectQuery()));
                }
                else
                {
                    if ($l_selectSubSelect->getSelectTable())
                    {
                        if (/*($this->m_property_rows[$l_select]['provides'] & C__PROPERTY__PROVIDES__VIRTUAL || $l_joinedTableAlias === null || $l_joinedTableAlias === 'obj_main') &&*/
                        $l_selectSubSelect->getSelectFieldObjectID()
                        )
                        {
                            $l_condition = $l_selectSubSelect->getSelectFieldObjectID() . ' = obj_main.isys_obj__id';
                        }
                        elseif ($l_conditionField = $l_selectSubSelect->getSelectReferenceKey())
                        {
                            $l_condition = ($l_joinedTableAlias ? $l_joinedTableAlias . '.' : '') .
                                $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] . ' = ' . $l_conditionField;
                        }
                        elseif ($l_conditionField = $l_selectSubSelect->getSelectFieldObjectID() && $l_joinedTableAlias !== 'obj_main' && $l_joinedTableAlias !== null)
                        {
                            $l_condition = $l_joinedTableAlias . '.' . $l_selectSubSelect->getSelectTable() . '__isys_obj__id = ' . $l_conditionField;
                        }
                        // @todo modify this if Config parameter for Grouping of the selection has been added. If grouping is active then skip this if so that the reference is the object-ID
                        elseif (($l_conditionField = $l_selectSubSelect->getSelectPrimaryKey()) && (bool) $this->m_property_rows[$l_select]['multi'] === true)
                        {
                            $l_condition = ($l_joinedTableAlias ? $l_joinedTableAlias . '.' : '') . $l_selectSubSelect->getSelectTable() . '__id = ' . $l_conditionField;
                        }
                        elseif ($l_selectSubSelect->getSelectTable())
                        {
                            $l_condition = ($l_joinedTableAlias ? $l_joinedTableAlias . '.' : '') . $l_selectSubSelect->getSelectTable() . '__id = ' .
                                $l_selectSubSelect->getSelectTable() . '__id';
                        } // if

                        if ($l_condition)
                        {
                            $l_selectCondition->addCondition(' AND ' . $l_condition);
                        } // if

                        // Add status check only if table contains 1 '_list '
                        if(substr_count($l_selectSubSelect->getSelectTable(), '_list') === 1)
                        {
                            $l_alias_table = (strpos($l_selectSubSelect->getSelectFieldObjectID(), '.') ?
                                substr($l_selectSubSelect->getSelectFieldObjectID(), 0, strpos($l_selectSubSelect->getSelectFieldObjectID(), '.')) . '.' :
                                (strpos($l_selectSubSelect->getSelectPrimaryKey(), '.') ?
                                    substr($l_selectSubSelect->getSelectPrimaryKey(), 0, strpos($l_selectSubSelect->getSelectPrimaryKey(), '.')) . '.' : ''));

                            $l_selectCondition->addCondition(
                                ' AND ' . $l_alias_table . $l_selectSubSelect->getSelectTable() . '__status = ' . $this->convert_sql_int(C__RECORD_STATUS__NORMAL)
                            );
                        } // if
                    }
                    elseif ($l_joinedTableAlias != 'obj_main')
                    {
                        $l_selectSubSelect->setSelectQuery(
                            str_replace(
                                $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD],
                                ($l_joinedTableAlias ? $l_joinedTableAlias . '.' : '') .
                                $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD],
                                $l_selectSubSelect->getSelectQuery()
                            )
                        );
                    }
                    elseif ($this->m_property_rows[$l_select]['provides'] & C__PROPERTY__PROVIDES__VIRTUAL && $l_selectSubSelect->getSelectFieldObjectID())
                    {
                        $l_condition = ' AND ' . $l_selectSubSelect->getSelectFieldObjectID() . ' = obj_main.isys_obj__id';
                        $l_selectCondition->addCondition(' AND ' . $l_condition);
                    }
                    elseif ($l_joinedTableAlias == 'obj_main')
                    {
                        $l_selectSubSelect->setSelectQuery(str_replace('isys_obj__', 'obj_main.isys_obj__', $l_selectSubSelect->getSelectQuery()));
                    } // if
                } // if

                $l_selectSubSelect
                    ->setSelectAlias($l_alias)
                    ->setSelectCondition($l_selectCondition);

                // Handling Selection with GROUP BY and wrap the selection with the GROUP_CONCAT
                if($l_selectSubSelect->getSelectGroupBy()->getGroupBy() && $p_for_list)
                {
                    if(!($l_selection = $l_selectSubSelect->getSelectGroupBy()->getGroupConcatSelection()))
                    {
                        $l_selection = trim(
                            substr(
                                $l_selectSubSelect->getSelectQuery(),
                                stripos($l_selectSubSelect->getSelectQuery(), 'SELECT') + 6,
                                ((substr_count($l_selectSubSelect->getSelectQuery(), 'FROM') > 1)? strrpos($l_selectSubSelect->getSelectQuery(), 'FROM'): strpos($l_selectSubSelect->getSelectQuery(), 'FROM')) - 6
                            )
                        );
                    }
                    if(strpos($l_selection, 'GROUP_CONCAT') === false)
                    {
                        switch($this->m_grouping)
                        {
                            case self::C__GROUPING__LIST:
                                $l_new_selection = 'CONCAT(\'<ul><li>\', GROUP_CONCAT(' . $l_selection . ' SEPARATOR \'</li><li>\'), \'</li></ul>\')';
                                break;
                            case self::C__GROUPING__COMMA:
                            default:
                                $l_new_selection = 'GROUP_CONCAT(' . $l_selection . ' SEPARATOR \', \')';
                                break;
                        }

                        $l_selectSubSelect->setSelectQuery(str_replace($l_selection, $l_new_selection, $l_selectSubSelect->getSelectQuery()));
                    } // if
                } // if

                $l_selects[$l_select] = $l_selectSubSelect;

                $this->set_property_key_selections($this->m_property_rows[$l_select]['class'] . '__' . $this->m_property_rows[$l_select]['key'], $l_selectSubSelect->getSelectQuery());
                continue;
            } // if

            $l_alias = 'j' . $this->retrieve_alias($l_table);

            // Then we check for special table-names inside the select.
            if (isset($this->m_aliases[$l_table]))
            {
                $l_alias = $this->m_aliases[$l_table];
            }
            elseif (isset($this->m_aliases[$l_table . '#main_obj']))
            {
                if ($l_table == 'isys_catg_connector_list')
                {
                    $l_alias = 'j' . $this->retrieve_alias($l_table, $l_table);
                }
                else
                {
                    $l_alias = 'j' . $this->m_aliases[$l_table . '#main_obj'];
                }
            }
            elseif ($l_table == 'isys_logbook')
            {
                $l_alias = 'j' . $this->retrieve_alias('isys_catg_logb_list', 'isys_logbook');
            }

            // @todo Add Config parameter for Grouping the selection
            if($this->m_property_rows[$l_select]['multi'])
            {
                switch($this->m_grouping)
                {
                    case self::C__GROUPING__LIST:
                        $l_field = 'CONCAT(\'<ul><li>\', GROUP_CONCAT(' . $l_alias . "." . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] . ' SEPARATOR \'</li><li>\'), \'</li></ul>\')';
                        break;
                    case self::C__GROUPING__COMMA:
                    default:
                        $l_field = 'GROUP_CONCAT(' . $l_alias . "." . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] . ' SEPARATOR \', \')';
                        break;
                }
            }
            else
            {
                $l_field = $l_alias . "." . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
            } // if

            $l_selects[$l_select] = $l_field . ' ' .
                (!$p_leave_field_identifiers ? " AS '" . $this->m_property_rows[$l_select]['title'] . "###" . $this->m_property_rows[$l_select][$l_cat] . "'"
                    : $this->convert_sql_text($this->m_property_rows[$l_select]['class'] . '__' . $this->m_property_rows[$l_select]['key']));

            $this->set_property_key_selections(
                $this->m_property_rows[$l_select]['class'] . '__' . $this->m_property_rows[$l_select]['key'],
                $l_alias . "." . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]
            );
        } // foreach

        // Clearing out all duplicate selects.
        return array_unique($l_selects);
    } // function

    /**
     * Method for creating the join-statements, based on the selected properties and conditions.
     *
     * @param   array   $p_property_ids The property-ID's of all the properties we need to join.
     * @param   boolean $l_main_obj
     *
     * @return  array
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    private function create_property_query_join(array $p_property_ids, $p_for_list = false)
    {
        // We need to know if all the necessary stuff has been done!
        if (!$this->m_prepared_data_for_query_construction)
        {
            $this->prepare_necessary_tasks($p_property_ids);
        } // if

        // We need this array to save "already joined" tables for saving a bit of performance.
        $l_already_joined_tables = $l_joins = [];

        if ($this->m_empty_values)
        {
            $l_join_type = "LEFT";
        }
        else
        {
            // Only root tables will be joined as INNER JOIN. Subquery joins will always be a LEFT JOIN
            $l_join_type = "INNER";
        } // if

        // Now we create the single JOIN's.
        foreach ($p_property_ids as $l_prop_id)
        {
            if (!isset($this->m_property_rows[$l_prop_id])) continue;

            $l_reference = $l_ref_field = $l_alias_sec = $l_alias_third = $l_alias_fourth = null;

            $l_prop_data = $this->m_property_rows[$l_prop_id];

            // We won't handle dynamic properties here.
            if ($l_prop_data['type'] == C__PROPERTY_TYPE__DYNAMIC && !isset($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]) || ($l_prop_data['provides'] & C__PROPERTY__PROVIDES__VIRTUAL))
            {
                continue;
            } // if

            {
                if ($l_prop_data['type'] == C__PROPERTY_TYPE__DYNAMIC)
                {
                    $l_table = $l_prop_data['table'];
                }
                else
                {
                    $l_table = current(explode('__', $l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
                }

                if ($l_table == 'isys_catg_custom_fields_list')
                {
                    $l_field_alias = $l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                    $l_field_key   = (strpos($l_field_alias, 'COMMENTARY') !== false) ? $l_field_alias : substr(
                        $l_field_alias,
                        strpos($l_field_alias, '_c_') + 1,
                        strlen($l_field_alias)
                    );
                    $l_field_type  = $l_prop_data['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                    $l_alias       = 'j' . $this->retrieve_alias($l_field_alias . '#isys_catg_custom_fields_list');

                    if ($l_field_type == C__PROPERTY__INFO__TYPE__DIALOG || $l_field_type == C__PROPERTY__INFO__TYPE__DIALOG_PLUS)
                    {
                        $l_join_type = 'LEFT';
                    }
                    elseif (!$this->m_empty_values && $l_join_type == 'LEFT')
                    {
                        $l_join_type = "INNER";
                    } // if

                    if (in_array($l_alias, $this->m_already_used_aliase)) continue;

                    $l_identifier = 'root-' . $l_prop_data['const'];

                    $l_joins_string = $l_join_type . ' JOIN ' . $l_table . ' AS ' . $l_alias . ' ON obj_main.isys_obj__id = ' . $l_alias . '.' . $l_table . '__isys_obj__id AND ' . $l_alias . '.' . $l_table . '__field_key = ' . $this->convert_sql_text(
                            $l_field_key
                        ) . ' ';

                    if (empty($this->m_parent_custom_field[$l_identifier]))
                    {
                        $l_description_alias = 'ROOT_' . $l_prop_data['const'];

                        $this->m_parent_custom_field[$l_identifier] = $l_description_alias . '.' . $l_table . '__data__id';

                        $l_joins_string = 'LEFT JOIN ' . $l_table . ' AS ' . $l_description_alias . ' ON obj_main.isys_obj__id = ' . $l_description_alias . '.' . $l_table . '__isys_obj__id
                                AND ' . $l_description_alias . '.' . $l_table . '__status = ' . $this->convert_sql_int(C__RECORD_STATUS__NORMAL) . '
                                AND ' . $l_description_alias . '.' . $l_table . '__isysgui_catg_custom__id = (SELECT isysgui_catg_custom__id FROM isysgui_catg_custom WHERE isysgui_catg_custom__const = ' . $this->convert_sql_text($l_prop_data['const']) . ')
                                AND ' . $l_description_alias . '.' . $l_table . '__field_type = ' . $this->convert_sql_text('commentary') . ' ' . $l_joins_string . '
                                AND ' . $l_alias . '.' . $l_table . '__data__id = ' . $this->m_parent_custom_field[$l_identifier] . ' ';
                    }
                    elseif ($this->m_parent_custom_field[$l_identifier] != $l_alias . '.' . $l_table . '__data__id')
                    {
                        $l_joins_string .= ' AND ' . $l_alias . '.' . $l_table . '__data__id = ' . $this->m_parent_custom_field[$l_identifier] . ' ';
                    } // if

                    $l_joins[] = $l_joins_string;

                    switch ($l_field_type)
                    {
                        case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                        case C__PROPERTY__INFO__TYPE__N2M:
                            $l_alias_sec = 'j' . $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj');

                            if (in_array($l_alias_sec, $this->m_already_used_aliase)) continue;

                            $l_joins[] = $l_join_type . ' JOIN isys_obj AS ' . $l_alias_sec . ' ON ' . $l_alias . '.' . $l_table . '__field_content = ' . $l_alias_sec . '.isys_obj__id ';
                            break;
                        case C__PROPERTY__INFO__TYPE__DIALOG:
                        case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                            $l_alias_sec = 'j' . $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom');

                            if (in_array($l_alias_sec, $this->m_already_used_aliase)) continue;

                            $l_joins[] = $l_join_type . ' JOIN isys_dialog_plus_custom AS ' . $l_alias_sec . ' ON ' . $l_alias . '.' . $l_table . '__field_content = ' . $l_alias_sec . '.isys_dialog_plus_custom__id ';
                            break;
                    } // switch
                    if ($l_alias !== null)
                    {
                        $this->m_already_used_aliase[] = $l_alias;
                    } // if
                    if ($l_alias_sec !== null)
                    {
                        $this->m_already_used_aliase[] = $l_alias_sec;
                    } // if
                    continue;
                } // if

                // We have to check for an existing "predefined" alias.
                if (isset($this->m_aliases[$l_table]))
                {
                    $l_alias = $this->m_aliases[$l_table];
                }
                else
                {
                    $l_alias = 'j' . $this->retrieve_alias($l_table);
                } // if

                if(isset($this->m_reference['root-' . $l_table]) && !$l_prop_data['data'][C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__VIRTUAL])
                {
                    $l_reference = $this->m_reference['root-' . $l_table];
                } // if

                if ($l_table == 'isys_logbook')
                {
                    if (!in_array('isys_catg_logb_list#isys_obj', $l_already_joined_tables))
                    {
                        $l_alias = 'j' . $this->retrieve_alias('isys_obj', 'isys_catg_logb_list');

                        if (in_array($l_alias, $this->m_already_used_aliase)) continue;

                        $l_already_joined_tables[] = 'isys_catg_logb_list#isys_obj';
                        $l_joins[$l_alias]         = $l_join_type . " JOIN isys_catg_logb_list AS " . $l_alias . " ON " . $l_alias . ".isys_catg_logb_list__isys_obj__id = obj_main.isys_obj__id";
                    }

                    if (!in_array('isys_logbook#isys_catg_logb_list', $l_already_joined_tables))
                    {
                        $l_alias     = 'j' . $this->retrieve_alias('isys_catg_logb_list', 'isys_logbook');
                        $l_alias_ref = 'j' . $this->retrieve_alias('isys_obj', 'isys_catg_logb_list');

                        if (in_array($l_alias, $this->m_already_used_aliase)) continue;

                        $l_already_joined_tables[] = 'isys_logbook#isys_catg_logb_list';
                        $l_joins[$l_alias]         = $l_join_type . " JOIN isys_logbook AS " . $l_alias . " ON " . $l_alias . ".isys_logbook__id = " . $l_alias_ref . ".isys_catg_logb_list__isys_logbook__id";
                    }
                    $this->m_already_used_aliase[] = $l_alias;
                    continue;
                }

                if(isset($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT]))
                {
                    /**
                     * @var $l_selectSubSelect idoit\Module\Report\SqlQuery\Structure\SelectSubSelect
                     */
                    $l_selectSubSelect = clone $l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT];
                    if($l_selectSubSelect->getSelectTable() && $l_selectSubSelect->getSelectFieldObjectID() != '')
                    {
                        // We don´t need to join any table
                        // $l_table = $l_selectSubSelect->getSelectTable();
                        continue;
                    } // if
                } // if


                if(isset($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
                {
                    $l_last_table = null;
                    $l_last_alias = null;

                    /**
                     * @var $l_selectJoin idoit\Module\Report\SqlQuery\Structure\SelectJoin
                     *
                     * @todo needs to be modified see property pos from isys_cmdb_dao_category_g_location
                     */
                    foreach($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN] AS $l_index => $l_selectJoin)
                    {
                        $l_selectJoin->setJoinType($l_join_type);
                        if($l_last_table === null)
                        {
                            $l_table = $l_selectJoin->getTable();
                            $l_last_alias = 'j' . $this->m_aliases[$l_table . '#isys_obj'];
                            $l_already_joined_tables[] = $l_table . '#isys_obj';

                            $l_selectJoin->setTableAlias($l_last_alias)->setOnLeftAlias($l_last_alias)->setOnRightALias('obj_main');
                        }
                        else
                        {
                            $l_table = $l_selectJoin->getTable();

                            if(in_array($l_table . '#' . $l_last_table, $l_already_joined_tables)) continue;

                            if($l_selectJoin->getRefTable()){
                                $l_last_table = $l_selectJoin->getRefTable();
                                $l_last_alias = 'j' . $this->m_aliases[$l_last_table . '#isys_obj'];
                            } // if
                            $l_alias = 'j' . $this->m_aliases[$l_table . '#' . $l_last_table];
                            $l_already_joined_tables[] = $l_table . '#' . $l_last_table;
                            $l_selectJoin->setTableAlias($l_alias)->setOnLeftAlias($l_last_alias)->setOnRightAlias($l_alias);
                            $l_last_alias = $l_alias;
                        }

                        if(!in_array($l_alias, $this->m_already_used_aliase))
                        {
                            $l_joins[$l_last_alias] = $l_selectJoin;
                            $this->m_already_used_aliase[] = $l_alias;
                        }

                        $l_last_table = $l_table;

                        if($l_table == 'isys_connection' || strpos($l_table, '_2_') ||
                            !(isset($l_prop_data['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT])))
                        {
                            continue;
                        }
                    }

                    continue;
                }

                if (!in_array($l_table . '#isys_obj', $l_already_joined_tables) && !in_array($l_alias, $this->m_already_used_aliase) && $l_alias != 'obj_main')
                {
                    if (isset($this->m_aliases[$l_table . '#main_obj']))
                    {
                        $l_alias = 'j' . $this->retrieve_alias('main_obj', $l_table);
                    } // if

                    if (!in_array($l_alias, $this->m_already_used_aliase))
                    {
                        $l_already_joined_tables[] = $l_table . '#isys_obj';

                        if (in_array($l_table, $l_already_joined_tables) && $l_reference !== null && $l_prop_data['multi'] > 0)
                        {
                            $l_join = $l_join_type . " JOIN " . $l_table . " AS " . $l_alias . " ON " . $l_alias . "." . $l_table . "__id = " . $l_reference .
                                ((substr_count($l_table, '_list') === 1) ?" AND " . $l_alias . "." . $l_table . "__status = " . $this->convert_sql_int(C__RECORD_STATUS__NORMAL): '');
                        }
                        else
                        {
                            $l_join                                = $l_join_type . " JOIN " . $l_table . " AS " . $l_alias . " ON " . $l_alias . "." . $l_table .
                                "__isys_obj__id = obj_main.isys_obj__id " .
                                ((substr_count($l_table, '_list') === 1) ?"AND " . $l_alias . '.' . $l_table . '__status = ' . $this->convert_sql_int(C__RECORD_STATUS__NORMAL): '');
                            $this->m_reference['root-' . $l_table] = $l_alias . "." . $l_table . '__id';
                        } // if
                        $l_joins[$l_alias]             = $l_join;
                        $this->m_already_used_aliase[] = $l_alias;
                    } // if
                } // if
            } // if
        } // foreach
        // Clearing out all duplicate joins.
        $l_joins = array_unique($l_joins);

        return $l_joins;
    }

    /**
     * Creates the condition for the query
     *
     * @param $p_conditions
     *
     * @return array
     * @author Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function create_property_query_condition($p_conditions, &$p_joins = [])
    {
        $l_return = [
            'conditions' => '',
            'joins'      => []
        ];

        // In this array we will save all the conditions for the report.
        $l_condition_complete = ' AND ';

        if ($this->m_query_as_report)
        {
            $l_alias_field = 'isys_obj__id';
        }
        else
        {
            $l_alias_field = 'isys_obj__title';
        } // if

        try
        {
            foreach ($p_conditions AS $l_condition_block)
            {
                if (is_array($l_condition_block))
                {
                    $l_inner_condition = '';

                    // Block Condition
                    foreach ($l_condition_block AS $l_condition)
                    {
                        // First Level
                        // Join from main object
                        $l_unit_id           = null;
                        $l_unit_property     = null;
                        $l_unit_field        = null;
                        $l_special_selection = false;
                        $l_special_field     = null;
                        $l_loc_condition_lft = null;
                        $l_loc_condition_rgt = null;
                        $l_selectCondition   = '';
                        $l_checkSelect       = null;
                        $l_checkCondition    = null;
                        $l_property          = $l_condition['property'];
                        list($l_category, $l_prop_key) = explode('-', $l_property);
                        $l_comparison = $l_condition['comparison'];
                        $l_unit       = $l_condition['unit'];
                        if (strpos($l_unit, '-') !== false)
                        {
                            list($l_unit_id, $l_unit_property) = explode('-', $l_unit);
                        }
                        $l_operator       = $l_condition['operator'];
                        $l_sub_conditions = $l_condition['subcnd'];

                        if (!isset($this->m_property_rows[$l_property]))
                        {
                            // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
                            $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                                    $l_category
                                ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key);
                            $l_row                = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                                ->get_row();

                            $l_cat_dao     = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                            $l_properties  = $l_cat_dao->get_properties();
                            $l_row['data'] = $l_properties[$l_row['key']];

                            // We save every row, because we will need them quite often in the upcoming code.
                            $this->m_property_rows[$l_property] = $l_row['id'];
                            if (!isset($this->m_property_rows[$l_row['id']]))
                            {
                                $this->m_property_rows[$l_row['id']] = $l_row;
                                $this->create_alias($l_row);
                            } // if

                            $l_select = $l_row['id'];
                            // Also we create table aliases for each possible join.

                            // Also we save some information, so that the logic will not try to join the "isys_obj" tables.
                            $this->m_aliases['isys_obj_type#isys_obj'] = $this->m_alias_cnt++;
                            $this->m_aliases['isys_obj#isys_obj']         = $this->m_alias_cnt++;
                        }
                        else
                        {
                            $l_select = $this->m_property_rows[$l_property];
                        }

                        if ($l_unit_id !== null)
                        {
                            $l_ignore_unit_field = false;
                            if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT]))
                            {
                                if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2]))
                                {
                                    $l_ignore_unit_field = ($this->m_property_rows[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0] !== null);
                                } // if
                            } // if
                            if (!isset($this->m_property_rows[$l_category . '-' . $l_unit_property]))
                            {
                                $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                                        $l_category
                                    ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_unit_property);
                                $l_row                = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                                    ->get_row();

                                $l_cat_dao     = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                                $l_properties  = $l_cat_dao->get_properties();
                                $l_row['data'] = $l_properties[$l_row['key']];

                                // We save every row, because we will need them quite often in the upcoming code.
                                $this->m_property_rows[$l_category . '-' . $l_unit_property] = $l_row;

                            }
                            if (!$l_ignore_unit_field)
                            {
                                $l_unit_field = $this->m_property_rows[$l_category . '-' . $l_unit_property][C__PROPERTY__DATA][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                            }
                        }

                        if (!isset($l_return['joins'][$l_select]))
                        {
                            //$l_return['joins'][$l_select] = $this->create_property_query_join([$l_select], false);
                        } // if

                        $l_table = current(explode('__', $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
                        $l_selectTitleField = null;

                        if ($l_table == 'isys_catg_custom_fields_list')
                        {
                            $l_field_alias = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                            $l_field_type  = $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                            $l_alias       = 'j' . $this->retrieve_alias($l_field_alias . '#isys_catg_custom_fields_list');

                            switch ($l_field_type)
                            {
                                case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                                case C__PROPERTY__INFO__TYPE__N2M:
                                    $l_alias_sec       = 'j' . $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj');
                                    $l_condition_field = $l_alias_sec . '.isys_obj__id';
                                    break;
                                case C__PROPERTY__INFO__TYPE__DIALOG:
                                case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                                    $l_alias_sec       = 'j' . $this->retrieve_alias('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom');
                                    $l_condition_field = $l_alias_sec . '.isys_dialog_plus_custom__id';
                                    break;
                                default:
                                    $l_condition_field = $l_alias . '.' . $l_table . '__field_content';
                                    break;
                            } // switch
                        }
                        else if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
                        {
                            $l_selectJoinData = clone $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
                            $l_amount = count($l_selectJoinData);
                            if($l_amount > 1)
                            {
                                $l_lastTable = $l_selectJoinData[$l_amount - 1]->getTable();;
                                $l_secondLastTable = $l_selectJoinData[$l_amount - 2]->getTable();
                            }
                            else
                            {
                                $l_lastTable = $l_selectJoinData[0]->getTable();
                                $l_secondLastTable = 'isys_obj';
                            }

                            $l_condition_field = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                            $l_alias = 'j' . $this->retrieve_alias($l_secondLastTable, $l_lastTable) . '.';
                            $l_condition_field = (isset($p_joins[$l_alias])? $l_alias: '') . $l_condition_field;
                        }
                        else if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT]))
                        {
                            /**
                             * @var $l_selectSubSelect \idoit\Module\Report\SqlQuery\Structure\SelectSubSelect
                             */
                            $l_selectSubSelect = clone $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT];

                            if($l_table == 'isys_obj' || $l_selectSubSelect->getSelectTable() === '')
                            {
                                $l_joinedTableAlias = 'obj_main.';
                                $l_selectSubSelect->setSelectQuery(str_replace('isys_obj__', 'obj_main.isys_obj__', $l_selectSubSelect->getSelectQuery()));
                            }
                            else
                            {
                                $l_joinedTableAlias = 'j' . $this->retrieve_alias($l_table) . '.';
                            } // if

                            if(!isset($p_joins[$l_joinedTableAlias]))
                            {
                                $l_joinedTableAlias = '';
                            } // if

                            if($l_selectSubSelect->getSelectTable())
                            {
                                if($l_selectSubSelect->getSelectReferenceKey())
                                {
                                    $l_condition_field = $l_joinedTableAlias . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                                }
                                elseif($l_selectSubSelect->getSelectPrimaryKey() && (bool) $this->m_property_rows[$l_select]['multi'] === true)
                                {
                                    $l_condition_field = $l_joinedTableAlias . $l_selectSubSelect->getSelectTable() . '__id';
                                }
                                elseif($l_selectSubSelect->getSelectFieldObjectID())
                                {
                                    $l_condition_field = $l_joinedTableAlias . $l_selectSubSelect->getSelectTable() . '__isys_obj__id = ';
                                }
                                elseif($l_selectSubSelect->getSelectTable())
                                {
                                    $l_condition_field = $l_joinedTableAlias . $l_selectSubSelect->getSelectTable() . '__id';
                                } // if
                            }
                            else
                            {
                                $l_condition_field = $l_joinedTableAlias . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                            } // if
                        }
                        // Condition for the sub select join
//                        else if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
//                        {
//                            $l_joinSelect = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
//                            $l_tables = $l_joinSelect->getJoinedTables();
//                            if(count($l_tables) > 1)
//                            {
//                                $l_table = $l_tables[0];
//                                $l_ref_table = $l_tables[1];
//                            }
//                            else
//                            {
//                                $l_table = $l_tables[0];
//                                $l_ref_table = 'isys_obj';
//                            } // if
//
//                            if(in_array($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE], $this->m_reference_types_condition))
//                            {
//                                $l_selectedField = 'reference';
//                                $l_selectTitleField = 'title';
//                            }
//                            else
//                            {
//                                $l_selectedField = 'title';
//                            }
//
//                            $l_alias = $this->m_aliases[$l_table . '#' . $l_ref_table . '#' . $this->m_property_rows[$l_select]['const'] . '-' . $this->m_property_rows[$l_select]['key']];
//                            $l_condition_field = 'j' . $l_alias . '.' . $l_selectedField;
//
//                            if($l_selectTitleField !== null)
//                            {
//                                $l_selectTitleField = 'j' . $l_alias . '.' . $l_selectTitleField;
//                            }
//                        }
                        else
                        {
                            if (($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] !== null && substr(
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                        0,
                                        5
                                    ) == 'isys_') || $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER
                            )
                            {
                                if (($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] === null || $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] === 'isys_obj') && $l_table == 'isys_obj')
                                {
                                    $l_ref_table_arr = explode('__', $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]);
                                    $l_ref_table     = $l_table;
                                    $l_table         = $l_ref_table_arr[0];
                                }
                                elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER)
                                {
                                    $l_ref_table = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0];
                                    if ($l_ref_table !== null)
                                    {
                                        if ($l_table == 'isys_obj' && $l_ref_table != 'isys_obj')
                                        {
                                            $l_puffer    = $l_table;
                                            $l_table     = $l_ref_table;
                                            $l_ref_table = $l_puffer;
                                        }
                                        elseif ($l_table != 'isys_obj' && $l_ref_table != 'isys_obj')
                                        {
                                            if ($l_ref_table != 'isys_connection')
                                            {
                                                $l_table     = $l_ref_table;
                                                $l_ref_table = 'isys_obj';
                                            }
                                        }
                                    }
                                    else
                                    {
                                        $l_ref_table = 'isys_obj';
                                    }
                                }
                                else
                                {
                                    $l_ref_table = $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0];
                                } // if

                                $l_alias = $this->retrieve_alias($l_table, $l_ref_table);

                                if ($l_comparison == 'under_location')
                                {
                                    $l_alias             = $this->retrieve_alias($l_ref_table, $l_table);
                                    $l_loc_condition_lft = 'j' . $l_alias . '.isys_catg_location_list__lft';
                                    $l_loc_condition_rgt = 'j' . $l_alias . '.isys_catg_location_list__rgt';
                                } // if

                                if ($l_ref_table == 'isys_connection')
                                {
                                    $l_condition_field = "job" . $l_alias . "." . $l_alias_field;
                                }
                                else if ($l_table == 'isys_contact')
                                {
                                    $l_alias           = $this->retrieve_alias('isys_obj', 'isys_contact_2_isys_obj');
                                    $l_condition_field = "job" . $l_alias . "." . $l_alias_field;
                                }
                                else if ($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'browser_location')
                                {
                                    $l_alias           = $this->retrieve_alias($l_table, 'isys_obj');
                                    $l_condition_field = "j" . $l_alias . "." . $l_alias_field;
                                }
                                else if (strpos($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_cable_connection__id') !== false)
                                {
                                    $l_alias           = $this->retrieve_alias('isys_cable_connection', 'isys_obj');
                                    $l_condition_field = "j" . $l_alias . "." . $l_alias_field;
                                }
                                else if (strpos(
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD],
                                        'isys_catg_connector_list__id'
                                    ) !== false
                                )
                                {
                                    if ($l_table != 'isys_catg_connector_list')
                                    {
                                        $l_alias           = $this->retrieve_alias('isys_catg_connector_list#isys_obj', $l_table);
                                        $l_condition_field = "j" . $l_alias . "." . $l_alias_field;
                                    }
                                    else
                                    {
                                        $l_alias           = $this->retrieve_alias('isys_obj', 'isys_catg_connector_list');
                                        $l_condition_field = "j" . $l_alias . "." . $l_alias_field;
                                    }
                                }
                                else
                                {
                                    if ($l_ref_table == 'isys_obj')
                                    {
                                        $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                                    }
                                    else
                                    {
                                        $l_condition_field = "j" . $l_alias . "." . $l_ref_table . "__id";
                                    } // if

                                    if (isset($this->m_text_fields[$this->m_property_rows[$l_select]['const']]))
                                    {
                                        if (isset($this->m_text_fields[$this->m_property_rows[$l_select]['const']][$this->m_property_rows[$l_select]['key']]))
                                        {
                                            $l_condition_field = substr(
                                                    $l_condition_field,
                                                    0,
                                                    (strpos($l_condition_field, '.') + 1)
                                                ) . $this->m_text_fields[$this->m_property_rows[$l_select]['const']][$this->m_property_rows[$l_select]['key']];
                                        } // if
                                    } // if
                                } // if
                            }
                            else
                            {
                                if ($l_table == 'isys_logbook')
                                {
                                    $l_alias = 'j' . $this->retrieve_alias('isys_catg_logb_list', $l_table);
                                }
                                else
                                {
                                    $l_alias = 'j' . $this->retrieve_alias($l_table);
                                } // if

                                // Then we check for special table-names inside the select.
                                if (isset($this->m_aliases[$l_table]))
                                {
                                    $l_alias = $this->m_aliases[$l_table];
                                } // if

                                if ($l_alias == 'obj_main' && (strpos($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_obj') > 0))
                                {
                                    $l_table = current(explode('__', $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
                                    $l_alias = 'j' . $this->retrieve_alias("isys_obj", $l_table);
                                } // if
                                $l_condition_field = $l_alias . "." . $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                            } // if
                        } // if

                        if(isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]) /* && (bool) $this->m_property_rows[$l_select]['multi'] == false*/)
                        {
                            $l_selectJoinData = clone $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
                            $l_amount = count($l_selectJoinData);

                            $l_newCondition = '';
                            $l_newJoin = '';

                            if($l_amount > 1)
                            {
                                /**
                                 * @var $l_selectJoin \idoit\Module\Report\SqlQuery\Structure\SelectJoin
                                 */
                                foreach($l_selectJoinData AS $l_index => $l_selectJoin)
                                {
                                    $l_joinTable = $l_selectJoin->getTable();
                                    if($l_index == 0)
                                    {
                                        $l_tableAlias = 'j' . $this->retrieve_alias($l_joinTable);
                                        if((bool) $this->m_property_rows[$l_select]['multi'] === true && $l_joinTable == 'isys_connection' || ($l_tableAlias !== 'j' && isset($p_joins[$l_tableAlias])))
                                        {
                                            $l_newCondition = $l_joinTable . '__id = ' . $l_tableAlias . '.' . $l_joinTable . '__id';
                                        }
                                        else
                                        {
                                            $l_newCondition = ' ' . $l_selectJoin->getOnLeft() . ' = obj_main.isys_obj__id ';
                                        } // if
                                        continue;
                                    }
                                    else
                                    {
                                        $l_newJoin .= ' LEFT JOIN ' . $l_joinTable . ' ON ' . $l_selectJoin->getOnLeft() . ' = ' . $l_selectJoin->getOnRight() . ' ';
                                    }

                                    if($l_index == ($l_amount - 1))
                                    {
                                        if((bool) $this->m_property_rows[$l_select]['multi'] === true)
                                        {
                                            $l_condition_field = $l_joinTable . '__id';
                                        }
                                        else{
                                            $l_condition_field = $l_selectJoin->getOnRight();
                                        }
                                    }
                                }
                            }
                            else
                            {
                                $l_tableAlias = 'j' . $this->retrieve_alias($l_selectJoinData[0]->getTable());
                                if((bool) $this->m_property_rows[$l_select]['multi'] === true && $l_tableAlias !== 'j' && isset($p_joins[$l_tableAlias]))
                                {
                                    $l_newCondition = $l_selectJoinData[0]->getTable() . '__id = ' . $l_tableAlias .
                                        '.' . $l_selectJoinData[0]->getTable() . '__id';
                                }
                                else
                                {
                                    $l_newCondition = ' ' . $l_selectJoinData[0]->getOnLeft() . ' = obj_main.isys_obj__id ';
                                } // if
                                if(strpos($l_condition_field, '.') !== false)
                                {
                                    $l_condition_field = substr($l_condition_field, strpos($l_condition_field, '.') + 1);
                                }
                            } // if

                            $l_newSelect = ' SELECT |placeholder| FROM ' . $l_selectJoinData[0]->getTable() . ' ';

                            $l_checkSelect = \idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                                $l_newSelect . $l_newJoin,
                                $l_selectJoinData[0]->getTable(),
                                $l_selectJoinData[0]->getTable() . '__id',
                                '',
                                '',
                                '',
                                \idoit\Module\Report\SqlQuery\Structure\SelectCondition::factory([' AND ' . $l_newCondition]),
                                \idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory([$l_selectJoinData[0]->getOnLeft()]),
                                ''
                            );
                        }

                        if ($l_comparison == 'subcnd' && is_array($l_sub_conditions))
                        {
                            // Sub Levels
                            // Join from category to referenced object
                            $this->m_referenced_fields[$l_property] = $l_condition_field;
                            $l_sub_inner_condition_string           = $this->handle_sub_conditions($l_property, $l_sub_conditions, $l_condition_field);
                            $l_inner_condition .= ' (' . $l_sub_inner_condition_string . ') ' . $l_operator;
                        }
                        else
                        {
                            if (isset($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES]) && $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG || $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG_PLUS)
                            {
                                if ((strpos($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                                            $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                            'cats'
                                        ) !== false) && strpos(
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                        '_list'
                                    ) !== false && strpos($l_condition_field, '__id_') === false
                                )
                                {
                                    $l_condition_field = str_replace('__id', '__title', $l_condition_field);
                                } // if
                            }
                            elseif ($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__TEXT && strpos(
                                    $l_condition_field,
                                    '__id'
                                ) > 0
                            )
                            {
                                $l_condition_field = str_replace('__id', '__title', $l_condition_field);
                            } // if

                            if (isset($this->m_property_rows[$l_select][C__PROPERTY__DATA][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK]) && $l_unit_id !== null)
                            {
                                if ($this->m_property_rows[$l_select][C__PROPERTY__DATA][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'convert')
                                {
                                    $l_method = $this->m_property_rows[$l_select][C__PROPERTY__DATA][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0];
                                    // convert value

                                    // We have to use the raw value if its from a select join property
                                    if(strpos($l_condition_field, '.title') !== false)
                                    {
                                        $l_condition_field = str_replace('.title', '.title_raw', $l_condition_field);
                                    } // if

                                    $l_condition['value'] = isys_convert::$l_method($l_condition['value'], $l_unit_id);
                                } // if
                            } // if

                            if ($l_condition['comparison'] == 'LIKE %...%')
                            {
                                $l_condition['comparison'] = 'LIKE';
                                $l_condition['value']      = '%' . $l_condition['value'] . '%';
                            } // if

                            if ($l_condition['comparison'] == 'NOT LIKE %...%')
                            {
                                $l_condition['comparison'] = 'NOT LIKE';
                                $l_condition['value']      = '%' . $l_condition['value'] . '%';
                            } // if

                            if (strpos($l_condition['value'], '[') === 0)
                            {
                                $l_condition['value'] = str_replace(
                                    [
                                        '[',
                                        ']',
                                        '"'
                                    ],
                                    '',
                                    $l_condition['value']
                                );
                            } // if

                            if (strpos($l_condition['value'], ',') !== false)
                            {
                                if ($l_condition['comparison'] == '=')
                                {
                                    $l_condition['comparison'] = 'IN';
                                }
                                else
                                {
                                    $l_condition['comparison'] = 'NOT IN';
                                } // if
                                $l_condition['value'] = explode(',', $l_condition['value']);

                                if($l_checkSelect)
                                {
                                    $l_checkSelect->setSelectGroupBy(null);
                                    $l_condition['value'] = "'" . implode(",", $l_condition['value']) . "'";
                                    $l_inner_condition .= ' ' . $l_condition['value'] . ' ' . $l_condition['comparison'] . ' (' . str_replace('|placeholder|', $l_condition_field, $l_checkSelect) . ')' .
                                        (isset($l_condition['operator']) ? $l_condition['operator'] : '');
                                }
                                else
                                {
                                    $l_condition['value'] = "'" . implode("','", $l_condition['value']) . "'";
                                    $l_inner_condition .= ' (' . $l_selectCondition . ' ' . $l_condition_field . ' ' . $l_condition['comparison'] . ' (' .
                                        $l_condition['value'] . ')) ' . (isset($l_condition['operator']) ? $l_condition['operator'] : '');
                                } // if
                            }
                            else
                            {
                                if ($l_comparison == 'under_location')
                                {
                                    $l_inner_condition .= ' (' . $l_loc_condition_lft . ' > ' . '(SELECT isys_catg_location_list__lft FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                                            $l_condition['value']
                                        ) . ') AND ' . $l_loc_condition_rgt . ' < ' . '(SELECT isys_catg_location_list__rgt FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                                            $l_condition['value']
                                        ) . ') ';

                                }
                                else
                                {
                                    if($this->m_property_rows[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['multiselection'] ||
                                        $this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER)
                                    {
                                        if ($l_condition['comparison'] == '=')
                                        {
                                            $l_condition['comparison'] = 'IN';
                                        }
                                        else
                                        {
                                            $l_condition['comparison'] = 'NOT IN';
                                        } // if

                                        if($l_checkSelect)
                                        {
                                            if(trim($l_condition['value']) == '')
                                            {
                                                if($l_condition['comparison'] == 'IN')
                                                {
                                                    $l_condition['comparison'] = ' IS NULL';
                                                }
                                                else
                                                {
                                                    $l_condition['comparison'] = ' IS NOT NULL';
                                                }

                                                $l_inner_condition .= ' ((' .
                                                    str_replace('|placeholder|', 'GROUP_CONCAT(' . $l_condition_field . ')', $l_checkSelect) . ') ' . $l_condition['comparison'];
                                            }
                                            else
                                            {
                                                $l_inner_condition .= ' (' . $this->convert_sql_text($l_condition['value']) . ' ' . $l_condition['comparison'] . ' (' .
                                                    str_replace('|placeholder|', $l_condition_field, $l_checkSelect->setSelectGroupBy(null)) . ') ';
                                            }
                                        }
                                        else
                                        {
                                            $l_inner_condition .= ' (' . $l_condition_field . ' ' . $l_condition['comparison'] . ' (' .
                                                $this->convert_sql_text($l_condition['value']) . ') ';
                                        } // if
                                    }
                                    else
                                    {
                                        if($l_selectTitleField !== null && !is_numeric($l_condition['value']) && $l_condition['value'] !== '' &&
                                            (strpos($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                                                    $this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                                    'cats'
                                                ) !== false) && strpos($this->m_property_rows[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], '_list') !== false)
                                        {
                                            $l_inner_condition .= ' (' . $l_selectTitleField . ' ' . $l_condition['comparison'] . ' ' .
                                                (($l_condition['comparison'] === 'IS NULL' || $l_condition['comparison'] === 'IS NOT NULL') ? '' :
                                                    (is_numeric($l_condition['value']) ? $this->convert_sql_int($l_condition['value']) :
                                                        $this->convert_sql_text($l_condition['value']))) . ' ';
                                        }
                                        else
                                        {
                                            $l_condition_part = $l_condition_field . ' ' . $l_condition['comparison'] . ' ' .
                                                (($l_condition['comparison'] === 'IS NULL' || $l_condition['comparison'] === 'IS NOT NULL') ? '' : (is_numeric(
                                                    $l_condition['value']
                                                ) ? $this->convert_sql_int($l_condition['value']) : $this->convert_sql_text($l_condition['value']))) . ' ';

                                            $l_inner_condition_extended = '';

                                            if ((is_numeric($l_condition['value']) || $l_condition['value'] === '') && $l_condition['comparison'] !== 'IS NULL' &&
                                                $l_condition['comparison'] !== 'IS NOT NULL')
                                            {
                                                if ((int) $l_condition['value'] === 0)
                                                {
                                                    if ($l_condition['comparison'] == '=')
                                                    {
                                                        $l_comparison_addition = ' IS NULL ';
                                                        $l_log_operator        = ' OR ';
                                                    }
                                                    else
                                                    {
                                                        $l_comparison_addition = ' IS NOT NULL';
                                                        $l_log_operator        = ' AND ';
                                                    } // if
                                                    $l_inner_condition_extended = $l_log_operator . $l_condition_field . ' ' . $l_comparison_addition . ' ';
                                                } // if
                                            } // if

                                            if($l_checkSelect)
                                            {
                                                $l_checkCondition = $l_checkSelect->getSelectCondition();
                                                $l_checkCondition->addCondition(' AND (' . $l_condition_part . $l_inner_condition_extended . ')');
                                                $l_checkSelect->setSelectCondition($l_checkCondition);
                                                $l_checkSelect->setSelectLimit(1);
                                                $l_inner_condition .= '(' . str_replace('|placeholder|', '1', $l_checkSelect);
                                            }
                                            else
                                            {
                                                $l_inner_condition .= ' ( (' . $l_condition_part . $l_inner_condition_extended . ')';
                                            } // if

                                        } // if
                                    } // if
                                } // if
                                $l_inner_condition .= ') ' . (isset($l_condition['operator']) ? $l_condition['operator'] : '');
                            } // if

                            if ($l_unit_field !== null)
                            {
                                $l_inner_condition .= ' AND ' . $l_alias . '.' . $l_unit_field . ' = ' . $this->convert_sql_text($l_unit_id) . ' ';
                            } // if
                        } // if
                    } // foreach
                    $l_condition_complete .= ' (' . $l_inner_condition . ') ';
                }
                else
                {
                    // Block comparison
                    $l_condition_complete .= $l_condition_block;
                } // if
            } // foreach
        }
        catch (Exception $e)
        {
            isys_notify::error($e->getMessage());
        } // try
        $l_return['conditions'] = (trim($l_condition_complete) == 'AND') ? '' : $l_condition_complete;

        return $l_return;
    } // function

    /**
     * Method for retrieving properties.
     *
     * @todo    Add custom categories
     *
     * @param   mixed   $p_property_id May be a array or an integer.
     * @param   mixed   $p_catg_id     May be a array or an integer.
     * @param   mixed   $p_cats_id     May be a array or an integer.
     * @param   integer $p_provides
     * @param   string  $p_condition
     * @param   boolean $p_dynamic_properties
     * @param   mixed   $p_catg_custom_id
     *
     * @return  isys_component_dao_result
     * @author  Leonard Fischer <lfischer@i-doit.org>
     */
    public function retrieve_properties($p_property_id = null, $p_catg_id = null, $p_cats_id = null, $p_provides = null, $p_condition = "", $p_dynamic_properties = false, $p_catg_custom_id = null)
    {
        $l_condition = " WHERE TRUE ";

        if ($p_property_id !== null)
        {
            if (is_array($p_property_id))
            {
                $l_condition .= "AND isys_property_2_cat__id " . $this->prepare_in_condition($p_property_id) . " ";
            }
            else if (is_numeric($p_property_id))
            {
                $l_condition .= "AND isys_property_2_cat__id = " . $this->convert_sql_id($p_property_id) . " ";
            } // if
        } // if

        if ($p_catg_id !== null)
        {
            if (is_array($p_catg_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_catg__id " . $this->prepare_in_condition($p_catg_id) . " ";
            }
            else if (is_numeric($p_catg_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_catg__id = " . $this->convert_sql_id($p_catg_id) . " ";
            } // if
        } // if

        if ($p_cats_id !== null)
        {
            if (is_array($p_cats_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_cats__id " . $this->prepare_in_condition($p_cats_id) . " ";
            }
            else if (is_numeric($p_cats_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_cats__id = " . $this->convert_sql_id($p_cats_id) . " ";
            } // if
        } // if

        if ($p_catg_custom_id !== null)
        {
            if (is_array($p_catg_custom_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_catg_custom__id " . $this->prepare_in_condition($p_catg_custom_id) . " ";
            }
            else if (is_numeric($p_catg_custom_id))
            {
                $l_condition .= "AND isys_property_2_cat__isysgui_catg_custom__id = " . $this->convert_sql_id($p_catg_custom_id) . " ";
            } // if
        }

        if ($p_provides !== null && $p_provides > 0)
        {
            $l_condition .= "AND isys_property_2_cat__prop_provides & " . $this->convert_sql_int($p_provides) . " ";
        } // if

        if (!$p_dynamic_properties)
        {
            $l_condition .= "AND isys_property_2_cat__prop_type = " . C__PROPERTY_TYPE__STATIC . " ";
        } // if

        $l_category_join = " LEFT JOIN isysgui_catg ON isysgui_catg__id = isys_property_2_cat__isysgui_catg__id " .
            " LEFT JOIN isysgui_cats ON isysgui_cats__id = isys_property_2_cat__isysgui_cats__id " .
            " LEFT JOIN isysgui_catg_custom ON isysgui_catg_custom__id = isys_property_2_cat__isysgui_catg_custom__id ";

        // We rename the fields for easier usage.
        $l_sql = "SELECT isys_property_2_cat__id AS 'id', " . "isys_property_2_cat__isysgui_catg__id AS 'catg', " . "isys_property_2_cat__isysgui_cats__id AS 'cats', " .
            "isys_property_2_cat__isysgui_catg_custom__id AS 'catg_custom', " . "isys_property_2_cat__cat_const AS 'const', " . "isys_property_2_cat__prop_type AS 'type', " .
            "isys_property_2_cat__prop_title AS 'title', " . "isys_property_2_cat__prop_key AS 'key', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__list_multi_value " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__list_multi_value " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN isysgui_catg_custom__list_multi_value END) AS 'multi', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__class_name " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__class_name " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN isysgui_catg_custom__class_name END) AS 'class', " .
            "(CASE WHEN isys_property_2_cat__isysgui_catg__id IS NOT NULL THEN isysgui_catg__source_table " .
            "WHEN isys_property_2_cat__isysgui_cats__id IS NOT NULL THEN isysgui_cats__source_table " .
            "WHEN isys_property_2_cat__isysgui_catg_custom__id IS NOT NULL THEN 'isys_catg_custom_fields_list' END) AS 'table', " .
            "isys_property_2_cat__prop_provides AS provides ".
            "FROM isys_property_2_cat " .
            $l_category_join . $l_condition;

        return $this->retrieve($l_sql . $p_condition . ";");
    } // function

    /**
     * This method retrieves all categories which have at least one properties that fit the given provide-parameter.
     *
     * @param   string  $p_category_type
     * @param   integer $p_provide
     * @param   boolean $p_dynamic_property
     *
     * @return  isys_component_dao_result
     */
    public function retrieve_categories_by_provide($p_provide, $p_category_type, $p_dynamic_property = false)
    {
        $l_sql = 'SELECT * FROM isys_property_2_cat
			LEFT JOIN isysgui_catg ON isysgui_catg__id = isys_property_2_cat__isysgui_catg__id
			LEFT JOIN isysgui_cats ON isysgui_cats__id = isys_property_2_cat__isysgui_cats__id
			LEFT JOIN isysgui_catg_custom ON isysgui_catg_custom__id = isys_property_2_cat__isysgui_catg_custom__id
			WHERE isys_property_2_cat__isysgui_cat' . $p_category_type . '__id > 0';

        if ($p_provide !== null)
        {
            $l_sql .= ' AND isys_property_2_cat__prop_provides & ' . $this->convert_sql_int($p_provide);
        } // if

        if (!$p_dynamic_property)
        {
            $l_sql .= ' AND isys_property_2_cat__prop_type = ' . $this->convert_sql_int(C__PROPERTY_TYPE__STATIC);
        } // if

        return $this->retrieve($l_sql . ' GROUP BY isys_property_2_cat__isysgui_cat' . $p_category_type . '__id;');
    } // function

    /**
     * Method which renews the property_2_cat table
     *
     * @return mixed
     * @author Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function rebuild_properties()
    {
        global $g_comp_database, $g_dirs;

        $l_log = isys_log_migration::get_instance();

        if ($l_log->get_log_file() == '')
        {
            $l_log_file = $g_dirs['temp'] . 'prop_' . date('Y-m-d_H_i_s', time()) . '.txt';
            $l_log->set_log_file($l_log_file);
        } // if

        $l_log->set_log_level(isys_log::C__ALL);

        $l_upd_prop = isys_factory::get_instance('isys_update_property_migration');

        $l_result = $l_upd_prop->set_database($g_comp_database)
            ->reset_property_table()
            ->collect_category_data()
            ->prepare_sql_queries('g')
            ->prepare_sql_queries('s')
            ->prepare_sql_queries('g_custom')
            ->execute_sql()
            ->get_results();

        $l_log->flush_log();

        return $l_result;
    } // function

    /**
     * Retrieves data by property chain.
     *
     * @param   integer $p_obj_id
     * @param   string  $p_chain
     *
     * @return  mixed
     * @author  Van Quyen Hoang <qhoang@i-doit.org>
     */
    public function retrieve_chained_obj_id($p_obj_id, $p_chain)
    {
        $l_return       = null;
        $l_assigned_key = null;
        $l_selects      = null;
        $l_joins        = [];
        $l_sub_joins    = [];
        $this->reset();
        if (strpos($p_chain, '--') !== false)
        {
            // more than one level chain
            $l_chain_arr      = explode('--', $p_chain);
            $l_obj_lvl_select = $l_obj_select = [];
            foreach ($l_chain_arr AS $l_key => $l_ref_key)
            {
                list($l_category_const, $l_prop_key) = explode('-', $l_ref_key);

                if (!isset($this->m_property_rows[$l_ref_key]))
                {
                    $l_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                            $l_category_const
                        ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key);
                    $l_row       = $this->retrieve_properties(null, null, null, null, $l_condition)
                        ->get_row();

                    $l_cat_dao     = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                    $l_properties  = $l_cat_dao->get_properties();
                    $l_row['data'] = $l_properties[$l_row['key']];

                    $this->m_property_rows[$l_row['id']] = $l_row;
                    $this->m_property_rows[$l_ref_key]   = $l_row['id'];
                }
                else
                {
                    $l_row = $this->m_property_rows[$this->m_property_rows[$l_ref_key]];
                } // if

                if ($l_key == 0 && $l_assigned_key === null)
                {
                    $l_prop_data['data'] = $l_row['data'];

                    $this->create_alias($l_prop_data);
                    $l_selects      = $this->create_property_query_select([$l_row['id']]);
                    $l_obj_select   = str_replace('title', 'id', $l_selects[$l_row['id']]);
                    $l_joins        = $this->create_property_query_join([$l_row['id']]);
                    $l_assigned_key = $l_row['id'];
                }
                else
                {
                    $l_lvls_arr = [
                        $l_assigned_key => isys_format_json::encode([$l_row['id']])
                    ];

                    $l_condition_field = substr($l_obj_select, 0, strpos($l_obj_select, ' '));

                    $l_lvls_select = $this->create_property_query_lvls_select($l_lvls_arr, $l_selects);
                    $l_sub_joins[] = array_pop($this->create_property_query_join_lvls($this->m_property_order, $l_selects, $l_obj_lvl_select, false, $l_condition_field));
                    $l_assigned_key .= '--' . $l_row['id'];

                    $l_obj_select = array_pop($l_lvls_select);
                }
            }

            if (count($l_sub_joins) > 0)
            {
                foreach ($l_sub_joins AS $l_lvl_join)
                {
                    $l_joins = array_merge($l_joins, $l_lvl_join);
                }
            }
        }
        else
        {
            list($l_category_const, $l_prop_key) = explode('-', $p_chain);

            if (!isset($this->m_property_rows[$p_chain]))
            {
                $l_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                        $l_category_const
                    ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key);
                $l_row       = $this->retrieve_properties(null, null, null, null, $l_condition)
                    ->get_row();

                $l_cat_dao                           = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                $l_properties                        = $l_cat_dao->get_properties();
                $l_row['data']                       = $l_properties[$l_row['key']];
                $this->m_property_rows[$l_row['id']] = $l_row;
                $this->m_property_rows[$p_chain]     = $l_row['id'];
            }
            else
            {
                $l_row = $this->m_property_rows[$this->m_property_rows[$p_chain]];
            } // if

            $l_prop_data['data'] = $l_row['data'];

            $this->create_alias($l_prop_data);
            $l_selects = $this->create_property_query_select([$l_row['id']]);

            $l_obj_select = str_replace('title', 'id', $l_selects[$l_row['id']]);
            $l_joins      = $this->create_property_query_join([$l_row['id']]);
        } // if

        $l_sql = "SELECT \n" . $l_obj_select . " \n\n" . "FROM isys_obj AS obj_main \n" . ((count($l_joins) > 0) ? implode(
                " \n",
                $l_joins
            ) : '') . " \n\n" . "WHERE TRUE \n" . "AND obj_main.isys_obj__id = " . $this->convert_sql_id($p_obj_id);

        try
        {
            $l_res = $this->retrieve($l_sql);

            if (count($l_res) > 0)
            {
                $l_return = [];

                while ($l_obj_row = $l_res->get_row())
                {
                    $l_return[] = array_pop($l_obj_row);
                } // while
            } // if
        }
        catch (Exception $e)
        {
            isys_notify::error('An Error occurred: ' . $e->getMessage() . ' File: ' . $e->getFile());
        } // try

        return $l_return;
    } // function

    /**
     * Method for creating an alias to the given properties.
     *
     * @param   array $p_props
     *
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @todo    Change the alias namespaces to "a_table.a_column-b_table.b_column => counter".
     * @todo    Also change the parameters to "$p_from" and "$p_to".
     */
    protected function create_alias(array $p_props)
    {
        if ($p_props['type'] == C__PROPERTY_TYPE__DYNAMIC)
        {
            $l_table = $p_props['table'];
        }
        else
        {
            $l_table = current(explode('__', $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
        }

        if(isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
        {
            if(count($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
            {
                $l_last_table = '';
                /**
                 * @var $l_selectJoin \idoit\Module\Report\SqlQuery\Structure\SelectJoin
                 */
                foreach($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN] AS $l_selectJoin)
                {
                    $l_join_table = $l_selectJoin->getTable();
                    if($l_selectJoin->getRefTable())
                    {
                        $l_last_table = $l_selectJoin->getRefTable();
                    }

                    if (!isset($this->m_aliases[$l_join_table . '#isys_obj']))
                    {
                        $this->m_aliases[$l_join_table . '#isys_obj'] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    }

                    if ($l_last_table != '' && !isset($this->m_aliases[$l_join_table . '#' . $l_last_table]))
                    {
                        $this->m_aliases[$l_join_table . '#' . $l_last_table] = $this->m_alias_cnt;
                        //$this->m_aliases[$l_join_table . '#' . $l_last_table . '#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if

                    $l_last_table = $l_join_table;
                }
                $this->m_property_order[] = $p_props['id'];
                return;
            }
        }

        if ($l_table == 'isys_catg_custom_fields_list')
        {
            $this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_catg_custom_fields_list#isys_obj'] = $this->m_alias_cnt;
            $this->m_alias_cnt++;

            if (isset($p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE]) &&
                ($p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE] == 'f_popup' || $p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE] == 'popup'))
            {
                if ($p_props['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER)
                {
                    if (!isset($this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_obj#isys_catg_custom_fields_list']))
                    {
                        $this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_obj#isys_catg_custom_fields_list'] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if
                }
                else
                {
                    if (!isset($this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_dialog_plus_custom#isys_catg_custom_fields_list']))
                    {
                        $this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_dialog_plus_custom#isys_catg_custom_fields_list'] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if
                } // if
            } // if
            $this->m_property_order[] = $p_props['id'];
            return;
        }
        elseif (isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT]))
        {
            if($p_props['data'][C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__VIRTUAL] === true) return;

            /**
             * @var $l_selectSubSelect idoit\Module\Report\SqlQuery\Structure\SelectSubSelect
             */
            $l_selectSubSelect = clone $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT];
            $l_join_table = $l_selectSubSelect->getSelectTable();

            if($l_join_table)
            {
                if (!isset($this->m_aliases[$l_join_table . '#isys_obj']))
                {
                    $this->m_aliases[$l_join_table . '#isys_obj'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                }
            }
        }
//        elseif (isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
//        {
//            /**
//             * @var $l_joinSubSelect idoit\Module\Report\SqlQuery\Structure\JoinSubSelect
//             */
//            $l_joinSubSelect = $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
//
//            $l_join_query = $l_joinSubSelect;
//            $l_join_tables = $l_join_query->getJoinedTables();
//            foreach($l_join_tables AS $l_jtable)
//            {
//                if($l_join_query)
//                {
//                    $l_alias_addition = '#' . $p_props['const'] . '-' . $p_props['key'];
//                }
//                else
//                {
//                    $l_alias_addition = '';
//                }
//
//                if (!isset($this->m_aliases[$l_jtable . '#main_obj']))
//                {
//                    $this->m_aliases[$l_jtable . '#main_obj'] = $this->m_alias_cnt;
//                    $this->m_alias_cnt++;
//                } // if
//
//                if (!isset($this->m_aliases[$l_jtable . '#isys_obj' . $l_alias_addition]))
//                {
//                    $this->m_aliases[$l_jtable . '#isys_obj' . $l_alias_addition] = $this->m_alias_cnt;
//                    if($l_alias_addition === '')
//                    {
//                        $this->m_alias_cnt++;
//                    }
//                } // if
//
//                if (!isset($this->m_aliases['isys_obj#' . $l_jtable . $l_alias_addition]))
//                {
//                    $this->m_aliases['isys_obj#' . $l_jtable . $l_alias_addition] = $this->m_alias_cnt;
//                    if($l_alias_addition === '')
//                    {
//                        $this->m_alias_cnt++;
//                    }
//                } // if
//                foreach ($l_join_tables AS $l_jtable2)
//                {
//                    if ($l_jtable !== $l_jtable2)
//                    {
//                        if (!isset($this->m_aliases[$l_jtable . '#' . $l_jtable2 . $l_alias_addition]))
//                        {
//                            $this->m_aliases[$l_jtable . '#' . $l_jtable2 . $l_alias_addition] = $this->m_alias_cnt;
//                            //$this->m_reference[$l_jtable.'#'.$l_jtable2] = 'j' . $this->m_alias_cnt . '.' . $l_join_reference;
//                            if($l_alias_addition === '')
//                            {
//                                $this->m_alias_cnt++;
//                            }
//                        } // if
//
//                        if (!isset($this->m_aliases[$l_jtable2 . '#' . $l_jtable . $l_alias_addition]))
//                        {
//                            $this->m_aliases[$l_jtable2 . '#' . $l_jtable . $l_alias_addition] = $this->m_alias_cnt;
//                            //$this->m_reference[$l_jtable2.'#'.$l_jtable] = 'j' . $this->m_alias_cnt . '.' . $l_join_reference;
//                            if($l_alias_addition === '')
//                            {
//                                $this->m_alias_cnt++;
//                            }
//                        } // if
//                    } // if
//                } // foreach
//            } // foreach
//            if($l_join_query)
//            {
//                $this->m_alias_cnt++;
//                $this->m_property_order['joins'][] = $p_props['id'];
//            }
//            else
//            {
//                $this->m_property_order[] = $p_props['id'];
//            } // if
//
//            return;
//        }
        elseif ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] !== null && strpos(
                ' ' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                'isys_'
            ) && !isset($this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table])
        )
        {
            if ($l_table == 'isys_logbook')
            {
                if (!isset($this->m_aliases['isys_catg_logb_list#isys_obj']))
                {
                    $this->m_aliases['isys_catg_logb_list#isys_obj'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_aliases['isys_logbook#isys_catg_logb_list'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
            } // if

            if (!isset($this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table]))
            {
                $this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if

            if ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] == 'isys_contact')
            {
                if (!isset($this->m_aliases['isys_contact_2_isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]]))
                {
                    $this->m_aliases['isys_contact_2_isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
                if (!isset($this->m_aliases['isys_contact_2_isys_obj#isys_obj']))
                {
                    $this->m_aliases['isys_contact_2_isys_obj#isys_obj'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
            } // if

            if (((strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                        $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                        'cats'
                    ) !== false) && strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], '_list') !== false)
            )
            {
                if (!isset($this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#isys_obj']))
                {
                    $this->m_aliases[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#isys_obj'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                }
                if (!isset($this->m_aliases['isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]]))
                {
                    $this->m_aliases['isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0]] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                }
            }
            $this->m_property_order[] = $p_props['id'];
            if ($l_table == 'isys_logbook') return;
        }
        elseif ($l_table == 'isys_catg_logb_list' || $l_table == 'isys_logbook')
        {
            if (!isset($this->m_aliases['isys_catg_logb_list#isys_obj']))
            {
                $this->m_aliases['isys_catg_logb_list#isys_obj'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
                $this->m_aliases['isys_logbook#isys_catg_logb_list'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
                $this->m_property_order[] = $p_props['id'];
            } // if

            if ($l_table == 'isys_logbook') return;
        }
        elseif ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] == $l_table . '__isys_obj__id')
        {
            if ($l_table == 'isys_cats_net_ip_addresses_list')
            {
                if (!isset($this->m_aliases['isys_catg_ip_list#isys_obj']))
                {
                    $this->m_aliases['isys_catg_ip_list#isys_obj'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_aliases['isys_cats_net_ip_addresses_list#isys_catg_ip_list'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_aliases['isys_obj#isys_cats_net_ip_addresses_list'] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_property_order[] = $p_props['id'];
                    return;
                } // if
            } // if
        }
        elseif (strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_cable_connection__id') !== false)
        {
            if (!isset($this->m_aliases[$l_table . '#isys_cable_connection']))
            {
                $this->m_aliases[$l_table . '#isys_cable_connection'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_cable_connection#' . $l_table]))
            {
                $this->m_aliases['isys_cable_connection#' . $l_table] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_obj#isys_cable_connection']))
            {
                $this->m_aliases['isys_obj#isys_cable_connection'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_cable_connection#isys_obj']))
            {
                $this->m_aliases['isys_cable_connection#isys_obj'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
        }
        elseif (strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_catg_connector_list__id') !== false)
        {
            if ($l_table !== 'isys_catg_connector_list' && !isset($this->m_aliases[$l_table . '#isys_catg_connector_list#isys_obj']))
            {
                $this->m_aliases[$l_table . '#isys_catg_connector_list#isys_catg_connector_list'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
                $this->m_aliases[$l_table . '#isys_catg_connector_list#isys_obj'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases[$l_table . '#isys_catg_connector_list']))
            {
                $this->m_aliases[$l_table . '#isys_catg_connector_list'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_catg_connector_list#' . $l_table]))
            {
                $this->m_aliases['isys_catg_connector_list#' . $l_table] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_catg_connector_list#isys_catg_connector_list']))
            {
                $this->m_aliases['isys_catg_connector_list#isys_catg_connector_list'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['main_obj#' . $l_table]))
            {
                $this->m_aliases['main_obj#' . $l_table] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases[$l_table . '#main_obj']))
            {
                $this->m_aliases[$l_table . '#main_obj'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_obj#isys_catg_connector_list']))
            {
                $this->m_aliases['isys_obj#isys_catg_connector_list'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases['isys_catg_connector_list#isys_obj']))
            {
                $this->m_aliases['isys_catg_connector_list#isys_obj'] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
        } // if

        if (!isset($this->m_aliases[$l_table . '#isys_obj']))
        {
            $this->m_aliases[$l_table . '#isys_obj'] = $this->m_alias_cnt;
            $this->m_alias_cnt++;
        } // if

        if (!isset($this->m_aliases['isys_obj#' . $l_table]))
        {
            $this->m_aliases['isys_obj#' . $l_table] = $this->m_alias_cnt;
            $this->m_alias_cnt++;
        } // if

        if(!in_array($p_props['id'], $this->m_property_order)) $this->m_property_order[] = $p_props['id'];
    } // function

    /**
     * Method for creating aliase for the referenced fields
     *
     * @param       $p_lvl
     * @param array $p_props
     * @param       $p_assigned_property
     */
    protected function create_alias_lvls_select(array $p_props, $p_assigned_property)
    {
        if ($p_props['type'] == C__PROPERTY_TYPE__DYNAMIC)
        {
            $l_table = $p_props['table'];
        }
        else
        {
            $l_table = current(explode('__', $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
        }

        if(!is_array($this->m_property_order[$p_assigned_property])) $this->m_property_order[$p_assigned_property] = array();

        if(isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
        {
            if(count($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
            {
                $l_last_table = '';
                /**
                 * @var $l_selectJoin \idoit\Module\Report\SqlQuery\Structure\SelectJoin
                 */
                foreach($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN] AS $l_selectJoin)
                {
                    $l_join_table = $l_selectJoin->getTable();
                    if (!isset($this->m_aliases_lvls[$l_join_table . '#isys_obj']))
                    {
                        $this->m_aliases_lvls[$l_join_table . '#isys_obj#' . $p_assigned_property] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    }
                    if ($l_last_table != '' && !isset($this->m_aliases[$l_join_table . '#' . $l_last_table . '#' . $p_assigned_property]))
                    {
                        $this->m_aliases_lvls[$l_join_table . '#' . $l_last_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if

                    $l_last_table = $l_join_table;
                }
            }
        }

        if ($l_table == 'isys_catg_custom_fields_list')
        {
            if (!isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_catg_custom_fields_list#isys_obj#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_catg_custom_fields_list#isys_obj#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            }

            if (isset($p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE]) &&
                ($p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE] == 'f_popup' || $p_props['data'][C__PROPERTY__UI][C__PROPERTY__UI__TYPE] == 'popup'))
            {
                if ($p_props['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__OBJECT_BROWSER)
                {
                    if (!isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_obj#isys_catg_custom_fields_list#' . $p_assigned_property]))
                    {
                        $this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_obj#isys_catg_custom_fields_list#' . $p_assigned_property] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if
                }
                else
                {
                    if (!isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_dialog_plus_custom#isys_catg_custom_fields_list#' . $p_assigned_property]))
                    {
                        $this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS] . '#isys_dialog_plus_custom#isys_catg_custom_fields_list#' . $p_assigned_property] = $this->m_alias_cnt;
                        $this->m_alias_cnt++;
                    } // if
                } // if
            } // if
            $this->m_property_order[$p_assigned_property][] = $p_props['id'];
            return;
        }
        elseif (isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT]))
        {
            /**
             * @var $l_selectSubSelect idoit\Module\Report\SqlQuery\Structure\SelectSubSelect
             */
            $l_selectSubSelect = clone $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__SELECT];
            $l_join_table = $l_selectSubSelect->getSelectTable();

            if($l_join_table)
            {
                $l_alias_addition = '#' . $p_props['const'] . '-' . $p_props['key'];
                if (!isset($this->m_aliases_lvls[$l_join_table . '#isys_obj#' . $p_assigned_property . $l_alias_addition]))
                {
                    $this->m_aliases_lvls[$l_join_table . '#isys_obj#' . $p_assigned_property . $l_alias_addition] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                }
            }
        }
//        elseif (isset($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
//        {
//            /**
//             * @var $l_joinQuery idoit\Module\Report\SqlQuery\Structure\JoinSubSelect
//             */
//            $l_joinQuery = $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
//
//            $l_join_tables = $l_joinQuery->getJoinedTables();
//            $l_join_query = $l_joinQuery;
//            foreach($l_join_tables AS $l_jtable)
//            {
//                if($l_join_query)
//                {
//                    $l_alias_addition = '#' . $p_props['const'] . '-' . $p_props['key'];
//                }
//                else
//                {
//                    $l_alias_addition = '';
//                }
//
//                if (!isset($this->m_aliases_lvls[$l_jtable . '#isys_obj#' . $p_assigned_property . $l_alias_addition]))
//                {
//                    $this->m_aliases_lvls[$l_jtable . '#isys_obj#' . $p_assigned_property .  $l_alias_addition] = $this->m_alias_cnt;
//                    if($l_alias_addition === '')
//                    {
//                        $this->m_alias_cnt++;
//                    }
//                } // if
//
//                if (!isset($this->m_aliases_lvls['isys_obj#' . $l_jtable . '#' . $p_assigned_property . $l_alias_addition]))
//                {
//                    $this->m_aliases_lvls['isys_obj#' . $l_jtable . '#' . $p_assigned_property . $l_alias_addition] = $this->m_alias_cnt;
//                    if($l_alias_addition === '')
//                    {
//                        $this->m_alias_cnt++;
//                    } // if
//                } // if
//                foreach ($l_join_tables AS $l_jtable2)
//                {
//                    if ($l_jtable !== $l_jtable2)
//                    {
//                        if (!isset($this->m_aliases_lvls[$l_jtable . '#' . $l_jtable2 . '#' . $p_assigned_property . $l_alias_addition]))
//                        {
//                            $this->m_aliases_lvls[$l_jtable . '#' . $l_jtable2 . '#' . $p_assigned_property . $l_alias_addition] = $this->m_alias_cnt;
//                            //$this->m_reference[$l_jtable.'#'.$l_jtable2] = 'j' . $this->m_alias_cnt . '.' . $l_join_reference;
//                            if($l_alias_addition === '')
//                            {
//                                $this->m_alias_cnt++;
//                            } // if
//                        } // if
//
//                        if (!isset($this->m_aliases_lvls[$l_jtable2 . '#' . $l_jtable . '#' . $p_assigned_property . $l_alias_addition]))
//                        {
//                            $this->m_aliases_lvls[$l_jtable2 . '#' . $l_jtable . '#' . $p_assigned_property . $l_alias_addition] = $this->m_alias_cnt;
//                            //$this->m_reference[$l_jtable2.'#'.$l_jtable] = 'j' . $this->m_alias_cnt . '.' . $l_join_reference;
//                            if($l_alias_addition === '')
//                            {
//                                $this->m_alias_cnt++;
//                            } // if
//                        } // if
//                    } // if
//                } // foreach
//            } // foreach
//            if($l_join_query)
//            {
//                $this->m_alias_cnt++;
//                $this->m_property_order[$p_assigned_property]['joins'][] = $p_props['id'];
//
//            }
//            else
//            {
//                $this->m_property_order[$p_assigned_property][] = $p_props['id'];
//            }
//            //array_unshift($this->m_property_order, $p_props['id']);
//
//            return;
//        }

        if ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] !== null && strpos(
                ' ' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                'isys_'
            ) && !isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table . '#' . $p_assigned_property])
        )
        {
            if (!isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $l_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls[$l_table . '#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$l_table . '#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if

            if ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] == 'isys_contact')
            {
                if (!isset($this->m_aliases_lvls['isys_contact_2_isys_obj#' . $l_table . '#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls['isys_contact_2_isys_obj#' . $l_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
                if (!isset($this->m_aliases_lvls[$l_table . '#isys_contact_2_isys_obj' . '#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls[$l_table . '#isys_contact_2_isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
                if (!isset($this->m_aliases_lvls['isys_contact_2_isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls['isys_contact_2_isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
                if (!isset($this->m_aliases_lvls['isys_contact_2_isys_obj#isys_obj#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls['isys_contact_2_isys_obj#isys_obj#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
            } // if

            if (((strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                        $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                        'cats'
                    ) !== false) && strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], '_list') !== false)
            )
            {
                if (!isset($this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#isys_obj' . '#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls[$p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
                if (!isset($this->m_aliases_lvls['isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls['isys_obj#' . $p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] . '#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                } // if
            }
        }
        elseif ($l_table == 'isys_logbook')
        {
            if (!isset($this->m_aliases_lvls['isys_catg_logb_list#isys_obj#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_catg_logb_list#isys_obj#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
                $this->m_aliases_lvls['isys_logbook#isys_catg_logb_list#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            $this->m_property_order[$p_assigned_property][] = $p_props['id'];
            return;
        }
        elseif ($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] == $l_table . '__isys_obj__id')
        {
            if ($l_table == 'isys_cats_net_ip_addresses_list')
            {
                if (!isset($this->m_aliases_lvls['isys_catg_ip_list#isys_obj#' . $p_assigned_property]))
                {
                    $this->m_aliases_lvls['isys_catg_ip_list#isys_obj#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_aliases_lvls['isys_cats_net_ip_addresses_list#isys_catg_ip_list#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_aliases_lvls['isys_obj#isys_cats_net_ip_addresses_list#' . $p_assigned_property] = $this->m_alias_cnt;
                    $this->m_alias_cnt++;
                    $this->m_property_order[$p_assigned_property][] = $p_props['id'];
                    return;
                } // if
            } // if
        }
        elseif (strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_cable_connection__id') !== false)
        {
            if (!isset($this->m_aliases_lvls[$l_table . '#isys_cable_connection' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$l_table . '#isys_cable_connection' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_cable_connection#' . $l_table . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_cable_connection#' . $l_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_obj#isys_cable_connection' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_obj#isys_cable_connection' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_cable_connection#isys_obj' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_cable_connection#isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
        }
        elseif (strpos($p_props['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_catg_connector_list__id') !== false)
        {
            if (!isset($this->m_aliases_lvls[$l_table . '#isys_catg_connector_list#isys_obj' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$l_table . '#isys_catg_connector_list#isys_catg_connector_list' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
                $this->m_aliases_lvls[$l_table . '#isys_catg_connector_list#isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls[$l_table . '#isys_catg_connector_list' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$l_table . '#isys_catg_connector_list' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_catg_connector_list#' . $l_table . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_catg_connector_list#' . $l_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_catg_connector_list#isys_catg_connector_list' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_catg_connector_list#isys_catg_connector_list' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_obj#isys_catg_connector_list' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_obj#isys_catg_connector_list' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
            if (!isset($this->m_aliases_lvls['isys_catg_connector_list#isys_obj' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_catg_connector_list#isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            } // if
        } // if

        if (!isset($this->m_aliases_lvls[$l_table . '#isys_obj' . '#' . $p_assigned_property]))
        {
            if (!isset($this->m_aliases_lvls[$l_table . '#isys_obj' . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls[$l_table . '#isys_obj' . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            }
            if (!isset($this->m_aliases_lvls['isys_obj' . '#' . $l_table . '#' . $p_assigned_property]))
            {
                $this->m_aliases_lvls['isys_obj' . '#' . $l_table . '#' . $p_assigned_property] = $this->m_alias_cnt;
                $this->m_alias_cnt++;
            }
        } // if
        if(!in_array($p_props['id'], $this->m_property_order[$p_assigned_property])) $this->m_property_order[$p_assigned_property][] = $p_props['id'];
    } // function

    /**
     * Method for retrieving an previously created alias.
     *
     * @param   string $p_table
     * @param   string $p_ref
     *
     * @return  integer
     * @author    Leonard Fischer <lfischer@i-doit.org>
     * @author    Van Quyen Hoang <qhoang@i-doit.org>
     */
    protected function retrieve_alias_lvls($p_table, $p_ref, $p_assigned_property)
    {
        if ($p_ref === null)
        {
            if (isset($this->m_aliases_lvls[$p_table . '#isys_obj' . '#' . $p_assigned_property]))
            {
                return $this->m_aliases_lvls[$p_table . '#isys_obj' . '#' . $p_assigned_property];
            }
            else
            {
                return $this->m_aliases_lvls[$p_table . '#isys_obj'];
            }

        }
        else
        {
            if (isset($this->m_aliases_lvls[$p_ref . '#' . $p_table . '#' . $p_assigned_property]))
            {
                return $this->m_aliases_lvls[$p_ref . '#' . $p_table . '#' . $p_assigned_property];
            }
            else
            {
                return $this->m_aliases_lvls[$p_ref . '#' . $p_table];
            }
        } // if
    } // function

    /**
     * This method prepares all the data, which are needed for selects, joins and conditions.
     *
     * @param   array $p_property_ids
     *
     * @return  isys_cmdb_dao_category_property_ng
     */
    protected function prepare_necessary_tasks(array $p_property_ids)
    {
        if (count($p_property_ids) > 0)
        {
            // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
            $l_res = $this->retrieve_properties($p_property_ids, null, null, null, "", true);

            // First we get all the needed data from the isys_property_2_cat table.
            while ($l_row = $l_res->get_row())
            {
                $l_row['prop_count'] = 0;
                $l_cat_dao        = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                $l_cat_properties = $l_cat_dao->get_properties();
                $l_properties     = array_merge($l_cat_properties, $l_cat_dao->get_dynamic_properties());
                $l_row['data']    = $l_properties[$l_row['key']];

                $l_table_arr = explode('_', $l_row['table']);

                if (array_pop($l_table_arr) !== 'list')
                {
                    $l_row['table'] = $l_row['table'] . '_list';
                } // if

                if($l_row['table'] == 'isys_catg_virtual_list')
                {
                    $l_row['table'] = ($l_cat_dao->get_table()) ?: $l_cat_dao->get_source_table();
                } // if

                foreach($l_cat_properties AS $l_prop)
                {
                    if($l_prop[C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__REPORT])
                    {
                        $l_row['prop_count']++;
                    } // if
                } // foreach

                // We save every row, because we will need them quite often in the upcoming code.
                $this->m_property_rows[$l_row['id']] = $l_row;

                // Also we create table aliases for each possible join.
                $this->create_alias($l_row);
            } // while
            // Ordered properties
            /*if(isset($this->m_property_order['joins']))
            {
                $l_joins = $this->m_property_order['joins'];
                unset($this->m_property_order['joins']);
                $this->m_property_order = array_merge($l_joins, $this->m_property_order);
            }*/
        } // if

        // Also we save some information, so that the logic will not try to join the "isys_obj" or "isys_cmdb_status" tables.
        $this->m_aliases['isys_obj#isys_obj']         = $this->m_alias_cnt++;

        $this->m_prepared_data_for_query_construction = true;

        return $this;
    } // function

    /**
     * Method for retrieving the properties of every category dao.
     *
     * @return  array
     * @author  Dennis Stücken <dstuecken@i-doit.de>
     */
    protected function properties()
    {
        return [];
    } // function

    /**
     * Method for retrieving an previously created alias.
     *
     * @param   string $p_table
     * @param   string $p_ref
     *
     * @return  integer
     * @author  Leonard Fischer <lfischer@i-doit.org>
     * @todo    Change the alias namespaces to "a_table.a_column-b_table.b_column => counter"
     * @todo    Also change the parameters to "$p_from" and "$p_to".
     */
    protected function retrieve_alias($p_table, $p_ref = null)
    {
        if ($p_ref === null)
        {
            return $this->m_aliases[$p_table . '#isys_obj'];
        }
        else
        {
            return $this->m_aliases[$p_ref . '#' . $p_table];
        } // if
    } // function

    /**
     * Method for quick and easy ui-type decision.
     *
     * @param   array $p_property
     *
     * @return  string
     * @author  Leonard Fischer <lfischer@i-doit.org>
     */
    protected function retrieve_ui_type(array $p_property)
    {
        $l_prop_ui = $p_property[C__PROPERTY__UI];
        if ($l_prop_ui[C__PROPERTY__UI__TYPE] == C__PROPERTY__UI__TYPE__DIALOG)
        {
            return C__PROPERTY__UI__TYPE__DIALOG;
        }
        else if ($l_prop_ui[C__PROPERTY__UI__TYPE] == C__PROPERTY__UI__TYPE__POPUP && $l_prop_ui[C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'dialog_plus')
        {
            return C__PROPERTY__UI__TYPE__DIALOG;
        }
        else if ($l_prop_ui[C__PROPERTY__UI__TYPE] == C__PROPERTY__UI__TYPE__POPUP && $l_prop_ui[C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'object_browser_ng')
        {
            // We assume "popup" = "object browser".
            return C__PROPERTY__UI__TYPE__POPUP;
        } // if

        return null;
    } // function

    private function handle_sub_conditions($p_assigned_key, $p_sub_conditions, $p_referenced_field)
    {
        // Join Object Table of $p_assigned_key
        $l_inner_condition = '';
        foreach ($p_sub_conditions[$p_assigned_key] AS $l_condition)
        {
            $l_property          = $l_condition['property'];
            $l_unit_id           = null;
            $l_unit_property     = null;
            $l_unit_field        = null;
            $l_special_selection = false;
            $l_special_field     = null;
            $l_ref_table         = null;
            $l_ref_table_arr     = null;
            $l_loc_condition_lft = null;
            $l_loc_condition_rgt = null;
            $l_assigned_key      = null;
            list($l_category, $l_prop_key) = explode('-', $l_property);

            $l_unit = $l_condition['unit'];
            if (strpos($l_unit, '-') !== false)
            {
                list($l_unit_id, $l_unit_property) = explode('-', $l_unit);
            }

            if (strpos($p_assigned_key, '--') > 0)
            {
                $l_assigned_key_arr = explode('--', $p_assigned_key);
                if (!is_numeric($l_assigned_key_arr[0]))
                {
                    $l_assigned_key = '';
                    for ($i = 0;$i < count($l_assigned_key_arr);$i++)
                    {
                        list($l_category_assigned, $l_prop_key_assigned) = explode('-', $l_assigned_key_arr[$i]);

                        // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
                        $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                                $l_category_assigned
                            ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key_assigned);
                        $l_id                 = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                            ->get_row_value('id');
                        $l_assigned_key .= $l_id . '--';
                    }
                    $l_assigned_key = rtrim($l_assigned_key, '--');
                }
            }
            elseif (!is_numeric($p_assigned_key))
            {
                list($l_category_assigned, $l_prop_key_assigned) = explode('-', $p_assigned_key);

                // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
                $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                        $l_category_assigned
                    ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key_assigned);
                $l_assigned_key       = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                    ->get_row_value('id');
            }
            else
            {
                $l_assigned_key = $p_assigned_key;
            }

            if (!isset($this->m_property_rows_lvls[$l_property]))
            {
                // This query will be used to receive all the necessary entries from the isys_property_2_cat table.
                $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                        $l_category
                    ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_prop_key);
                $l_row                = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                    ->get_row();
                $l_cat_dao            = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                $l_properties         = $l_cat_dao->get_properties();
                $l_row['data']        = $l_properties[$l_row['key']];

                $this->m_property_rows_lvls[$l_property] = $l_row['id'];
                if (!isset($this->m_property_rows_lvls[$l_row['id']]))
                {
                    // We save every row, because we will need them quite often in the upcoming code.
                    $this->m_property_rows_lvls[$l_row['id']] = $l_row;
                } // if
                // Also we create table aliases for each possible join.
                $this->create_alias_lvls_select($l_row, $l_assigned_key);
                $l_select = $l_row['id'];
            }
            else
            {
                if (is_array($this->m_property_rows_lvls[$l_property]))
                {
                    $l_select = $this->m_property_rows_lvls[$l_property]['id'];
                    $l_row    = $this->m_property_rows_lvls[$l_property];
                }
                else
                {
                    $l_select = $this->m_property_rows_lvls[$l_property];
                    $l_row    = $this->m_property_rows_lvls[$this->m_property_rows_lvls[$l_property]];
                }
                $this->create_alias_lvls_select($l_row, $l_assigned_key);
            }// if

            if ($l_unit_id !== null)
            {
                $l_ignore_unit_field = false;
                if (isset($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT]))
                {
                    if (isset($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2]))
                    {
                        $l_ignore_unit_field = ($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0] !== null);
                    } // if
                } // if

                if (!isset($this->m_property_rows_lvls[$l_category . '-' . $l_unit_property]))
                {
                    $l_property_condition = ' AND isys_property_2_cat__cat_const = ' . $this->convert_sql_text(
                            $l_category
                        ) . ' AND isys_property_2_cat__prop_key = ' . $this->convert_sql_text($l_unit_property);
                    $l_row                = $this->retrieve_properties(null, null, null, null, $l_property_condition)
                        ->get_row();

                    $l_cat_dao     = $this->get_dao_instance($l_row['class'], ($l_row['catg_custom'] ?: null));
                    $l_properties  = $l_cat_dao->get_properties();
                    $l_row['data'] = $l_properties[$l_row['key']];

                    // We save every row, because we will need them quite often in the upcoming code.
                    $this->m_property_rows_lvls[$l_category . '-' . $l_unit_property] = $l_row;
                } // if

                if (!$l_ignore_unit_field)
                {
                    $l_unit_field = $this->m_property_rows_lvls[$l_category . '-' . $l_unit_property][C__PROPERTY__DATA][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                } // if
            } // if

            $l_dummy_selects = [];
            $l_lvls          = [
                $l_assigned_key => [$l_select]
            ];

            $l_joins = $this->create_property_query_join_lvls($l_lvls, $l_dummy_selects, $l_dummy_selects, true, $p_referenced_field);
            if (count($l_joins) > 0)
            {
                $l_join_key = key($l_joins);
                if (isset($this->m_sub_joins[$l_join_key]))
                {
                    $this->m_sub_joins[$l_join_key] = array_unique(array_merge($this->m_sub_joins[$l_join_key], $l_joins[$l_join_key]));
                }
                else
                {
                    $this->m_sub_joins += $l_joins;
                } // if
            } // if

            $l_table = current(explode('__', $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]));
            $l_selectTitleField = null;

            if ($l_table == 'isys_catg_custom_fields_list')
            {
                $l_field_alias = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD_ALIAS];
                $l_field_type  = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE];
                $l_alias       = 'j' . $this->retrieve_alias_lvls($l_field_alias . '#isys_catg_custom_fields_list', null, $l_assigned_key);

                switch ($l_field_type)
                {
                    case C__PROPERTY__INFO__TYPE__OBJECT_BROWSER:
                    case C__PROPERTY__INFO__TYPE__N2M:
                        $l_alias_sec       = 'j' . $this->retrieve_alias_lvls('isys_catg_custom_fields_list', $l_field_alias . '#isys_obj', $l_assigned_key);
                        $l_condition_field = $l_alias_sec . '.isys_obj__id';
                        break;
                    case C__PROPERTY__INFO__TYPE__DIALOG:
                    case C__PROPERTY__INFO__TYPE__DIALOG_PLUS:
                        $l_alias_sec       = 'j' . $this->retrieve_alias_lvls('isys_catg_custom_fields_list', $l_field_alias . '#isys_dialog_plus_custom', $l_assigned_key);
                        $l_condition_field = $l_alias_sec . '.isys_dialog_plus_custom__id';
                        break;
                    default:
                        $l_condition_field = $l_alias . '.' . $l_table . '__field_content';
                        break;
                } // switch
            }
//            else if (isset($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN]))
//            {
//                /**
//                 * @var $l_joinSubSelect idoit\Module\Report\SqlQuery\Structure\JoinSubSelect
//                 */
//                $l_joinSelect = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__JOIN];
//                $l_tables = $l_joinSelect->getJoinedTables();
//                if(count($l_tables) > 1)
//                {
//                    $l_table = $l_tables[0];
//                    $l_ref_table = $l_tables[1];
//                }
//                else
//                {
//                    $l_table = $l_tables[0];
//                    $l_ref_table = 'isys_obj';
//                } // if
//
//                if(in_array($this->m_property_rows[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE], $this->m_reference_types_condition))
//                {
//                    $l_selectedField = 'reference';
//                    $l_selectTitleField = 'title';
//                }
//                else
//                {
//                    $l_selectedField = 'title';
//                }
//
//                $l_alias = 'j' . $this->m_aliases_lvls[$l_table . '#' . $l_ref_table . '#' . $l_assigned_key . '#' . $this->m_property_rows_lvls[$l_select]['const'] . '-' . $this->m_property_rows_lvls[$l_select]['key']];
//                $l_condition_field = $l_alias . '.' . $l_selectedField;
//
//                if($l_selectTitleField !== null)
//                {
//                    $l_selectTitleField = $l_alias . '.' . $l_selectTitleField;
//                } // if
//            }
            else
            {
                // We have to check for an existing "predefined" alias.
                if (isset($this->m_aliases_lvls[$l_table]))
                {
                    $l_alias = $this->m_aliases_lvls[$l_table];
                }
                else
                {
                    $l_alias = 'j' . $this->retrieve_alias_lvls($l_table, null, $l_assigned_key);
                } // if

                if ($l_table == 'isys_logbook')
                {
                    $l_ref_table = $l_table;
                    $l_table     = 'isys_catg_logb_list';
                    $l_alias     = 'j' . $this->retrieve_alias_lvls($l_table, $l_ref_table, $l_assigned_key);
                } // if

                if (($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] !== null && substr(
                            $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                            0,
                            5
                        ) == 'isys_') || $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'browser_location'
                )
                {
                    if (($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] === null || $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0] === 'isys_obj') && $l_table == 'isys_obj')
                    {
                        $l_ref_table_arr = explode('__', $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]);
                        $l_table         = $l_ref_table_arr[0];
                        $l_ref_table     = 'isys_obj';
                        $l_special_field = 'isys_obj__id';
                    }
                    else
                    {
                        $l_ref_table = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0];
                    } // if

                    $l_alias = $this->retrieve_alias_lvls($l_table, $l_ref_table, $l_assigned_key);

                    if ($l_condition['comparison'] == 'under_location')
                    {
                        $l_alias             = $this->retrieve_alias_lvls('isys_obj', $l_table, $l_assigned_key);
                        $l_loc_condition_lft = 'j' . $l_alias . '.isys_catg_location_list__lft';
                        $l_loc_condition_rgt = 'j' . $l_alias . '.isys_catg_location_list__rgt';
                    } // if

                    // We have to join 'job' on references to isys_connection.
                    if ($l_ref_table == 'isys_connection')
                    {
                        $l_condition_field = "job" . $l_alias . ".isys_obj__id";
                    }
                    else if ($l_ref_table == 'isys_contact')
                    {
                        $l_alias           = $this->retrieve_alias_lvls('isys_obj', 'isys_contact_2_isys_obj', $l_assigned_key);
                        $l_condition_field = "job" . $l_alias . ".isys_obj__id";
                    }
                    elseif ($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['p_strPopupType'] == 'browser_location')
                    {
                        $l_alias           = $this->retrieve_alias_lvls($l_table, 'isys_obj', $l_assigned_key);
                        $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                    }
                    else
                    {
                        if ($l_special_selection)
                        {
                            $l_condition_field = "j" . $l_alias . "." . (($l_special_field !== null) ? $l_special_field : $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]);
                        }
                        elseif ($l_ref_table == 'isys_obj')
                        {
                            $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                        }
                        else
                        {
                            $l_condition_field = "j" . $l_alias . "." . $l_ref_table . "__id";
                        } // if

                        if (isset($this->m_text_fields[$this->m_property_rows_lvls[$l_select]['const']]))
                        {
                            if (isset($this->m_text_fields[$this->m_property_rows_lvls[$l_select]['const']][$this->m_property_rows_lvls[$l_select]['key']]))
                            {
                                $l_condition_field = substr(
                                        $l_condition_field,
                                        0,
                                        (strpos($l_condition_field, '.') + 1)
                                    ) . $this->m_text_fields[$this->m_property_rows_lvls[$l_select]['const']][$this->m_property_rows_lvls[$l_select]['key']];
                            } // if
                        } // if
                    } // if
                }
                elseif (strpos($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_cable_connection__id') !== false)
                {
                    $l_alias           = $this->retrieve_alias_lvls('isys_cable_connection', 'isys_obj', $l_assigned_key);
                    $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                }
                elseif (strpos($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD], 'isys_catg_connector_list__id') !== false)
                {
                    if ($l_table != 'isys_catg_connector_list')
                    {
                        $l_alias           = $this->retrieve_alias_lvls('isys_catg_connector_list#isys_obj', $l_table, $l_assigned_key);
                        $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                    }
                    else
                    {
                        $l_alias           = $this->retrieve_alias_lvls('isys_catg_connector_list', 'isys_obj', $l_assigned_key);
                        $l_condition_field = "j" . $l_alias . ".isys_obj__id";
                    }
                }
                else
                {
                    if ($l_ref_table == '' && !in_array($l_alias, $this->m_already_used_aliase))
                    {
                        $l_alias           = 'j' . $this->retrieve_alias_lvls($l_table, 'isys_obj', $l_assigned_key);
                        $l_condition_field = $l_alias . ".isys_obj__id";
                    }
                    else
                    {
                        $l_condition_field = $l_alias . "." . $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                    }
                } // if
            }

            if ($l_condition['comparison'] == 'subcnd')
            {
                $l_inner_condition .= $this->handle_sub_conditions(
                        $p_assigned_key . '--' . $l_property,
                        $p_sub_conditions,
                        $l_condition_field
                    ) . ' ' . $l_condition['operator'];
            }
            else
            {

                if (isset($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES]) && $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG || $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__DIALOG_PLUS)
                {
                    if ((strpos($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                                $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                'cats'
                            ) !== false) && strpos(
                            $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                            '_list'
                        ) !== false && strpos($l_condition_field, '__id_') === false
                    )
                    {
                        $l_condition_field = str_replace('__id', '__title', $l_condition_field);
                    } // if
                }
                elseif ($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE] == C__PROPERTY__INFO__TYPE__TEXT && strpos(
                        $l_condition_field,
                        '__id'
                    ) > 0
                )
                {
                    $l_condition_field = str_replace('__id', '__title', $l_condition_field);
                } // if

                if (isset($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK]) && $l_unit_id !== null)
                {
                    if ($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][1] == 'convert')
                    {
                        $l_method = $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__FORMAT][C__PROPERTY__FORMAT__CALLBACK][2][0];
                        // convert value
                        $l_condition['value'] = isys_convert::$l_method($l_condition['value'], $l_unit_id);
                    } // if
                } // if

                if ($l_condition['comparison'] == 'LIKE %...%')
                {
                    $l_condition['comparison'] = 'LIKE';
                    $l_condition['value']      = '%' . $l_condition['value'] . '%';
                } // if

                if ($l_condition['comparison'] == 'NOT LIKE %...%')
                {
                    $l_condition['comparison'] = 'NOT LIKE';
                    $l_condition['value']      = '%' . $l_condition['value'] . '%';
                } // if

                $l_condition['value'] = str_replace(
                    [
                        '[',
                        ']',
                        '"'
                    ],
                    '',
                    $l_condition['value']
                );
                if (strpos($l_condition['value'], ',') !== false)
                {
                    if($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['multiselection'])
                    {
                        $l_inner_condition .= ' ( ';
                        $l_comparison_find_in_set = ($l_condition['comparison'] == '!=' ? 'NOT' : '');
                        foreach($l_condition['value'] AS $l_id)
                        {
                            $l_inner_condition .= ' ' . $l_comparison_find_in_set . ' FIND_IN_SET(' . $this->convert_sql_text($l_id) . ', ' . $l_condition_field . ') OR';
                        }
                        $l_inner_condition = rtrim($l_inner_condition, 'OR');
                        $l_inner_condition .= ' ) ';
                    }
                    else
                    {
                        if ($l_condition['comparison'] == '=')
                        {
                            $l_condition['comparison'] = 'IN';
                        }
                        else
                        {
                            $l_condition['comparison'] = 'NOT IN';
                        }
                        $l_inner_condition .= ' ' . $l_condition_field . ' ' . $l_condition['comparison'] . ' (' . $l_condition['value'] . ') ' .
                            (isset($l_condition['operator']) ? $l_condition['operator'] : '');
                    }
                }
                else
                {
                    if ($l_condition['comparison'] == 'under_location')
                    {
                        $l_inner_condition .= ' (' . $l_loc_condition_lft . ' > ' . '(SELECT isys_catg_location_list__lft FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                                $l_condition['value']
                            ) . ') AND ' . $l_loc_condition_rgt . ' < ' . '(SELECT isys_catg_location_list__rgt FROM isys_catg_location_list WHERE isys_catg_location_list__isys_obj__id = ' . $this->convert_sql_id(
                                $l_condition['value']
                            ) . ') ';
                    }
                    else
                    {
                        if($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS]['multiselection'])
                        {
                            $l_comparison_find_in_set = ($l_condition['comparison'] == '!=' ? 'NOT' : '');
                            $l_inner_condition .= ' (' . $l_comparison_find_in_set . ' FIND_IN_SET(' . $this->convert_sql_text($l_condition['value']) . ', ' . $l_condition_field . ') ';
                        }
                        else
                        {
                            if($l_selectTitleField !== null && !is_numeric($l_condition['value']) && $l_condition['value'] !== '' &&
                                (strpos($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], 'catg') !== false || strpos(
                                        $this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0],
                                        'cats'
                                    ) !== false) && strpos($this->m_property_rows_lvls[$l_select]['data'][C__PROPERTY__DATA][C__PROPERTY__DATA__REFERENCES][0], '_list') !== false)
                            {
                                $l_inner_condition .= ' (' . $l_selectTitleField . ' ' . $l_condition['comparison'] . ' ' .
                                    (($l_condition['comparison'] === 'IS NULL' || $l_condition['comparison'] === 'IS NOT NULL') ?
                                        '' : (is_numeric($l_condition['value']) ? $this->convert_sql_int($l_condition['value']) :
                                            $this->convert_sql_text($l_condition['value']))) . ' ';
                            }
                            else
                            {
                                $l_inner_condition .= ' (' . $l_condition_field . ' ' . $l_condition['comparison'] . ' ' .
                                    (($l_condition['comparison'] === 'IS NULL' || $l_condition['comparison'] === 'IS NOT NULL') ? '' : $this->convert_sql_text(
                                        $l_condition['value']
                                    )) . ' ';

                                if (($l_condition['value'] === '' || is_numeric($l_condition['value'])) && $l_condition['comparison'] !== 'IS NULL' &&
                                    $l_condition['comparison'] !== 'IS NOT NULL'
                                )
                                {
                                    if ((int) $l_condition['value'] === 0)
                                    {
                                        if ($l_condition['comparison'] == '=')
                                        {
                                            $l_comparison_addition = ' IS ';
                                            $l_log_operator        = ' OR ';
                                        }
                                        else
                                        {
                                            $l_comparison_addition = ' IS NOT ';
                                            $l_log_operator        = ' AND ';
                                        } // if
                                        $l_inner_condition .= $l_log_operator . $l_condition_field . ' ' . $l_comparison_addition . ' NULL ';
                                    } // if
                                } // if
                            } // if
                        } // if
                    } // if
                    $l_inner_condition .= ') ' . (isset($l_condition['operator']) ? $l_condition['operator'] : '');
                }

                if ($l_unit_field !== null)
                {
                    $l_inner_condition .= ' AND ' . $l_alias . '.' . $l_unit_field . ' = ' . $this->convert_sql_text($l_unit_id) . ' ';
                } // if
            }
        }

        return ' (' . trim($l_inner_condition) . ') ';
    } // function

    /**
     * Retrieves property id by condition
     *
     * @param string $p_condition
     *
     * @return mixed
     * @throws isys_exception_database
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_property_id_by_condition($p_condition = 'TRUE')
    {
        $l_sql = 'SELECT isys_property_2_cat__id FROM isys_property_2_cat
          LEFT JOIN isysgui_catg ON isysgui_catg__id = isys_property_2_cat__isysgui_catg__id
          LEFT JOIN isysgui_cats ON isysgui_cats__id = isys_property_2_cat__isysgui_cats__id
          LEFT JOIN isysgui_catg_custom ON isysgui_catg_custom__id = isys_property_2_cat__isysgui_catg_custom__id
          WHERE ' . $p_condition . ' LIMIT 1;';
        return $this->retrieve($l_sql)->get_row_value('isys_property_2_cat__id');
    } // function
} // class