<?php
use idoit\Module\Cmdb\Model\Matcher\Ci\CiMatcher;
use idoit\Module\Cmdb\Model\Matcher\MatchConfig;
use idoit\Module\Cmdb\Model\Matcher\Identifier\ObjectTitle;
use idoit\Module\Cmdb\Model\Matcher\Identifier\Fqdn;
use idoit\Module\Cmdb\Model\Matcher\Identifier\Hostname;
use idoit\Module\Cmdb\Model\Matcher\Identifier\IpAddress;
use idoit\Module\Cmdb\Model\Matcher\Identifier\Mac;
use idoit\Module\Cmdb\Model\Matcher\Identifier\ModelSerial;
use idoit\Module\Cmdb\Model\Matcher\Ci\MatchKeyword;

/**
 * Class isys_jdisc_dao_matching
 *
 * @author   Van Quyen Hoang <qhoang@i-doit.com>
 */
class isys_jdisc_dao_matching
{
    /**
     * @var isys_jdisc_dao_data
     */
    protected $m_dao;

    /**
     * @var
     */
    private static $m_instance;

    /**
     * @var int
     */
    private $m_server_id = null;

    /**
     * @var CiMatcher[]
     */
    private $m_ciMatcher;

    /**
     * @var isys_cache
     */
    private $m_cache;

    /**
     * Retrieves object id by device id
     *
     * @param $p_device_id
     *
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_object_id_by_device_id($p_device_id)
    {
        if($this->m_cache->exists('deviceid-' . $this->m_server_id . '-' . $p_device_id))
        {
            return $this->m_cache->get('deviceid-' . $this->m_server_id . '-' . $p_device_id);
        }
        else
        {
            $l_obj_id =  $this->find_object_id($p_device_id, 'deviceid-' . $this->m_server_id, $this->build_device_keywords($p_device_id));
            $this->m_cache->set('deviceid-' . $this->m_server_id . '-' . $p_device_id, $l_obj_id);
            return $l_obj_id;
        } // if
    } // function

    /**
     * Retrieves object id by module id
     *
     * @param $p_module_id
     *
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_object_id_by_module_id($p_module_id)
    {
        if($this->m_cache->exists('moduleid-' . $this->m_server_id . '-' . $p_module_id))
        {
            return $this->m_cache->get('moduleid-' . $this->m_server_id . '-' . $p_module_id);
        }
        else
        {
            $l_obj_id = $this->find_object_id('module-' . $p_module_id, 'moduleid-' . $this->m_server_id, $this->build_module_keywords($p_module_id));
            $this->m_cache->set('moduleid-' . $this->m_server_id . '-' . $p_module_id, $l_obj_id);
            return $l_obj_id;
        } // if
    } // function

    /**
     * Search object by device id or object matcher
     *
     * @param int  $p_device_id
     * @param \idoit\Module\Cmdb\Model\Matcher\Ci\MatchKeyword[] $p_keywords
     * @param int  $p_bits
     * @param int  $p_min_match
     *
     * @return bool|int
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    private function find_object_id($p_id = null, $p_identifier_key = null, array $p_keywords = [])
    {
        $l_objectID = 0;
        // First get object id by identifier
        if($p_id != '' && $p_identifier_key !== null)
        {
            // First check in cache
            $l_objectID = isys_cmdb_dao_category_g_identifier::get_object_id_by_identifer($p_id);

            if($l_objectID)
            {
                return $l_objectID;
            } // if

            $l_objectID = isys_cmdb_dao_category_g_identifier::instance(isys_application::instance()->database)
                ->get_object_id_by_key_value(
                    isys_cmdb_dao_category_g_identifier::get_identifier_type(),
                    isys_cmdb_dao_category_g_identifier::get_identifier_key(),
                    $p_id
                );
        } // if

        // Second get object id by keywords
        if(!$l_objectID && count($p_keywords))
        {
            $l_match = $this->m_ciMatcher->match($p_keywords);

            if($l_match->getMatchCount() > 1)
            {
                $l_found_matches = $l_match->getMatchResult();
                $l_objectID = $l_match->getId();

                if(strpos($p_identifier_key, 'moduleid-'))
                {
                    $this->m_dao->get_log()
                        ->info('Module id "' . str_replace('module-', '', $p_id) . '" found. Using object id ' . $l_objectID . '. Found ' . $l_match->getMatchCount() . ' or more objects.');
                }
                else
                {
                    $this->m_dao->get_log()
                        ->info('Device id "' . $p_id . '" found. Using object id ' . $l_objectID . '. Found ' . $l_match->getMatchCount() . ' or more objects.');
                } // if

                $this->m_dao->get_log()
                    ->debug('Please check the following Objects:');
                // Output which objects were found
                foreach($l_found_matches AS $l_key => $l_found_match)
                {
                    $this->m_dao->get_log()->debug('- Object ' . $l_found_match->getTitle() . ' (ID: ' . $l_found_match->getID() . ')');
                } // foreach
            }
            else
            {
                $l_objectID = $l_match->getId();
            } // if

            if($l_objectID > 0)
            {
                isys_cmdb_dao_category_g_identifier::set_missing_identifiers($l_objectID, $p_id);
            } // if
        } // if

        if($l_objectID)
        {
            isys_cmdb_dao_category_g_identifier::set_object_id_by_identifier($l_objectID, $p_id);
        } // if

        return ($l_objectID > 0) ? $l_objectID : false;
    } // function

    /**
     * Build Matchkeywords for devices
     *
     * @param int $p_device_id
     *
     * @return MatchKeyword[]
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    private function build_device_keywords($p_device_id)
    {
        $l_device_info = $this->m_dao->get_device_info($p_device_id);
        $l_keywords   = [];
        $l_keywords[] = new MatchKeyword(ObjectTitle::KEY, $l_device_info['name']);
        $l_keywords[] = new MatchKeyword(ModelSerial::KEY, $l_device_info['serialnumber']);
        $l_keywords[] = new MatchKeyword(Hostname::KEY, $l_device_info['hostname']);
        $l_keywords[] = new MatchKeyword(IpAddress::KEY, $l_device_info['address']);
        $l_keywords[] = new MatchKeyword(Fqdn::KEY, $l_device_info['fqdn']);

        $l_unique_mac_addresses = $this->m_dao->get_mac_addresses($p_device_id, true);
        if (count($l_unique_mac_addresses))
        {
            $l_keywords[] = new MatchKeyword(Mac::KEY, current($l_unique_mac_addresses));
        } // if
        return $l_keywords;
    } // function

    /**
     * Build Matchkeywords for modules
     *
     * @param int $p_module_id
     *
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    private function build_module_keywords($p_module_id)
    {
        $l_module_info = $this->m_dao->get_modules_info(null, $p_module_id);
        $l_keywords = [];
        if(count($l_module_info))
        {
            $l_module = current($l_module_info);

            $l_serial           = trim($l_module['serial']);
            $l_raw_title        = trim($l_module['title']);
            $l_title            = $l_raw_title . (($l_serial != '') ? ' - ' . $l_serial : '');

            $l_keywords[] = new MatchKeyword(ModelSerial::KEY, $l_serial);
            $l_keywords[] = new MatchKeyword(ObjectTitle::KEY, $l_title);
        } // if
        return $l_keywords;
    } // function

    /**
     * Factory method build singleton
     *
     * @param int                 $p_config_id isys_obj_match__id
     * @param int                 $p_server_id
     * @param isys_jdisc_dao_data $p_jdisc_dao
     *
     * @return isys_jdisc_dao_matching
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public static function initialize($p_config_id, $p_server_id, isys_jdisc_dao_data $p_jdisc_dao)
    {
        if (!self::$m_instance)
        {
            self::$m_instance = new self($p_server_id, new CiMatcher(MatchConfig::factory($p_config_id, isys_application::instance()->container)), $p_jdisc_dao);
        } // if

        return self::$m_instance;
    } // function

    /**
     * @return isys_jdisc_dao_matching
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    public static function instance()
    {
        return self::$m_instance;
    } // function

    /**
     * isys_jdisc_dao_matching constructor.
     *
     * @param int                 $p_server_id
     * @param CiMatcher           $p_matcher
     * @param isys_jdisc_dao_data $p_jdisc_dao
     */
    public function __construct($p_server_id, CiMatcher $p_matcher, isys_jdisc_dao_data $p_jdisc_dao)
    {
        $this->m_ciMatcher = $p_matcher;
        $this->m_server_id = $p_server_id;
        $this->m_dao = $p_jdisc_dao;
        $this->m_cache = isys_cache::keyvalue();
        $this->m_cache->ns('jdisc');
    } // function

    /**
     * Destruct
     */
    public function __destruct()
    {
        // Flush cache
        $this->m_cache->ns_invalidate('jdisc');
    } // function
}