<?php
/**
 * i-doit - Updates
 * This migration will translate the old CSV identificators to the new ones!
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Leonard Fischer <lfischer@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_comp_database_system;

$g_migration_identifier = "migration_csv_profiles";

if ($this->is_migration_done($g_migration_identifier))
{
    $g_migration_log[] = "<span class=\"bold grey indent\">CSV Profiles have already been migrated.</span>";
}
else
{
    try
    {
        $l_dao = new isys_cmdb_dao($g_comp_database);
        $l_mapping = [
            'LC__CMDB__CATG__GLOBAL_TITLE' => 'objectTitle',
            'LC__CMDB__CATG__GLOBAL_SYSID' => 'sysid',
            'LC_UNIVERSAL__OBJECT_TYPE' => 'objectTypeConst',
            'LC__CMDB__CATG__ACCOUNTING_INVENTORY_NO' => 'inventoryNo',
            'LC__CMDB__CATS__ROOM_NUMBER' => 'roomNumber',
            'LC__CMDB__CATG__INTERFACE_P_SERIAL' => 'modelSerial',
            'LC__CATG__IP_ADDRESS' => 'ipAddress',
            'LC__CATP__IP__HOSTNAME' => 'hostname',
            'LC__LOGIN__USERNAME' => 'username',
            'LC__CMDB__CATG__LOCATION_PARENT' => 'location',
        ];

        $l_profiles = $l_dao->retrieve('SELECT isys_csv_profile__id AS id, isys_csv_profile__title AS title, isys_csv_profile__data AS data
            FROM isys_csv_profile
            WHERE isys_csv_profile__data LIKE "%identificationKeys%";');

        if (count($l_profiles))
        {
            while ($l_profile = $l_profiles->get_row())
            {
                $l_data = isys_format_json::decode($l_profile['data']);

                if (isset($l_data['identificationKeys']) && is_array($l_data['identificationKeys']))
                {
                    foreach ($l_data['identificationKeys'] as &$l_value)
                    {
                        if (isset($l_mapping[$l_value['localIdent']]))
                        {
                            $l_value['localIdent'] = $l_mapping[$l_value['localIdent']];
                        }
                        else
                        {
                            $g_migration_log[] = "<span class=\"indent grey\">Could not migrate '" . $l_value['localIdent'] . "'.</span>";
                        } // if
                    } // foreach

                    $l_data['identificationCounter'] = 1;

                    $l_dao->update('UPDATE isys_csv_profile
                        SET isys_csv_profile__data = ' . $l_dao->convert_sql_text(isys_format_json::encode($l_data)) . '
                        WHERE isys_csv_profile__id = ' . $l_dao->convert_sql_id($l_profile['id']) . ';');
                    $l_dao->apply_update();

                    $g_migration_log[] = "<span class=\"indent bold\">Migrated CSV profile '" . $l_profile['title'] . "' (#" . $l_profile['id'] . ").</span>";
                } // if
            } // while
        }
        else
        {
            $g_migration_log[] = "<span class=\"indent bold\">No CSV profiles to migrate.</span>";
        } // if

        // Set flag in db.
        $this->migration_done($g_migration_identifier);

        $g_migration_log[] = "<span class=\"bold\">Migration successful.</span>";
    }
    catch (Exception $e)
    {
        $g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
    } // try
} // if